/**
 * 汎用コンポーネント テキスト
 */


(function() {
	var rule = {
		_name : 'BASICTEXT01',
		_doc  : 'テキスト',

		start : {
			'#doc' : function step1StartDoc() {
				this.paraTree = new gcparts.ElementTree();
				this.addStep1 = false;
				this.lineBreak = 0;
				this.imageJSONArray = [];

				// 文字色取り込み 1: 取り込む（デフォルト）, 0: 取り込まない
				this.fontDesign = this.getParameter('fontDesign');
				if (this.fontDesign) {
					this.fontDesign = this.fontDesign - 0;
				} else {
					this.fontDesign = 1;
				}

				// 画像代替テキスト取り込み 0:取り込まない（デフォルト） 1:取り込む
				this.useImageAlt = this.getParameter('useImageAlt');
				if (this.useImageAlt) {
					this.useImageAlt = this.useImageAlt - 0;
				} else {
					this.useImageAlt = 0;
				}

				// 画像の使用 0:アイコン化（デフォルト）1:削除
				this.imageType = this.getParameter('imageType');
				if (this.imageType) {
					this.imageType = this.imageType - 0;
				} else {
					this.imageType = 0;
				}

				// 要素の追加先を決定するファンクション
				this.getTree = function() {
					if (this.anchorTree) {
						return this.anchorTree;
					} else {
						return this.paraTree;
					}
				}
				// カスタムツリーを段落orリストとしてツリーに追加する
				this.appendTree = function() {
					if (this.paraTree.countAddedItem('step1.paragraph') > 0) {
						this.addTree(this.paraTree);
					} else if (this.paraTree.countAddedItem('step1.list') > 0) {
						this.addTree(this.paraTree);
					} else {
						if (this.paraTree.countAddedItem('step1.listItem') > 0) {
							this.addElement(this.getItem('step1.list'));
							this.addTree(this.paraTree);
							this.endElement();
						} else {
							this.addElement(this.getItem('step1.paragraph'));
							this.addTree(this.paraTree);
							this.endElement();
						}
					}
				}

				// 置換キーワード定義（JSON形式）
				var _keywords = this.getItem('start.keywords');
				if (_keywords) {
					try {
						var _value = _keywords.getElement().textContent || '';
						if (_value.match(/^[ \r\n\t\f]*$/)) {
							if (gclog.isDebugEnabled()) {
								gclog.debug('[start.keywords] is empty', 2);
							}
						} else {
							if (!_value.match(/[\{\}]/)) {
								_value = '{' + _value + '}';
							}
							if (gclog.isDebugEnabled()) {
								gclog.debug('[start.keywords]' + _value, 2);
							}
							// 本来ならJSON.parseするが、カンマで終わるフォーマットを不正扱いするので、evalを使用する
							this.keywords = eval('(' + _value + ')');
						}
					} catch(err) {
						gclog.error('cannot read gcl format[start.keywords]' + err, 2);
					}
				}
				this.convertKeywords = function(text) {
					if (!this.keywords) {
						return text;
					}
					var result = text;
					for (var k in this.keywords) {
						result = result.split(k).join(this.keywords[k]);
					}
					return result;
				}

				// 画像無視リスト（完全に無視し、出力からも消える。spacer.gifなど目印さえも要らない場合など）
				this.ignoreImages = [];
				var _ig = this.getItem('start.ignoreImages');
				if (_ig) {
					var _ignoreImages = _ig.getAttribute('value').replace(/\s*/g, '').split(',');
					for (var i = 0; i < _ignoreImages.length; i++) {
						var _image = _ignoreImages[i];

						// *以外の正規表現パターン文字をエスケープ
						// ^$\.+?()[]{}|/
						_image = _image.replace(/([\^\$\\\.\+\?\(\)\[\]\{\}\|\/])/g, "\\$1");
						// *はワイルドカード扱いにする
						_image = _image.replace(/\*/g, '(.*)');
						_image = '^' + _image + '$';
						gclog.debug('ignore image regexp = ' + _image, 2);
						this.ignoreImages[i] = new RegExp(_image);
					}
				}
				this.isIgnoreImage = function(src) {
					for (var i = 0; i < this.ignoreImages.length; i++) {
						if (src.match(this.ignoreImages[i])) {
							//gclog.debug('isIgnoreImage is true src = ' + src, 2);
							return true;
						}
					}
					return false;
				}

				// 画像出力リスト（そのまま表示する。絵文字アイコンのような画像をそのまま出したい場合など）
				var _di = this.getItem('start.displayImages');
				this.displayImages = [];
				if (_di) {
					var _displayImages = _di.getAttribute('value').replace(/\s*/g, '').split(',');
					for (var i = 0; i < _displayImages.length; i++) {
						var _image = _displayImages[i];
						// *以外の正規表現パターン文字をエスケープ
						// ^$\.+?()[]{}|/
						_image = _image.replace(/([\^\$\\\.\+\?\(\)\[\]\{\}\|\/])/g, "\\$1");
						// *はワイルドカード扱いにする
						_image = _image.replace(/\*/g, '(.*)');
						_image = '^' + _image + '$';
						gclog.debug('display image regexp = ' + _image, 2);
						this.displayImages[i] = new RegExp(_image);
					}
				}
				this.isDisplayImage = function(src) {
					for (var i = 0; i < this.displayImages.length; i++) {
						if (src.match(this.displayImages[i])) {
							//gclog.debug('isDisplayImage is true src = ' + src, 2);
							return true;
						}
					}
					return false;
				}
				return this.OUT;
			},
		},

		step1 : {

			'p | div | blockquote' : function step1StartParagraph(tagName, attrs) {

				if (this.countAddedStep('step1') == 0) {
					this.addElement(this.getStep());
				}
				if (this.paraTree && this.paraTree.content) {
					// カスタムツリーに貯めた段落をツリーに出力
					this.paraTree.endElement();
					this.appendTree();
				}
				this.paraTree = new gcparts.ElementTree();
				this.paraTree.addElement(this.getItem('step1.paragraph'));
				return this.CONTINUE;
			},

			'/p | /div | /blockquote' : function step1EndParagraph(tagName) {

				if (this.paraTree && this.paraTree.content) {
					// カスタムツリーに貯めた段落をツリーに出力
					this.paraTree.endElement();
					this.appendTree();
				}
				this.paraTree = new gcparts.ElementTree();

				return this.CONTINUE;
			},

			'span' : function step1StartSpan(tagName, attrs) {
				var elem = this.getItem('step1.text');
				if (this.fontDesign == 1) {
					var style = attrs.get('style');
					if (style) {
						var props = style.replace(/\s*/g, '').split(';');
						var s = '';
						for (var i = 0; i < props.length; i++) {
							if (props[i].match(/^color:/)) {
								s += props[i];
							}
						}
						if (s.length > 0) {
							elem.setAttribute('style', s);
						}
					}
				}
				this.getTree().addElement(elem);
				return this.CONTINUE;
			},

			'font' : function step1StartFont(tagName, attrs) {
				var elem = this.getItem('step1.text');
				if (this.fontDesign == 1) {
					var color = attrs.get('color');
					if (color) {
						elem.setAttribute('style', 'color:' + color);
					}
				}
				this.getTree().addElement(elem);
				return this.CONTINUE;
			},

			'/span | /font' : function step1EndSpanFont(tagName) {
				this.getTree().endElement();
				return this.CONTINUE;
			},

			'b | strong' : function step1StartStrong(tagName, attrs) {
				var elem = this.getItem('step1.strong');
				this.getTree().addElement(elem);
				return this.CONTINUE;
			},

			'/b | /strong' : function step1EndStrong(tagName) {
				this.getTree().endElement();
				return this.CONTINUE;
			},

			'em' : function step1StartEm(tagName, attrs) {
				var elem = this.getItem('step1.em');
				this.getTree().addElement(elem);
				return this.CONTINUE;
			},

			'/em' : function step1EndEm(tagName) {
				this.getTree().endElement();
				return this.CONTINUE;
			},

			's | strike | del' : function step1StartDel(tagName, attrs) {
				var elem = this.getItem('step1.delete');
				this.getTree().addElement(elem);
				return this.CONTINUE;
			},

			'/s | /strike | /del' : function step1EndStrike(tagName) {
				this.getTree().endElement();
				return this.CONTINUE;
			},

			'br' : function step1StartBr(tagName, attrs) {
				this.lineBreak++;
				if (this.lineBreak == 1) {
					this.getTree().addElement(tagName, attrs);
					this.getTree().endElement();
				}
				return this.CONTINUE;
			},

			'a' : function step1StartA(tagName, attrs) {
				this.anchorTree = new gcparts.ElementTree();
				this.anchorTree.addElement(tagName, attrs);

				return this.CONTINUE;
			},

			'/a' : function step1EndA(tagName) {
				if (this.anchorTree && this.anchorTree.content) {
					this.anchorTree.endElement();
					this.paraTree.addTree(this.anchorTree);
					this.paraTree.content = true;
				}
				this.anchorTree = undefined;
				return this.CONTINUE;
			},

			'ul | ol | dl' : function step1StartUlOlDl(tagName, attrs) {

				if (this.countAddedStep('step1') == 0) {
					this.addElement(this.getStep());
				}
				if (this.paraTree && this.paraTree.content) {
					// カスタムツリーに貯めた段落をツリーに出力
					this.paraTree.endElement();
					this.appendTree();
				}
				this.paraTree = new gcparts.ElementTree();
				this.paraTree.addElement(this.getItem('step1.list'));
				return this.CONTINUE;
			},

			'/ul | /ol | /dl' : function step1EndUlOlDl(tagName) {

				if (this.paraTree && this.paraTree.content) {
					this.paraTree.endElement();
					this.appendTree();
				}
				this.paraTree = new gcparts.ElementTree();
				return this.CONTINUE;
			},

			'li | dt | dd' : function step1StartLiDtDd(tagName, attrs) {

				var t = this.getTree();
				var elem;
				if (t.countAddedItem('step1.list') == 0) {
					elem = this.getItem('step1.text');
				} else {
					elem = this.getItem('step1.listItem');
				}
				var c = elem.getAttribute('class');
				if (c) {
					c += " " + tagName;
				} else {
					c = tagName;
				}
				elem.setAttribute('class', c);
				t.addElement(elem);
				return this.CONTINUE;
			},

			'/li | /dt | /dd' : function step1EndLiDtDd(tagName, attrs) {
				this.getTree().endElement();
				return this.CONTINUE;
			},

			'#text' : function step1Text(data) {
				var d = data.replace(/^\s*|\s*$/g, '');
				if (d) {
					var tree = this.getTree();
					tree.addText(this.convertKeywords(d));
					tree.content = true;
					this.lineBreak = 0;
				}
				return this.CONTINUE;
			},

			'img' : function step1Img(tagName, attrs) {

				var tree = this.getTree();
				var imgArea = this.getItem('step1.imgArea');
				var imgAlt = this.getItem('step1.imgAlt');
				var icon1 = this.getItem('step1.icon1');
				var icon2 = this.getItem('step1.icon2');
				var src = attrs.get('src');
				var alt = attrs.get('alt');

				if (this.isIgnoreImage(src)) {
					// この画像は無視し、出力からも除外する
					return this.CONTINUE;
				} else if (this.isDisplayImage(src)) {
					icon2.setAttribute('src', src);
					tree.addElement(icon2);
					tree.endElement();
					tree.content = true;
					return this.CONTINUE;
				}

				var idx = this.imageJSONArray.length;
				this.imageJSONArray[idx] = src;

				switch(this.imageType) {
				case 1:
					// 削除
					if (this.useImageAlt == 1 && alt) {
						// ALT属性を代わりに表示する
						var c = imgAlt.getAttribute('class') || '';
						c += ' no_' + idx;
						imgAlt.setAttribute('class', c);
						tree.addElement(imgAlt);
						tree.addText(this.convertKeywords(alt));
						tree.endElement();
						tree.content = true;
						this.lineBreak = 0;
					}

					// imgは出力から消すが、spanを跡に残す
					if (imgArea) {
						var c = imgArea.getAttribute('class') || '';
						c += ' no_' + idx;
						imgArea.setAttribute('class', c);
						tree.addElement(imgArea);
						tree.endElement();
						tree.content = true;
					}
					break;
				default:
					// アイコン化

					if (this.useImageAlt == 1 && alt) {
						// ALT属性を代わりに表示する
						var c = imgAlt.getAttribute('class') || '';
						c += ' no_' + idx;
						imgAlt.setAttribute('class', c);
						tree.addElement(imgAlt);
						tree.addText(this.convertKeywords(alt));
						tree.endElement();
						tree.content = true;
						this.lineBreak = 0;
					}

					// アイコン表示
					if (imgArea) {
						var c = imgArea.getAttribute('class') || '';
						c += ' no_' + idx;
						imgArea.setAttribute('class', c);
						tree.addElement(imgArea);
						tree.addElement(icon1);
						tree.endElement();	// close img element
						tree.endElement();	// close anchor element
						tree.content = true;
					}

					break;
				}
				return this.CONTINUE;
			},

			'script | style | object | iframe | form' : function step1StartTrash(tagName, attrs) {
				return this.TRASH;
			},

			'h1 | h2 | h3 | h4 | h5 | h6' : function step1StartH1toH6(tagName, attrs) {
				if (this.countAddedStep('step1') == 0) {
					this.addElement(this.getStep());
				}
				if (this.paraTree && this.paraTree.content) {
					// カスタムツリーに貯めた段落をツリーに出力
					this.paraTree.endElement();
					this.appendTree();
				}
				this.paraTree = new gcparts.ElementTree();

				this.lineBreak = 0;
				return this.SUSPEND;
			},

			'/#doc' : function finishedEndDoc() {
				if (this.countAddedStep('step1') == 0) {
					this.addElement(this.getStep());
				}
				if (this.paraTree && this.paraTree.content) {
					this.paraTree.endElement();
					this.appendTree();
				}
				return this.OUT;
			}
		},


		step2s : {

			'in' : function step2sIn(tagName, attrs) {
				switch(tagName) {
				case 'h1':
				case 'h2':
				case 'h3':
				case 'h4':
				case 'h5':
				case 'h6':
					this.text2 = {};
					this.link = undefined;
					return this.CONTINUE;

				default:
					return this.STAY;
				}
			},
			'out' : function step2sOut(tagName) {

				var t = undefined;
				if (this.text2.text) {
					t = this.text2.text;
				} else if (this.text2.alt) {
					t = this.text2.alt;
				} else if (this.text2.title) {
					t = this.text2.title;
				}
				if (t) {
					var tree;
					if (this.paraTree.isEmpty()) {
						tree = this;
					} else {
						tree = this.paraTree;
					}
					tree.addElement(this.getStep());
					if (this.link) {
						tree.addElement(this.link.tagName, this.link.attrs);
					}
					tree.addText(this.convertKeywords(t));
					if (this.link) {
						tree.endElement();
					}
					tree.endElement();
					tree.content = true;
				}
				return this.OUT;
			},

			'script | style | object | iframe | table' : function step2sStartTrash(tagName, attrs) {
				return this.TRASH;
			},

			'a': function step2sStartA(tagName, attrs) {
				this.link = { tagName : tagName, attrs : attrs };

				var title = attrs.get('title');
				if (title) {
					// 見出しテキスト候補３
					this.text2.title = title;
				}
				return this.CONTINUE;
			},

			'#text' : function step2sText(data) {
				var d = data.replace(/^\s*|\s*$/g, '');
				if (d) {
					// 見出しテキスト
					if (this.text2.text) {
						this.text2.text += data;
					} else {
						this.text2.text = data;
					}
				}
				return this.CONTINUE;
			},

			'img': function step2sImg(tagName, attrs) {
				var alt = attrs.get('alt');
				if (alt) {
					// 見出しテキスト候補２
					this.text2.alt = alt;
				}
				return this.CONTINUE;
			}
		},

		'finished' : {
			'/#doc' : function finishedEndDoc() {
				// JSONデータを吐き出して終了
				var jsonElem = this.getItem('finished.json');
				var jsonData;
				try {
					jsonData = JSON.stringify(this.imageJSONArray);
				} catch(err) {
					gclog.error('cannot toJSON', 2);
					gclog.error(err, 2);
				}
				if (jsonElem && jsonData) {
					this.addElement(jsonElem);
					this.addText(jsonData);
					this.endElement();
				}

				// 画像表示用エリアの出力
				switch(this.imageType) {
				case 1:
					// 削除
					break;
				default:
					// アイコン化
					var imageView = this.getItem('finished.imageView');
					if (imageView) {
						this.addElement(imageView);
						this.endElement();
					}
					break;
				}
				return this.EXIT;
			}
		},

		'failed' : {
			'/#doc' : function failedEndDoc() {
				return this.ERROR;
			}
		}
	};
	gcparts.partsRules[rule._name] =  rule;
}());
