/**
 * jQuery Mobile フォームパーツ
 */

(function() {
	var rule = {
		_name: "JQMBUTTON01",
		_dic: "jQueryMobile ボタンパーツ",

		start : {
			'#doc' : function step1StartDoc() {

				// パラメータの取得・初期化
				// フォームID
				this.formId = this.getParameter('formId') || '';

				// 入力HTMLに含まれるフォームコンポーネント以外のコンテンツを出力するかどうか
				this.panelMode = this.getParameter('panelMode') || 'OFF';

				// リンクをボタン化するかどうか
				this.anchorButton = this.getParameter('anchorButton') || '0';
				// ボタン化する対象リンクの条件（id/class属性値）
				this.anchorButtonValue = this.getParameter('anchorButtonValue') || '';
				switch(this.anchorButton) {
				case '3':
				case '4':
					if (!this.anchorButtonValue) {
						// 条件となる値が指定されていなければ、ボタン化しない
						this.anchorButton = '0';
					}
					break;
				}
				// ミニサイズ表示
				this.miniSize = this.getParameter('miniSize') || 'OFF';
				// インライン表示（ボタン）
				this.inline = this.getParameter('inline') || 'OFF';
				// 画像ボタンをテキスト化するか
				this.imageToTextButton = this.getParameter('imageToTextButton') || 'OFF';
				// 注釈
				this.footerTexts = [];
				// ボタンテキスト
				this.buttonTexts = [];
				// アイコンの表示および種類
				this.iconTypes = [];
				// アイコンの表示位置
				this.iconPositions = [];

				for (var i = 0; i < 100; i++) {
					this.footerTexts[i] = this.getParameter('footerText' + (i + 1)) || '';
					this.buttonTexts[i] = this.getParameter('buttonText' + (i + 1)) || '';
					this.iconTypes[i] = this.getParameter('iconType' + (i + 1)) || 'OFF';
					this.iconPositions[i] = this.getParameter('iconPosition' + (i + 1)) || 'left';
				}
				// エラーメッセージ部分の探し方
				this.errorTextType = this.getParameter('errorTextType') || '';
				// エラーメッセージ部分の指定値
				this.errorTextValue = this.getParameter('errorTextValue') || '';

				// ボタン出現カウンタ
				this.buttonIndex = 0;

				// バッファ変数
				// 見出し
				this.titleTree = new gcparts.ElementTree('titleTree');
				// コンテンツツリー
				this.contentTree = new gcparts.ElementTree('contentTree');
				// ボタンツリー
				this.buttonTree;

				// title / label / errorTextのどれかによって処理を違えるべく、step2sにサスペンド時に値を保持する
				this.itemName;

				// PC側のフォーム系要素の属性のうち、削除する属性定義（ブラックリスト）（item:start.removeAttrs）
				var _removes = this.getItem('start.removeAttrs');
				if (_removes) {
					this.removes = _removes.getAttribute('value').replace(/\s*/g, '').split(',');
				} else {
					this.removes = [];
				}
				this.removeAttrs = function(attrs) {
					if (!attrs) {
						return attrs;
					}
					this.removes.forEach(function(r){
						r = r.toLowerCase();
						if (attrs.has(r)) {
							attrs.remove(r);
						}
					});
					return attrs;
				}
				this.matchClassAttribute = function(classNames, target) {
					var _class = classNames.split(' ');
					for (var i = 0; i < _class.length; i++) {
						if (_class[i] == target) {
							// find class attribute value
							return true;
						}
					}
					return false;
				}
				this.hasEventAttribute = function(attrs) {
					if (!attrs) {
						return false;
					}
					for (var ar in attrs.attrs) {
						ar = ar.toLowerCase();
						if (ar.match(/^on/)) {
							return true;
						}
					}
					if (attrs.has('href')) {
						var _href = attrs['href'];
						if (_href.match(/javascript:/i)) {
							return true;
						}
					}
					return false;
				}
				return this.OUT;
			},
		},

		step1: {

			'input': function step1StartInput(tagName, attrs) {
				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();

				switch(_type) {
				case 'image':
					if (this.imageToTextButton == 'ON') {
						var _alt = attrs.get('alt');
						if (_alt) {
							attrs.set('type', 'submit');
							attrs.set('value', _alt);
						}
					}
					break;
				case 'button':
				case 'submit':
				case 'reset':
					break;
				default:
					// ボタン系以外のinput要素はstep2sへ
					return this.SUSPEND;
				}
				if (this.miniSize == 'ON') {
					// ミニサイズ表示
					attrs.set('data-mini', 'true');
				}
				if (this.inline == 'ON') {
					attrs.set('data-inline', 'true');
				}
				if (this.buttonIndex < this.iconTypes.length && this.iconTypes[this.buttonIndex] != 'OFF') {
					attrs.set('data-icon', this.iconTypes[this.buttonIndex]);
					if (this.iconPositions[this.buttonIndex]) {
						attrs.set('data-iconpos', this.iconPositions[this.buttonIndex]);
					}
				}
				if (this.buttonIndex < this.buttonTexts.length && this.buttonTexts[this.buttonIndex]) {
					if (_type == 'image') {
						attrs.set('type', 'submit');
					}
					attrs.set('value', this.buttonTexts[this.buttonIndex]);
				}

				if (this.formId) {
					// form属性の設定
					attrs.set('form', this.formId);
				}

				var _button = this.getItem('button');
				if (_button) {
					_button.mergeAttribute(this.removeAttrs(attrs));
					this.contentTree.addElement(_button);
				} else {
					this.contentTree.addElement(tagName, this.removeAttrs(attrs));
				}
				this.contentTree.endElement();

				// 注釈テキストの表示
				if (this.panelMode == 'OFF') {
					if (this.buttonIndex < this.footerTexts.length && this.footerTexts[this.buttonIndex]) {
						var _note = this.getItem('note');
						if (_note) {
							this.contentTree.addElement(_note);
							this.contentTree.addText(this.footerTexts[this.buttonIndex]);
							this.contentTree.endElement();
						} else {
							this.contentTree.addText(this.footerTexts[this.buttonIndex]);
						}
					}
				}

				this.buttonIndex++;
				return this.CONTINUE;
			},

			'/input': function step1EndInput(tagName) {
				return this.CONTINUE;
			},

			'button': function step1StartButton(tagName, attrs) {
				if (this.miniSize == 'ON') {
					// ミニサイズ表示
					attrs.set('data-mini', 'true');
				}
				if (this.inline == 'ON') {
					attrs.set('data-inline', 'true');
				}
				if (this.buttonIndex < this.iconTypes.length && this.iconTypes[this.buttonIndex] != 'OFF') {
					attrs.set('data-icon', this.iconTypes[this.buttonIndex]);
					if (this.iconPositions[this.buttonIndex]) {
						attrs.set('data-iconpos', this.iconPositions[this.buttonIndex]);
					}
				}
				this.buttonIndex++;
				if (this.formId) {
					// form属性の設定
					attrs.set('form', this.formId);
				}
				this.buttonTree = new gcparts.ElementTree('buttonTree');
				this.buttonTree.addElement(tagName, this.removeAttrs(attrs));
				return this.CONTINUE;
			},

			'/button': function step1EndButton(tagName) {
				if (this.buttonIndex < this.buttonTexts.length && this.buttonTexts[this.buttonIndex]) {
					this.buttonTree.addText(this.buttonTexts[this.buttonIndex]);
				}

				this.buttonTree.endElement();
				this.contentTree.addTree(this.buttonTree);

				// 注釈テキストの表示
				if (this.panelMode == 'OFF') {
					if (this.buttonIndex < this.footerTexts.length && this.footerTexts[this.buttonIndex]) {
						var _note = this.getItem('note');
						if (_note) {
							this.contentTree.addElement(_note);
							this.contentTree.addText(this.footerTexts[this.buttonIndex]);
							this.contentTree.endElement();
						} else {
							this.contentTree.addText(this.footerTexts[this.buttonIndex]);
						}
					}
				}

				this.buttonIndex++;
				this.buttonTree = undefined;

				return this.CONTINUE;
			},

			'a': function step1StartAnchor(tagName, attrs) {

				switch(this.anchorButton) {
				case '0':
					// ボタン化しない
					return this.SUSPEND;
				case '1':
					// ボタン化する
					if (!attrs.has('href')) {
						return this.SUSPEND;
					}
					break;
				case '2':
					// イベントハンドラがあるとき、ボタン化する
					if (!this.hasEventAttribute(attrs)) {
						return this.SUSPEND;
					}
					break;
				case '3':
					// id属性値が指定したものと一致したとき、ボタン化する
					var _id = attrs.get('id');
					if (this.anchorButtonValue != _id) {
						return this.SUSPEND;
					}
					break;
				case '4':
					// class属性値が指定したものと一致したとき、ボタン化する
					var _class = attrs.get('class');
					if (this.anchorButtonValue != _class) {
						return this.SUSPEND;
					}
					break;
				}
				// ボタン表示する
				attrs.set('data-role', 'button');

				if (this.miniSize == 'ON') {
					// ミニサイズ表示
					attrs.set('data-mini', 'true');
				}

				if (this.inline == 'ON') {
					attrs.set('data-inline', 'true');
				}
				if (this.buttonIndex < this.iconTypes.length && this.iconTypes[this.buttonIndex] != 'OFF') {
					attrs.set('data-icon', this.iconTypes[this.buttonIndex]);
					if (this.iconPositions[this.buttonIndex]) {
						attrs.set('data-iconpos', this.iconPositions[this.buttonIndex]);
					}
				}

				this.buttonTree = new gcparts.ElementTree('buttonTree');
				this.buttonTree.addElement(tagName, this.removeAttrs(attrs));
				return this.CONTINUE;
			},

			'/a': function step1EndAnchor(tagName) {

				if (this.buttonIndex < this.buttonTexts.length && this.buttonTexts[this.buttonIndex]) {
					this.buttonTree.addText(this.buttonTexts[this.buttonIndex]);
				}

				this.buttonTree.endElement();
				this.contentTree.addTree(this.buttonTree);

				// 注釈テキストの表示
				if (this.panelMode == 'OFF') {
					if (this.buttonIndex < this.footerTexts.length && this.footerTexts[this.buttonIndex]) {
						var _note = this.getItem('note');
						if (_note) {
							this.contentTree.addElement(_note);
							this.contentTree.addText(this.footerTexts[this.buttonIndex]);
							this.contentTree.endElement();
						} else {
							this.contentTree.addText(this.footerTexts[this.buttonIndex]);
						}
					}
				}

				this.buttonIndex++;
				this.buttonTree = undefined

				return this.CONTINUE;
			},

			'#text': function step1Text(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}
				if (this.buttonTree) {
					if (this.buttonIndex < this.buttonTexts.length && this.buttonTexts[this.buttonIndex]) {
						return this.CONTINUE;
					}
					this.buttonTree.addText(data);
				} else {
					if (this.panelMode == 'OFF') {
						// パネルモードOFF
						return this.CONTINUE;
					}
					this.contentTree.addText(data);
				}
				return this.CONTINUE;
			},

			'img': function start1Img(tagName, attrs) {

				if (this.buttonTree) {
					if (this.imageToTextButton == 'ON') {
						if (this.buttonIndex < this.buttonTexts.length && this.buttonTexts[this.buttonIndex]) {
							return this.CONTINUE;
						}
						var _alt = attrs.get('alt');
						if (_alt) {
							this.buttonTree.addText(_alt);
						}
					} else {
						this.buttonTree.addElement(tagName, this.removeAttrs(attrs));
						this.buttonTree.endElement();
					}
					return this.CONTINUE;
				} else {
					return this.SUSPEND;
				}
			},

			'/img': function start1EndImg(tagName) {
				return this.CONTINUE;
			},

			'*': function step1StartOther(tagName, attrs) {

				if (this.buttonTree) {
					return this.CONTINUE;
				}
				return this.SUSPEND;
			},

			'/*': function step1EndOther(tagName, attrs) {

				if (this.skip) {
					return this.CONTINUE;
				}

				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}
				this.contentTree.endElement();
				return this.CONTINUE;
			}
		},

		step2s: {

			'*': function step2sStartOther(tagName, attrs) {
				var gclabel = attrs.get("data-gc-label");
				switch(gclabel) {
				case 'title':
					// 見出し用のラベルパスまたは固定文言の指定
					this.itemName = 'title';
					return this.SUSPEND;
				}
				// エラーメッセージ部分の指定がされている場合は、SUSPEND
				if (this.errorTextValue) {
					switch(this.errorTextType) {
					case '1':
						// data-gc-label属性指定
						if (this.errorTextValue == gclabel) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '2':
						// id属性指定
						if (this.errorTextValue == attrs.get('id')) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '3':
						// class属性指定
						if (this.matchClassAttribute(attrs.get('class'), this.errorTextValue)) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '4':
						// 要素名指定
						if (this.errorTextValue.toLowerCase() == tagName) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					}
				}

				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.contentTree.addElement(step);
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.contentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'/*': function step2sEndOther(tagName) {

				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.contentTree.endElement();
					break;
				}
				return this.CONTINUE;
			},

			'#text': function step2sText(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}
				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				this.contentTree.addText(data);
				return this.CONTINUE;
			},

			'input': function step2sStartInput(tagName, attrs) {
				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();

				switch(_type) {
				case 'button':
				case 'submit':
				case 'reset':
				case 'image':
					// ボタン系のinputは元のステップに戻る
					return this.OUT_RESEND;
				}
				// ボタン以外は捨てる
				return this.TRASH;
			},

			'button': function step2sStartButton(tagName, attrs) {
				return this.OUT_RESEND;
			},


			'a': function step2sStartAnchor(tagName, attrs) {

				switch(this.anchorButton) {
				case '0':
					// ボタン化しない
					break;
				case '1':
					// ボタン化する
					if (attrs.has('href')) {
						return this.OUT_RESEND;
					}
					break;
				case '2':
					// イベントハンドラがあるとき、ボタン化する
					if (this.hasEventAttribute(attrs)) {
						return this.OUT_RESEND;
					}
					break;
				case '3':
					// id属性値が指定したものと一致したとき、ボタン化する
					var _id = attrs.get('id');
					if (this.anchorButtonValue == _id) {
						return this.OUT_RESEND;
					}
					break;
				case '4':
					// class属性値が指定したものと一致したとき、ボタン化する
					var _class = attrs.get('class');
					if (this.anchorButtonValue == _class) {
						return this.OUT_RESEND;
					}
					break;
				}

				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}
				this.contentTree.addElement(tagName, this.removeAttrs(attrs));
				return this.CONTINUE;
			},

			'/a': function step2sEndAnchor(tagName) {
				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}
				this.contentTree.endElement();
				return this.CONTINUE;
			},

			'select | textarea | keygen' : function step2sStartOtherFormComponents(tagName, attrs) {
				return this.TRASH;
			},

			'script | noscript' : function step2sStartScript(tagName, attrs) {
				return this.TRASH;
			}

		},


		step3s: {

			'in': function step3sIn(tagName, attrs) {

				if (this.itemName == 'title') {
					this.currentTree = this.titleTree;
				} else {
					this.currentTree = this.contentTree;
					this.currentTree.addElement(this.getItem(this.itemName));
				}
				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.currentTree.addElement(step);
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'out': function step3sOut(tagName) {
				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.endElement();
					break;
				}
				if (this.itemName != 'title') {
					this.currentTree.endElement();
				}
				return this.OUT;
			},

			'#text': function step3sText(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');

				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}
				this.currentTree.addText(data);
				return this.CONTINUE;
			},

			'input': function step3sStartInput(tagName, attrs) {
				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();
				switch(_type) {
				case 'button':
				case 'submit':
				case 'reset':
				case 'image':
					return this.OUT_RESEND;
				}
				return this.TRASH;
			},

			'button': function step3sStartButton(tagName, attrs) {
				return this.OUT_RESEND;
			},

			'select | textarea | keygen': function step2sStartOtherFormComponents(tagName, attrs) {
				return this.TRASH;
			},

			'a': function step3sStartAnchor(tagName, attrs) {

				switch(this.anchorButton) {
				case '0':
					// ボタン化しない
					break;
				case '1':
					// ボタン化する
					if (attrs.has('href')) {
						return this.OUT_RESEND;
					}
					break;
				case '2':
					// イベントハンドラがあるとき、ボタン化する
					if (this.hasEventAttribute(attrs)) {
						return this.OUT_RESEND;
					}
					break;
				case '3':
					// id属性値が指定したものと一致したとき、ボタン化する
					var _id = attrs.get('id');
					if (this.anchorButtonValue == _id) {
						return this.OUT_RESEND;
					}
					break;
				case '4':
					// class属性値が指定したものと一致したとき、ボタン化する
					var _class = attrs.get('class');
					if (this.anchorButtonValue == _class) {
						return this.OUT_RESEND;
					}
					break;
				}

				this.contentTree.addElement(tagName, this.removeAttrs(attrs));
				return this.CONTINUE;
			},

			'*': function step3sStartOther(tagName, attrs) {
				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'/*': function step3sEndOther(tagName) {
				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.endElement();
					break;
				}
				return this.CONTINUE;
			},

			'script | noscript' : function step3sStartScript(tagName, attrs) {
				return this.TRASH;
			}

		},


		finished : {
			'/#doc' : function finishedEndDoc() {


				if (this.titleTree.isEmpty() &&
						this.contentTree.isEmpty()) {
					// ツリーに出力がなければ外枠も出力しない
					return this.EXIT;
				}

				var _wrapper = this.getItem('wrapper');
				var _title = this.getItem('title');

				this.addElement(_wrapper);

				if (this.titleTree.isEmpty()) {
					var _dataRole = _wrapper.getAttribute('data-role') || '';
					var _titleText = '';
					switch(_dataRole){
					case 'fieldcontain':
					//case 'controlgroup':
						_wrapper.setAttribute('class', 'ui-hide-label');
						break;
					}
					_title.setAttribute('class', 'ui-hidden-accessible');
					_titleText = '';

					this.addElement(_title);
					this.addText(_titleText);
					this.endElement();
				} else {
					this.addElement(_title);
					this.addTree(this.titleTree);
					this.endElement();
				}

				if (!this.contentTree.isEmpty()) {
					this.addTree(this.contentTree);
				}
				this.endElement();	// closer wrapper element

				return this.EXIT;
			}
		},

		failed : {
			'/#doc' : function failedEndDoc() {
				return this.ERROR;
			}
		}
	};
	gcparts.partsRules[rule._name] =  rule;
}());
