/**
 * jQuery Mobile フォームパーツ
 */

(function() {
	var rule = {
		_name: "JQMDATEFIELDS01",
		_dic: "jQueryMobile 年月日入力フィールドパーツ",

		start : {
			'#doc' : function step1StartDoc() {

				// パラメータの取得・初期化
				// フォームID
				this.formId = this.getParameter('formId') || '';

				// 入力HTMLに含まれるフォームコンポーネント以外のコンテンツを出力するかどうか
				this.panelMode = this.getParameter('panelMode') || 'OFF';

				// 注釈
				this.footerTexts = [];
				// ラベル
				this.labels = [];
				for (var i = 0; i < 100; i++) {
					this.footerTexts[i] = this.getParameter('footerText' + (i + 1)) || '';
					this.labels[i] = this.getParameter('labelText' + (i + 1)) || '';
				}
				// ラベルをプレースホルダ化するかどうか
				this.placeholder = this.getParameter('placeholder') || 'OFF';
				
				
				// セレクトボックス出現カウンタ
				this.selectGroupIndex = 0;

				// ミニサイズ表示
				this.miniSize = this.getParameter('miniSize') || 'OFF';
				// インライン表示
				this.inline = this.getParameter('inline') || 'OFF';
				// 入力の必須化
				this.required = this.getParameter('required') || 'OFF';
				// メニュータイプ
				this.menuType = this.getParameter('menuType') || 'OFF';
				// エラーメッセージ部分の探し方
				this.errorTextType = this.getParameter('errorTextType') || '';
				// エラーメッセージ部分の指定値
				this.errorTextValue = this.getParameter('errorTextValue') || '';

				// バッファ変数
				// 見出し
				this.titleTree = new gcparts.ElementTree('titleTree');
				// コンテンツツリー
				this.contentTree = new gcparts.ElementTree('contents');
				// 年月日フィールド用ツリー
				this.dateFieldTree = new gcparts.ElementTree('dateFieldTree');

				this.fields = [];

				this.tagNames = {};


				// 注釈エリア用ツリー
				this.footerTree = new gcparts.ElementTree('footerTree');

				// title / errorTextのどれかによって処理を違えるべく、step2sにサスペンド時に値を保持する
				this.itemName;

				// PC側のフォーム系要素の属性のうち、削除する属性定義（ブラックリスト）（item:start.removeAttrs）
				var _removes = this.getItem('start.removeAttrs');
				if (_removes) {
					this.removes = _removes.getAttribute('value').replace(/\s*/g, '').split(',');
				} else {
					this.removes = [];
				}
				this.removeAttrs = function(attrs) {
					if (!attrs) {
						return attrs;
					}
					this.removes.forEach(function(r){
						r = r.toLowerCase();
						if (attrs.has(r)) {
							attrs.remove(r);
						}
					});
					return attrs;
				}
				this.matchClassAttribute = function(classNames, target) {
					var _class = classNames.split(' ');
					for (var i = 0; i < _class.length; i++) {
						if (_class[i] == target) {
							// find class attribute value
							return true;
						}
					}
					return false;
				}
				return this.OUT;
			},
		},

		step1: {

			'select': function step1StartSelect(tagName, attrs) {

				if (this.miniSize == 'ON') {
					// ミニサイズ表示
					attrs.set('data-mini', 'true');
				}
				if (this.required == 'ON') {
					// 入力の必須化
					attrs.set('required', 'required');
				}
				if (this.inline == 'ON') {
					// インライン表示
					attrs.set('data-inline', 'true');
				}
				if (this.menuType == 'JQM') {
					// JQM独自メニュー
					attrs.set('data-native-menu', 'false');
				}
				if (this.formId) {
					// form属性の設定
					attrs.set('form', this.formId);
				}

				var _select = this.getItem('select');
				if (_select) {
					_select.mergeAttribute(this.removeAttrs(attrs));
					this.dateFieldTree.addElement(_select);
				} else {
					this.dateFieldTree.addElement(tagName, this.removeAttrs(attrs));
				}
				this.dateFieldTree.selectName = attrs.get('name');

				if (this.placeholder == 'ON') {
					var _placeholder = this.getItem('placeholder');
					var _t = (this.selectGroupIndex < this.labels.length) ? this.labels[this.selectGroupIndex] : '';
					if (_placeholder && _t) {
						this.dateFieldTree.addElement(_placeholder);
						this.dateFieldTree.addText(_t);
						this.dateFieldTree.endElement();
					}
				}

				this.skip = true;
				this.tagNames[tagName] = tagName;
				return this.CONTINUE;
			},

			'/select': function step1EndSelect(tagName) {
				if (!this.dateFieldTree.isEmpty()) {
					this.dateFieldTree.endElement();

					this.fields.push(this.dateFieldTree);
					this.dateFieldTree = new gcparts.ElementTree('dateFieldTree');

					this.selectGroupIndex++;
				}
				this.skip = false;
				return this.CONTINUE;
			},

			'option': function step1StartOption(tagName, attrs) {

				if (this.dateFieldTree.countAddedItem('placeholder') > 0
						&& !this.dateFieldTree.hasElement('option')
						&& !attrs.get('value')) {
					return this.TRASH;
				}

				this.dateFieldTree.addElement(tagName, this.removeAttrs(attrs));
				return this.CONTINUE;
			},

			'/option': function step1EndOption(tagName) {
				this.dateFieldTree.endElement();
				return this.CONTINUE;
			},

			'input': function step1StartInput(tagName, attrs) {
				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();

				switch(_type) {
				case 'button':
				case 'submit':
				case 'reset':
				case 'image':
				case 'hidden':
				case 'radio':
				case 'checkbox':
				case 'file':
					return this.SUSPEND;
				}

				if (this.miniSize == 'ON') {
					// ミニサイズ表示
					attrs.set('data-mini', 'true');
				}
				if (this.required == 'ON') {
					// 入力の必須化
					attrs.set('required', 'required');
				}
				if (this.inline == 'ON') {
					// インライン表示
					attrs.set('data-inline', 'true');
				}
				if (this.menuType == 'JQM') {
					// JQM独自メニュー
					attrs.set('data-native-menu', 'false');
				}
				if (this.formId) {
					// form属性の設定
					attrs.set('form', this.formId);
				}

				if (this.placeholder == 'ON') {
					var _t = (this.selectGroupIndex < this.labels.length) ? this.labels[this.selectGroupIndex] : '';
					if (_t) {
						attrs.set('placeholder', _t);
					}
				}

				if (gcutil.isGroup('ios')) {
					attrs.set('type', 'text');
					attrs.set('pattern', '[0-9]*');
				} else {
					attrs.set('type', 'number');
				}

				var _textbox = this.getItem('textbox');
				if (_textbox) {
					_textbox.mergeAttribute(this.removeAttrs(attrs));
					this.dateFieldTree.addElement(_textbox);
					this.dateFieldTree.endElement();
				} else {
					this.dateFieldTree.addElement(tagName, this.removeAttrs(attrs));
					this.dateFieldTree.endElement();
				}

				this.fields.push(this.dateFieldTree);

				this.dateFieldTree = new gcparts.ElementTree('dateFieldTree');

				this.selectGroupIndex++;
				this.tagNames[tagName] = tagName;
				this.skip = true;

				return this.CONTINUE;
			},

			'/input': function step1EndInput(tagName) {
				this.skip = false;
				return this.CONTINUE;
			},

			'#text': function step1Text(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}
				if (this.skip) {
					this.dateFieldTree.addText(data);
				} else {
					if (this.panelMode == 'OFF') {
						// パネルモードOFF
						return this.CONTINUE;
					}
					this.contentTree.addText(data);
				}
				return this.CONTINUE;
			},

			'*': function step1StartOther(tagName, attrs) {
				if (this.skip) {
					return this.CONTINUE;
				}
				return this.SUSPEND;
			},

			'/*': function step1EndOther(tagName) {
				if (this.skip) {
					return this.CONTINUE;
				}
				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				this.contentTree.endElement();
				return this.CONTINUE;
			}

		},

		step2s: {

			'*': function step2sStartOther(tagName, attrs) {
				var gclabel = attrs.get("data-gc-label");
				switch(gclabel) {
				case 'title':
					// 見出し用のラベルパスまたは固定文言の指定
					this.itemName = 'title';
					return this.SUSPEND;
				}
				// エラーメッセージ部分の指定がされている場合は、SUSPEND
				if (this.errorTextValue) {
					switch(this.errorTextType) {
					case '1':
						// data-gc-label属性指定
						if (this.errorTextValue == gclabel) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '2':
						// id属性指定
						if (this.errorTextValue == attrs.get('id')) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '3':
						// class属性指定
						if (this.matchClassAttribute(attrs.get('class'), this.errorTextValue)) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '4':
						// 要素名指定
						if (this.errorTextValue.toLowerCase() == tagName) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					}
				}
				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.contentTree.addElement(step);
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.contentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'/*': function step2sEndOther(tagName) {

				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.contentTree.endElement();
					break;
				}
				return this.CONTINUE;
			},

			'#text': function step2sText(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}

				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				this.contentTree.addText(data);
				return this.CONTINUE;
			},

			'select' : function step2sStartSelect(tagName, attrs) {
				return this.OUT_RESEND;
			},

			'input': function step2sStartInput(tagName, attrs) {

				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();
				switch(_type) {
				case 'button':
				case 'submit':
				case 'reset':
				case 'image':
				case 'hidden':
				case 'radio':
				case 'checkbox':
				case 'file':
					return this.TRASH;
				}
				return this.OUT_RESEND;
			},

			'button | textarea | keygen' : function step2sStartOtherFormComponents(tagName, attrs) {
				return this.TRASH;
			},

			'script | noscript' : function step2sStartScript(tagName, attrs) {
				return this.TRASH;
			}

		},


		step3s: {

			'in': function step3sIn(tagName, attrs) {

				if (this.itemName == 'title') {
					this.currentTree = this.titleTree;
				} else {
					this.currentTree = this.contentTree;
					this.currentTree.addElement(this.getItem(this.itemName));
				}
				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.currentTree.addElement(step);
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'out': function step3sOut(tagName) {
				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.endElement();
					break;
				}
				if (this.itemName != 'title') {
					this.currentTree.endElement();
				}
				return this.OUT;
			},

			'#text': function step3sText(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');

				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}
				this.currentTree.addText(data);
				return this.CONTINUE;
			},

			'select': function step3sStartSelect(tagName, attrs) {
				return this.OUT_RESEND;
			},

			'input': function step3sStartInput(tagName, attrs) {

				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();
				switch(_type) {
				case 'button':
				case 'submit':
				case 'reset':
				case 'image':
				case 'hidden':
				case 'radio':
				case 'checkbox':
				case 'file':
					return this.TRASH;
				}
				return this.OUT_RESEND;
			},

			'button | textarea | keygen' : function step3sEndOtherFormComponents(tagName, attrs) {
				return this.TRASH;
			},

			'*': function step3sStartOther(tagName, attrs) {
				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'/*': function step3sEndOther(tagName) {
				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.endElement();
					break;
				}
				return this.CONTINUE;
			},

			'script | noscript' : function step3sStartScript(tagName, attrs) {
				return this.TRASH;
			}

		},


		finished : {
			'/#doc' : function finishedEndDoc() {

				var _count = this.fields.length;

				if (this.titleTree.isEmpty() &&
						_count == 0 &&
						this.contentTree.isEmpty()) {
					// ツリーに出力がなければ外枠も出力しない
					return this.EXIT;
				}

				var _wrapper = this.getItem('wrapper');
				var _title = this.getItem('title');

				this.addElement(_wrapper);

				if (this.titleTree.isEmpty()) {
					var _dataRole = _wrapper.getAttribute('data-role') || '';
					var _titleText = '';
					switch(_dataRole){
					case 'fieldcontain':
					//case 'controlgroup':
						_wrapper.setAttribute('class', 'ui-hide-label');
						break;
					}
					_title.setAttribute('class', 'ui-hidden-accessible');
					_titleText = '';

					this.addElement(_title);
					this.addText(_titleText);
					this.endElement();
				} else {
					this.addElement(_title);
					this.addTree(this.titleTree);
					this.endElement();
				}

				if (_count > 0) {
				
					// グルーピングする
					var _isGrouping = true;
					if (this.placeholder == 'ON' && this.tagNames['select'] && this.tagNames['input']) {
						// selectとinputが混在している
						_isGrouping = false;
					} else if (this.placeholder == 'ON') {
						_isGrouping = true;
					} else {
						_isGrouping = false;
					}
					if (_isGrouping) {
						var _group = this.getItem('group');
						this.addElement(_group);

						for (var i = 0; i < _count; i++) {
							this.addTree(this.fields[i]);
						}
						this.endElement();	// close controlgroup element
					} else {
						var _g = this.getItem('grid' + _count);
						if (_g) {
							this.addElement(_g);
						}
						
						for (var i = 0; i < _count; i++) {
							var _b = this.getItem('block'+ _count + '.' + i);
							if (_b) {
								this.addElement(_b);
							}
							this.addTree(this.fields[i]);
							var _t = (_count < this.labels.length) ? this.labels[i] : '';
							if (_t) {
								var _l = this.getItem('label' + _count);
								if (_l) {
									this.addElement(_l);
								}
								this.addText(_t);
								if (_l) {
									this.endElement();
								}
							}
							if (_b) {
								this.endElement();
							}
						}
						if (_g) {
							this.endElement();
						}
					}
				
				}

				if (!this.contentTree.isEmpty()) {
					this.addTree(this.contentTree);
				}

				this.endElement();	// closer wrapper element

				return this.EXIT;
			}
		},

		failed : {
			'/#doc' : function failedEndDoc() {
				return this.ERROR;
			}
		}
	};
	gcparts.partsRules[rule._name] =  rule;
}());
