/**
 * jQuery Mobile フォームパーツ
 */

(function() {
	var rule = {
		_name: "JQMFORM01",
		_dic: "jQueryMobileフォーム",

		start : {
			'#doc' : function step1StartDoc() {

				this.inputs = [];
				this.selects = [];
				this.textareas = [];
				this.buttons = [];

				this.useAjax = this.getParameter('useAjax');
				if (!this.useAjax || this.useAjax != 'true') {
					this.useAjax = 'false';
				}
				this.formId = this.getParameter('formId');

				this.invisibleTree = new gcparts.ElementTree("invisible");
				this.invisibleTree.addElement("div", {"style" : "display:inline", "class": "gcpInvisibleBlock"});
				this.label = false;
				this.labelText = "";
				this.text = false;
				
				this.autoComplete = this.getParameter('autoComplete');
				if (!this.autoComplete || this.autoComplete != 'true') {
					this.autoComplete = 'false';
				}

				return this.OUT;
			},
		},

		step1: {
			'label': function step1StartLabel(tagName, attrs) {
				this.label = true;
				
				return this.CONTINUE;
			},
			'#text': function step1Text(data) {
				if (this.label) {
					this.labelText += data;
					this.text = true;
					if (this.tagName) {
						this.textAfter = true;
					} else {
						this.textAfter = false;
					}
				}
				
				return this.CONTINUE;
			},
			'/label': function step1EndLabel(tagName) {
				if (this.label && this.text && this.tagName) {
					this.invisibleTree.addElement("label", {'style': 'display:none;'});
					if (!this.textAfter) {
						this.invisibleTree.addText(this.labelText);
					}
					if (this.formId) {
						this.attrs.set("form", this.formId);
					}
					this.invisibleTree.addElement(this.tagName, this.attrs);
					this.invisibleTree.endElement();
					if (this.textAfter) {
						this.invisibleTree.addText(this.labelText);
					}
					this.invisibleTree.endElement();
				}
				this.tagName = undefined;
				this.attrs = undefined;
				this.label = false;
				this.labelText = "";
				this.text = false;
				return this.CONTINUE;
			},

			'form': function step1StartForm(tagName, attrs) {

				attrs.set('data-ajax', this.useAjax);
				
				if (this.formId) {
					attrs.set('id', this.formId);
				} else {
					if (attrs.has('id') && attrs.get('id')) {
						this.formId = attrs.get('id');
					}
				}
				if (!attrs.has('action') || !attrs.get('action')) {
					var uri = gcutil.getUri();
					if (uri) {
						attrs.set('action', uri);
					}
				}
				this.addElement(tagName, attrs);
				return this.CONTINUE;
			},

			'input': function step1StartInput(tagName, attrs) {
				var type = attrs.get('type');

				switch(type) {
					case 'hidden':
						if (this.formId) {
							attrs.set('form', this.formId);
						}
						this.addElement(tagName, attrs);
						this.endElement();
						break;
					default:
					
						if (this.label) {
							attrs.set("disabled", "disabled");
							attrs.set("style", "display:none");
							attrs.set("class", "gcFormInvisible");
							this.tagName = tagName;
							this.attrs = attrs;
						}
						break;
				}
				return this.CONTINUE;
			},

			'/input': function step1EndInput(tagName) {
				return this.CONTINUE;
			},

			'select | textarea | button': function step1StartSuspend(tagName, attrs) {
				return this.SUSPEND;
			}

		},

		'step2s': {
			'in': function step2sIn(tagName, attrs) {

				switch(tagName) {
				case 'select':
					var len = this.selects.length;
					this.selects[len] = {};
					this.selects[len].attrs = attrs.attrs;
					this.selects[len].options = [];
					this.current = this.selects[len];
					break;
				case 'textarea':
					var len = this.textareas.length;
					this.textareas[len] = {};
					this.textareas[len].attrs = attrs.attrs;
					this.current = this.textareas[len];
					break;
				case 'button':
					var len = this.buttons.length;
					this.buttons[len] = {};
					this.buttons[len].attrs = attrs.attrs;
					this.current = this.buttons[len];
					break;
				}
				this.invisibleTree.addElement("label", {'style': 'display:none;'});
				attrs.set("disabled", "disabled");
				attrs.set("style", "display:none");
				attrs.set("class", "gcFormInvisible");
				if (this.formId) {
					attrs.set("form", this.formId);
				}
				this.invisibleTree.addElement(tagName, attrs);

				return this.CONTINUE;
			},

			'out': function step2sOut(tagName, attrs) {
				this.current = undefined;
				this.invisibleTree.endElement();
				this.invisibleTree.endElement();
				return this.OUT;
			},

			'option': function step2sStartOption(tagName, attrs) {
				this.current.options[this.current.options.length] = attrs.attrs;
				this.invisibleTree.addElement(tagName, attrs);
				return this.CONTINUE;
			},

			'/option': function step2sEndOption(tagName) {
				this.invisibleTree.endElement();
				return this.CONTINUE;
			},

			'#text': function step2sText(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');

				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}

				this.invisibleTree.addText(data);
				
				if (this.current.options && this.current.options[this.current.options.length - 1]) {
					this.current.options[this.current.options.length - 1].text = data;
				} else {
					this.current.text = data;
				}
				return this.CONTINUE;
			}

		},

		'finished' : {
			'/#doc' : function finishedEndDoc() {
				this.invisibleTree.endElement();
				if (this.autoComplete == 'true') {
					this.addTree(this.invisibleTree);
				}
				this.endElement();
				return this.EXIT;
			}
		},

		'failed' : {
			'/#doc' : function failedEndDoc() {
				return this.ERROR;
			}
		}
	};
	gcparts.partsRules[rule._name] =  rule;
}());
