/**
 * jQuery Mobile フォームパーツ
 */

(function() {
	var rule = {
		_name: "JQMPANEL01",
		_dic: "jQueryMobile パネルパーツ",

		start : {
			'#doc' : function step1StartDoc() {

				// パラメータの取得・初期化
				// フォームID
				this.formId = this.getParameter('formId') || '';
				// type属性
				this.type = this.getParameter('type') || 'none';
				// クリアボタン表示
				this.clearButton = this.getParameter('clearButton') || 'OFF';
				// ミニサイズ表示
				this.miniSize = this.getParameter('miniSize') || 'OFF';
				// インライン表示（ボタン）
				this.inline = this.getParameter('inline') || 'OFF';
				// エラーメッセージ部分の探し方
				this.errorTextType = this.getParameter('errorTextType') || '';
				// エラーメッセージ部分の指定値
				this.errorTextValue = this.getParameter('errorTextValue') || '';
				// ラベルIDの振り直し
				this.resetLabelId = this.getParameter('resetLabelId') || 'OFF';

				// バッファ変数
				// 見出し
				this.titleTree = new gcparts.ElementTree('titleTree');
				// コンテンツツリー
				this.contentTree = new gcparts.ElementTree('contents');

				// title / label / errorTextのどれかによって処理を違えるべく、step2sにサスペンド時に値を保持する
				this.itemName;

				// PC側のフォーム系要素の属性のうち、削除する属性定義（ブラックリスト）（item:start.removeAttrs）
				var _removes = this.getItem('start.removeAttrs');
				if (_removes) {
					this.removes = _removes.getAttribute('value').replace(/\s*/g, '').split(',');
				} else {
					this.removes = [];
				}
				this.removeAttrs = function(attrs) {
					if (!attrs) {
						return attrs;
					}
					this.removes.forEach(function(r){
						r = r.toLowerCase();
						if (attrs.has(r)) {
							attrs.remove(r);
						}
					});
					return attrs;
				}
				this.matchClassAttribute = function(classNames, target) {
					var _class = classNames.split(' ');
					for (var i = 0; i < _class.length; i++) {
						if (_class[i] == target) {
							// find class attribute value
							return true;
						}
					}
					return false;
				}
				return this.OUT;
			},
		},

		step1: {

			'input': function step1StartInput(tagName, attrs) {
				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();

				if (_type != 'radio' && _type != 'checkbox') {
					if (this.labelTree && !this.labelTree.isEmpty()) {
						this.contentTree.addTree(this.labelTree);
					}
					this.labelTree = undefined;
					return this.SUSPEND;
				}

				this.labelId = attrs.get('id');
				if (!this.labelId) {
					this.labelId = 'gcp_' + attrs.get('name') + '_' + attrs.get('value');
					attrs.set('id', this.labelId);
				} else if (this.resetLabelId == 'ON') {
					this.labelId = 'gcp_' + attrs.get('name') + '_' + attrs.get('value');
					attrs.set('id', this.labelId);
				}
				if (this.miniSize == 'ON') {
					// ミニサイズ表示
					attrs.set('data-mini', 'true');
				}
				if (this.formId) {
					// form属性の設定
					attrs.set('form', this.formId);
				}
				var _input = this.getItem('input');
				if (_input) {
					if (_input.hasAttribute('type')) {
						_input.removeAttribute('type');
					}
					_input.mergeAttribute(this.removeAttrs(attrs));
					this.contentTree.addElement(_input);
				} else {
					this.contentTree.addElement(tagName, this.removeAttrs(attrs));
				}

				return this.CONTINUE;

			},

			'/input' : function step1EndInput(tagName, attrs) {
				this.contentTree.endElement();
				// テキストノード待ち
				this.skip = true;
				return this.CONTINUE;
			},

			'#text': function step1Text(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}

				if (this.labelTree) {
					this.labelTree.addText(data);
				} else if (this.skip) {
					var _label = this.getItem('label');
					if (this.labelId) {
						_label.setAttribute('for', this.labelId);
						this.labelId = undefined;
					}
					this.contentTree.addElement(_label);
					this.contentTree.addText(data);
					this.contentTree.endElement();
					this.skip = false;
				}
				return this.CONTINUE;
			},

			'label': function step1StartLabel(tagName, attrs) {
				this.skip = true;
				this.labelTree = new gcparts.ElementTree('labelTree');
				if (attrs.has('for')) {
					this.labelTree.addElement(tagName, this.removeAttrs(attrs));
				}
				return this.CONTINUE;
			},

			'/label': function step1EndLabel(tagName, attrs) {

				if (this.labelTree && this.labelTree.hasElement(tagName)) {
					// label要素追加済み
					this.labelTree.endElement();
					this.contentTree.addTree(this.labelTree);
				} else if (this.labelTree && !this.labelTree.isEmpty()) {
					var _label = this.getItem('label');
					if (this.labelId) {
						_label.setAttribute('for', this.labelId);
						this.labelId = undefined;
					}
					this.contentTree.addElement(_label);
					this.contentTree.addTree(this.labelTree);
					this.contentTree.endElement();
				}
				this.skip = false;
				this.labelTree = undefined;
				return this.CONTINUE;
			},

			'select | button | textarea | keygen' : function step1StartOtherFormComponents(tagName, attrs) {
				if (this.labelTree && !this.labelTree.isEmpty()) {
					this.contentTree.addTree(this.labelTree);
				}
				this.skip = false;
				this.labelTree = undefined;
				return this.SUSPEND;
			},

			'script | noscript' : function step1StartScript(tagName, attrs) {
				if (this.labelTree && !this.labelTree.isEmpty()) {
					this.contentTree.addTree(this.labelTree);
				}
				this.skip = false;
				this.labelTree = undefined;
				return this.TRASH;
			},

			'*': function step1StartOther(tagName, attrs) {
				if (this.skip) {
					//gclog.debug(tagName + ' in label process ...CONTINUE');
					return this.CONTINUE;
				}
				if (this.labelTree && !this.labelTree.isEmpty()) {
					this.contentTree.addTree(this.labelTree);
				}
				this.labelTree = undefined;
				return this.SUSPEND;
			},

			'/*': function step1EndOther(tagName) {
				if (this.skip) {
					//gclog.debug(tagName + ' in label process ...CONTINUE');
					return this.CONTINUE;
				}
				this.contentTree.endElement();
				return this.CONTINUE;
			}

		},

		step2s: {

			'*': function step2sStartOther(tagName, attrs) {
				var gclabel = attrs.get("data-gc-label");
				switch(gclabel) {
				case 'title':
					// 見出し用のラベルパスまたは固定文言の指定
					this.itemName = 'title';
					return this.SUSPEND;
				}
				// エラーメッセージ部分の指定がされている場合は、SUSPEND
				if (this.errorTextValue) {
					switch(this.errorTextType) {
					case '1':
						// data-gc-label属性指定
						if (this.errorTextValue == gclabel) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '2':
						// id属性指定
						if (this.errorTextValue == attrs.get('id')) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '3':
						// class属性指定
						if (this.matchClassAttribute(attrs.get('class'), this.errorTextValue)) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '4':
						// 要素名指定
						if (this.errorTextValue.toLowerCase() == tagName) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					}
				}
				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.contentTree.addElement(step);
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.contentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'/*': function step2sEndOther(tagName) {
				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.contentTree.endElement();
					break;
				}
				return this.CONTINUE;
			},

			'#text': function step2sText(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}
				this.contentTree.addText(data);
				return this.CONTINUE;
			},

			'input': function step2sStartInput(tagName, attrs) {
				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();

				if (_type == 'hidden') {
					// hiddenは出力から除外する
					return this.TRASH;
				}
				if (_type == 'radio' || _type == 'checkbox') {
					return this.OUT_RESEND;
				}
				if (this.inline == 'ON') {
					switch(_type) {
					case 'button':
					case 'submit':
					case 'reset':
					case 'image':
						// ボタンのインライン表示
						attrs.set('data-inline', 'true');
						break;
					}
				}
				if (this.type != 'none' && _type == 'text') {
					// type属性の変更（元がtextの場合）
					if (this.type == 'number' && gcutil.isGroup('ios')) {
						attrs.set('type', 'text');
						attrs.set('pattern', '[0-9]*');
					} else {
						attrs.set('type', this.type);
					}
				}
				if (this.clearButton == 'ON') {
					// クリアボタン
					attrs.set('data-clear-btn', 'true');
				}
				if (this.miniSize == 'ON') {
					// ミニサイズ表示
					attrs.set('data-mini', 'true');
				}
				if (this.formId) {
					// form属性の設定
					attrs.set('form', this.formId);
				}
				var _input = this.getItem('input');
				if (_input) {
					if (_input.hasAttribute('type')) {
						_input.removeAttribute('type');
					}
					_input.mergeAttribute(this.removeAttrs(attrs));
					this.contentTree.addElement(_input);
				} else {
					this.contentTree.addElement(tagName, this.removeAttrs(attrs));
				}
				return this.CONTINUE;
			},

			'/input': function step2sEndInput(tagName) {
				this.contentTree.endElement();
				return this.CONTINUE;
			},

			'select | button | textarea | keygen' : function step2sStartOtherFormComponents(tagName, attrs) {
				if (this.miniSize == 'ON') {
					// ミニサイズ表示
					attrs.set('data-mini', 'true');
				}
				if (this.inline == 'ON') {
					switch(tagName) {
					case 'button':
						// インライン表示
						attrs.set('data-inline', 'true');
						break;
					}
				}
				if (this.formId) {
					// form属性の設定
					attrs.set('form', this.formId);
				}
				var _formComp = this.getItem(tagName);
				if (_formComp) {
					_formComp.mergeAttribute(this.removeAttrs(attrs));
					this.contentTree.addElement(_formComp);
				} else {
					this.contentTree.addElement(tagName, this.removeAttrs(attrs));
				}
				return this.CONTINUE;
			},

			'/select | /button | /textarea | /keygen' : function step2sEndOtherFormComponents(tagName, attrs) {
				this.contentTree.endElement();
				return this.CONTINUE;
			},

			'label': function step2sStartLabel(tagName, attrs) {
				return this.OUT_RESEND;
			},

			'script | noscript' : function step2sStartScript(tagName, attrs) {
				return this.TRASH;
			}

		},


		step3s: {

			'in': function step3sIn(tagName, attrs) {

				if (this.itemName == 'title') {
					this.currentTree = this.titleTree;
				} else {
					this.currentTree = this.contentTree;
					this.currentTree.addElement(this.getItem(this.itemName));
				}
				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.currentTree.addElement(step);
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'out': function step3sOut(tagName) {
				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.endElement();
					break;
				}
				if (this.itemName != 'title') {
					this.currentTree.endElement();
				}
				return this.OUT;
			},

			'#text': function step3sText(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');

				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}
				this.currentTree.addText(data);
				return this.CONTINUE;
			},

			'input': function step3sStartInput(tagName, attrs) {
				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();

				if (_type == 'hidden') {
					// hiddenは出力から除外する
					return this.TRASH;
				}
				if (_type == 'radio' || _type == 'checkbox') {
					return this.OUT_RESEND;
				}
				if (this.type != 'none' && _type == 'text') {
					attrs.set('type', this.type);
				}
				if (this.clearButton == 'ON') {
					attrs.set('data-clear-btn', 'true');
				}
				if (this.miniSize == 'ON') {
					// ミニサイズ表示
					attrs.set('data-mini', 'true');
				}
				if (this.inline == 'ON') {
					switch(_type) {
					case 'button':
					case 'submit':
					case 'reset':
					case 'image':
						// ボタンのインライン表示
						attrs.set('data-inline', 'true');
						break;
					}
				}
				if (this.formId) {
					// form属性の設定
					attrs.set('form', this.formId);
				}
				var _input = this.getItem('input');
				if (_input) {
					_input.removeAttribute("type");
					_input.mergeAttribute(this.removeAttrs(attrs));
					this.contentTree.addElement(_input);
				} else {
					this.contentTree.addElement(tagName, this.removeAttrs(attrs));
				}
				return this.CONTINUE;
			},

			'/input': function step3sEndInput(tagName, attrs) {
				this.currentTree.endElement();
				return this.CONTINUE;
			},

			'label' : function step3sStartLabel(tagName, attrs) {
				return this.OUT_RESEND;
			},

			'select | textarea | button | keygen': function step3sStartOtherFormComponents(tagName, attrs) {
				if (this.miniSize == 'ON') {
					// ミニサイズ表示
					attrs.set('data-mini', 'true');
				}
				if (this.inline == 'ON') {
					switch(tagName) {
					case 'button':
						// ボタンのインライン表示
						attrs.set('data-inline', 'true');
						break;
					}
				}
				if (this.formId) {
					// form属性の設定
					attrs.set('form', this.formId);
				}
				var _formComp = this.getItem(tagName);
				if (_formComp) {
					_formComp.mergeAttribute(this.removeAttrs(attrs));
					this.contentTree.addElement(_formComp);
				} else {
					this.contentTree.addElement(tagName, this.removeAttrs(attrs));
				}
				return this.CONTINUE;
			},

			'/select | /textarea | /button | /keygen': function step3sEndOtherFormComponents(tagName) {
				this.currentTree.endElement();
				return this.CONTINUE;
			},

			'*': function step3sStartOther(tagName, attrs) {
				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'/*': function step3sEndOther(tagName) {
				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.endElement();
					break;
				}
				return this.CONTINUE;
			},

			'script | noscript' : function step3sStartScript(tagName, attrs) {
				return this.TRASH;
			}

		},


		finished : {
			'/#doc' : function finishedEndDoc() {

				if (this.titleTree.isEmpty() &&
						this.contentTree.isEmpty()) {
					// ツリーに出力がなければ外枠も出力しない
					return this.EXIT;
				}

				var _wrapper = this.getItem('wrapper');
				var _title = this.getItem('title');

				this.addElement(_wrapper);

				if (this.titleTree.isEmpty()) {
					var _dataRole = _wrapper.getAttribute('data-role') || '';
					var _titleText = '';
					switch(_dataRole){
					case 'fieldcontain':
					//case 'controlgroup':
						_wrapper.setAttribute('class', 'ui-hide-label');
						break;
					}
					_title.setAttribute('class', 'ui-hidden-accessible');
					_titleText = '';

					this.addElement(_title);
					this.addText(_titleText);
					this.endElement();
				} else {
					this.addElement(_title);
					this.addTree(this.titleTree);
					this.endElement();
				}

				var _contents = this.getItem("contents");
				if (_contents) {
					this.addElement(_contents);
				}

				if (!this.contentTree.isEmpty()) {
					this.addTree(this.contentTree);
				}

				if (_contents) {
					this.endElement();
				}

				this.endElement();	// closer wrapper element

				return this.EXIT;
			}
		},

		failed : {
			'/#doc' : function failedEndDoc() {
				return this.ERROR;
			}
		}
	};
	gcparts.partsRules[rule._name] =  rule;
}());
