/**
 * jQuery Mobile フォームパーツ
 */

(function() {
	var rule = {
		_name: "JQMRADIO01",
		_dic: "jQueryMobile ラジオボタンパーツ",

		start : {
			'#doc' : function step1StartDoc() {

				// パラメータの取得・初期化
				// フォームID
				this.formId = this.getParameter('formId') || '';

				// 入力HTMLに含まれるフォームコンポーネント以外のコンテンツを出力するかどうか
				this.panelMode = this.getParameter('panelMode') || 'OFF';

				// 注釈
				this.footerTexts = [];
				for (var i = 0; i < 100; i++) {
					this.footerTexts[i] = this.getParameter('footerText' + (i + 1)) || '';
				}
				// ラジオグループ出現カウンタ
				this.radioGroupIndex = 0;

				// ミニサイズ表示
				this.miniSize = this.getParameter('miniSize') || 'OFF';
				// 入力の必須化
				this.required = this.getParameter('required') || 'OFF';
				// エラーメッセージ部分の探し方
				this.errorTextType = this.getParameter('errorTextType') || '';
				// エラーメッセージ部分の指定値
				this.errorTextValue = this.getParameter('errorTextValue') || '';
				// ラベルIDの振り直し
				this.resetLabelId = this.getParameter('resetLabelId') || 'OFF';
				// ラジオボタンのグループ連結
				this.grouping = this.getParameter('grouping') || 'OFF';

				this.groupItem = this.getItem('group');
				if (this.grouping == 'OFF') {
					// 横並びする場合はグルーピングさせる
					if (this.groupItem.getAttribute('data-type') == 'horizontal') {
						this.grouping = 'name';
					}
				}

				// バッファ変数
				// 見出し
				this.titleTree = new gcparts.ElementTree('titleTree');
				// ラジオコンポーネントツリー
				this.radioTree = new gcparts.ElementTree('radioTree');

				// 注釈用ツリー
				this.contentTree = new gcparts.ElementTree('contentTree');
				this.footerTree = new gcparts.ElementTree('footerTree');

				// title / label / errorTextのどれかによって処理を違えるべく、step2sにサスペンド時に値を保持する
				this.itemName;

				// PC側のフォーム系要素の属性のうち、削除する属性定義（ブラックリスト）（item:start.removeAttrs）
				var _removes = this.getItem('start.removeAttrs');
				if (_removes) {
					this.removes = _removes.getAttribute('value').replace(/\s*/g, '').split(',');
				} else {
					this.removes = [];
				}
				this.removeAttrs = function(attrs) {
					if (!attrs) {
						return attrs;
					}
					this.removes.forEach(function(r){
						r = r.toLowerCase();
						if (attrs.has(r)) {
							attrs.remove(r);
						}
					});
					return attrs;
				}
				this.matchClassAttribute = function(classNames, target) {
					var _class = classNames.split(' ');
					for (var i = 0; i < _class.length; i++) {
						if (_class[i] == target) {
							// find class attribute value
							return true;
						}
					}
					return false;
				}
				return this.OUT;
			},
		},

		step1: {

			'input': function step1StartInput(tagName, attrs) {
				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();

				if (_type != 'radio') {

					if (this.labelTree && !this.labelTree.isEmpty()) {
						if (this.radioTree.isEmpty()) {
							this.contentTree.addTree(this.labelTree);
						} else {
							if (this.panelMode != 'OFF') {
								this.footerTree.addTree(this.labelTree);
							}
						}
					}
					this.labelTree = undefined;
					return this.SUSPEND;
				}

				switch(this.grouping) {
				case 'force':
					// 強制で全てのラジオをグルーピングする
					break;
				default:
					if (this.radioTree.radioName && this.radioTree.radioName != attrs.get('name')) {
						var _group = (this.grouping == 'OFF') ? undefined : this.groupItem;
						if (_group) {
							this.contentTree.addElement(_group);
							this.contentTree.addTree(this.radioTree);
							this.contentTree.endElement();
						} else {
							this.contentTree.addTree(this.radioTree);
						}

						if(this.panelMode == 'OFF') {
							// 注釈の表示
							if(this.radioGroupIndex < this.footerTexts.length && this.footerTexts[this.radioGroupIndex]) {
								var _note = this.getItem('note');
								if (_note) {
									this.contentTree.addElement(_note);
									this.contentTree.addText(this.footerTexts[this.radioGroupIndex]);
									this.contentTree.endElement();
								} else {
									this.contentTree.addText(this.footerTexts[this.radioGroupIndex]);
								}
							}
						}

						if (!this.footerTree.isEmpty()) {
							this.contentTree.addTree(this.footerTree);
						}

						this.radioGroupIndex++;
						this.radioTree = new gcparts.ElementTree('radioTree');
						this.footerTree = new gcparts.ElementTree('footerTree');
					}
					break;
				}

				this.labelId = attrs.get('id');
				if (!this.labelId) {
					this.labelId = 'gcp_' + attrs.get('name') + '_' + attrs.get('value');
					attrs.set('id', this.labelId);
				} else if (this.resetLabelId == 'ON') {
					this.labelId = 'gcp_' + attrs.get('name') + '_' + attrs.get('value');
					attrs.set('id', this.labelId);
				}
				if (this.miniSize == 'ON') {
					// ミニサイズ表示
					attrs.set('data-mini', 'true');
				}
				if (this.required == 'ON') {
					// 入力の必須化
					attrs.set('required', 'required');
				}
				if (this.formId) {
					// form属性の設定
					attrs.set('form', this.formId);
				}

				var _radio = this.getItem('radio');
				var _item = this.getItem('item');
				if (_item) {
					this.radioTree.addElement(_item);
				}

				if (_radio) {
					_radio.mergeAttribute(this.removeAttrs(attrs));
					this.radioTree.addElement(_radio);
				} else {
					this.radioTree.addElement(tagName, this.removeAttrs(attrs));
				}

				this.radioTree.radioName = attrs.get('name');

				return this.CONTINUE;

			},

			'/input' : function step1EndInput(tagName, attrs) {
				this.radioTree.endElement();
				// テキストノード待ち
				this.skip = true;
				return this.CONTINUE;
			},

			'#text': function step1Text(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}

				if (this.labelTree) {
					this.labelTree.addText(data);
				} else if (this.skip) {
					var _label = this.getItem('label');
					if (this.labelId) {
						_label.setAttribute('for', this.labelId);
						this.labelId = undefined;
					}

					this.radioTree.addElement(_label);
					this.radioTree.addText(data);
					this.radioTree.endElement();

					var _item = this.getItem('item');
					if (_item) {
						this.radioTree.endElement();
					}

					this.skip = false;
				} else if (this.panelMode != 'OFF') {
					if (this.radioTree.isEmpty()) {
						// ラジオ出現前のコンテンツ等
						this.contentTree.addText(data);
					} else {
						// ラジオ出現後のコンテンツ
						this.footerTree.addText(data);
					}
				}
				return this.CONTINUE;
			},

			'label': function step1StartLabel(tagName, attrs) {
				this.skip = true;
				this.labelTree = new gcparts.ElementTree('labelTree');

				if (attrs.has('for')) {
					this.labelTree.addElement(tagName, this.removeAttrs(attrs));
				}
				return this.CONTINUE;
			},

			'/label': function step1EndLabel(tagName, attrs) {

				var _item = this.getItem('item');
				if (this.labelTree && this.labelTree.hasElement(tagName)) {
					// label要素追加済み
					this.labelTree.endElement();
					this.radioTree.addTree(this.labelTree);

					if (_item) {
						this.radioTree.endElement();
					}

				} else if (this.labelTree && !this.labelTree.isEmpty()) {
					var _label = this.getItem('label');
					if (this.labelId) {
						_label.setAttribute('for', this.labelId);
						this.labelId = undefined;
					}
					this.radioTree.addElement(_label);
					this.radioTree.addTree(this.labelTree);
					this.radioTree.endElement();

					if (_item) {
						this.radioTree.endElement();
					}

				}
				this.skip = false;
				this.labelTree = undefined;
				return this.CONTINUE;
			},

			'*': function step1StartOther(tagName, attrs) {
				if (this.skip) {
					return this.CONTINUE;
				}
				if (this.panelMode != 'OFF' && this.labelTree && !this.labelTree.isEmpty()) {
					if (this.radioTree.isEmpty()) {
						// ラジオ出現前のコンテンツ等
						this.contentTree.addTree(this.labelTree);
					} else {
						// ラジオ出現後のコンテンツ
						this.footerTree.addTree(this.labelTree);
					}
				}
				this.labelTree = undefined;
				return this.SUSPEND;
			},

			'/*': function step1EndOther(tagName) {
				if (this.skip) {
					return this.CONTINUE;
				}
				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}
				if (this.radioTree.isEmpty()) {
					this.contentTree.endElement();
				} else {
					this.footerTree.endElement();
				}
				return this.CONTINUE;
			}

		},

		step2s: {

			'*': function step2sStartOther(tagName, attrs) {
				var gclabel = attrs.get("data-gc-label");
				switch(gclabel) {
				case 'title':
					// 見出し用のラベルパスまたは固定文言の指定
					this.itemName = 'title';
					return this.SUSPEND;
				}
				// エラーメッセージ部分の指定がされている場合は、SUSPEND
				if (this.errorTextValue) {
					switch(this.errorTextType) {
					case '1':
						// data-gc-label属性指定
						if (this.errorTextValue == gclabel) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '2':
						// id属性指定
						if (this.errorTextValue == attrs.get('id')) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '3':
						// class属性指定
						if (this.matchClassAttribute(attrs.get('class'), this.errorTextValue)) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '4':
						// 要素名指定
						if (this.errorTextValue.toLowerCase() == tagName) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					}
				}
				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					if (this.radioTree.isEmpty()) {
						this.contentTree.addElement(step);
					} else {
						this.footerTree.addElement(step);
					}
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					if (this.radioTree.isEmpty()) {
						this.contentTree.addElement(tagName, this.removeAttrs(attrs));
					} else {
						this.footerTree.addElement(tagName, this.removeAttrs(attrs));
					}
					break;
				}
				return this.CONTINUE;
			},

			'/*': function step2sEndOther(tagName) {

				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					if (this.radioTree.isEmpty()) {
						this.contentTree.endElement();
					} else {
						this.footerTree.endElement();
					}
					break;
				}
				return this.CONTINUE;
			},

			'#text': function step2sText(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}

				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				if (this.radioTree.isEmpty()) {
					this.contentTree.addText(data);
				} else {
					this.footerTree.addText(data);
				}
				return this.CONTINUE;
			},

			'input': function step2sStartInput(tagName, attrs) {
				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();

				if (_type == 'radio') {
					return this.OUT_RESEND;
				}

				return this.TRASH;
			},

			'select | button | textarea | keygen' : function step2sStartOtherFormComponents(tagName, attrs) {
				return this.TRASH;
			},

			'label': function step2sStartLabel(tagName, attrs) {
				return this.OUT_RESEND;
			},

			'script | noscript' : function step2sStartScript(tagName, attrs) {
				return this.TRASH;
			}

		},

		step3s: {
			'in': function step3sIn(tagName, attrs) {

				if (this.itemName == 'title') {
					this.currentTree = this.titleTree;
				} else {
					if (this.radioTree.isEmpty()) {
						this.currentTree = this.contentTree;
					} else {
						this.currentTree = this.footerTree;
					}
					this.currentTree.addElement(this.getItem(this.itemName));
				}
				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.currentTree.addElement(step);
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'out': function step3sOut(tagName) {
				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.endElement();
					break;
				}
				if (this.itemName != 'title') {
					this.currentTree.endElement();
				}
				return this.OUT;
			},

			'#text': function step3sText(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');

				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}
				this.currentTree.addText(data);
				return this.CONTINUE;
			},

			'input': function step3sStartInput(tagName, attrs) {
				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();

				if (_type == 'radio') {
					return this.OUT_RESEND;
				}
				return this.TRASH;
			},

			'label' : function step3sStartLabel(tagName, attrs) {
				return this.OUT_RESEND;
			},

			'select | button | textarea | keygen' : function step3sStartOtherFormComponents(tagName, attrs) {
				return this.TRASH;
			},

			'*': function step3sStartOther(tagName, attrs) {
				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'/*': function step3sEndOther(tagName) {
				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.endElement();
					break;
				}
				return this.CONTINUE;
			},

			'script | noscript' : function step3sStartScript(tagName, attrs) {
				return this.TRASH;
			}
		},

		finished: {

			'/#doc' : function finishedEndDoc() {

				if (this.titleTree.isEmpty() &&
						this.contentTree.isEmpty() &&
						this.radioTree.isEmpty() &&
						this.footerTree.isEmpty()) {
					// ツリーに出力がなければ外枠も出力しない
					return this.EXIT;
				}

				var _wrapper = this.getItem('wrapper');
				var _title = this.getItem('title');

				this.addElement(_wrapper);
				if (this.titleTree.isEmpty()) {
					var _dataRole = _wrapper.getAttribute('data-role') || '';
					var _titleText = '';
					switch(_dataRole){
					case 'fieldcontain':
						_wrapper.setAttribute('class', 'ui-hide-label');
						break;
					}
					_title.setAttribute('class', 'ui-hidden-accessible');
					_titleText = '';

					this.addElement(_title);
					this.addText(_titleText);
					this.endElement();

				} else {
					this.addElement(_title);
					this.addTree(this.titleTree);
					this.endElement();
				}

				var _contents = this.getItem("contents");
				if (_contents) {
					this.addElement(_contents);
				}

				if (!this.contentTree.isEmpty()) {
					this.addTree(this.contentTree);
				}

				if (!this.radioTree.isEmpty()) {
					var _group;
					if (this.grouping != 'OFF') {
						_group  = this.groupItem;
					}
					if (_group) {
						this.addElement(_group);
						this.addTree(this.radioTree);
						this.endElement();
					} else {
						this.addTree(this.radioTree);
					}

					if(this.panelMode == 'OFF') {
						// 注釈の表示
						if(this.radioGroupIndex < this.footerTexts.length && this.footerTexts[this.radioGroupIndex]) {
							var _note = this.getItem('note');
							if (_note) {
								this.addElement(_note);
								this.addText(this.footerTexts[this.radioGroupIndex]);
								this.endElement();
							} else {
								this.addText(this.footerTexts[this.radioGroupIndex]);
							}
						}
					}
				}

				if (_contents) {
					this.endElement();
				}

				if (!this.footerTree.isEmpty()) {
					this.addElement(this.getItem('note'));
					this.addTree(this.footerTree);
					this.endElement();
				}
				this.endElement();	// closer wrapper element

				return this.EXIT;
			}

		},

		failed : {
			'/#doc' : function failedEndDoc() {
				return this.ERROR;
			}
		}
	};
	gcparts.partsRules[rule._name] =  rule;
}());
