/**
 * jQuery Mobile フォームパーツ
 */

(function() {
	var rule = {
		_name: "JQMSELECT01",
		_dic: "jQueryMobile セレクトボックスパーツ",

		start : {
			'#doc' : function step1StartDoc() {

				// パラメータの取得・初期化
				// フォームID
				this.formId = this.getParameter('formId') || '';

				// 入力HTMLに含まれるフォームコンポーネント以外のコンテンツを出力するかどうか
				this.panelMode = this.getParameter('panelMode') || 'OFF';

				// 注釈
				this.footerTexts = [];
				for (var i = 0; i < 100; i++) {
					this.footerTexts[i] = this.getParameter('footerText' + (i + 1)) || '';
				}
				// セレクトボックス出現カウンタ
				this.selectGroupIndex = 0;

				// ミニサイズ表示
				this.miniSize = this.getParameter('miniSize') || 'OFF';
				// インライン表示
				this.inline = this.getParameter('inline') || 'OFF';
				// 入力の必須化
				this.required = this.getParameter('required') || 'OFF';
				// メニュータイプ
				this.menuType = this.getParameter('menuType') || 'OFF';
				// エラーメッセージ部分の探し方
				this.errorTextType = this.getParameter('errorTextType') || '';
				// エラーメッセージ部分の指定値
				this.errorTextValue = this.getParameter('errorTextValue') || '';

				// セレクトボックスのグルーピング
				this.grouping = this.getParameter('grouping') || 'OFF';
				this.groupItem = this.getItem('group');
				if (this.grouping == 'OFF') {
					// 横並びする場合はグルーピングさせる
					if (this.groupItem.getAttribute('data-type') == 'horizontal') {
						// セレクトボックスの場合、radio,checkboxと違ってnameが一緒のケースはないので、強制連結モードにする
						this.grouping = 'force';
					}
				}

				// バッファ変数
				// 見出し
				this.titleTree = new gcparts.ElementTree('titleTree');
				// コンテンツツリー
				this.contentTree = new gcparts.ElementTree('contents');
				// セレクトボックスツリー
				this.selectTree = new gcparts.ElementTree('selectTree');

				// 注釈エリア用ツリー
				this.footerTree = new gcparts.ElementTree('footerTree');

				// title / errorTextのどれかによって処理を違えるべく、step2sにサスペンド時に値を保持する
				this.itemName;

				// PC側のフォーム系要素の属性のうち、削除する属性定義（ブラックリスト）（item:start.removeAttrs）
				var _removes = this.getItem('start.removeAttrs');
				if (_removes) {
					this.removes = _removes.getAttribute('value').replace(/\s*/g, '').split(',');
				} else {
					this.removes = [];
				}
				this.removeAttrs = function(attrs) {
					if (!attrs) {
						return attrs;
					}
					this.removes.forEach(function(r){
						r = r.toLowerCase();
						if (attrs.has(r)) {
							attrs.remove(r);
						}
					});
					return attrs;
				}
				this.matchClassAttribute = function(classNames, target) {
					var _class = classNames.split(' ');
					for (var i = 0; i < _class.length; i++) {
						if (_class[i] == target) {
							// find class attribute value
							return true;
						}
					}
					return false;
				}
				return this.OUT;
			},
		},

		step1: {

			'select': function step1StartSelect(tagName, attrs) {

				switch(this.grouping) {
				case 'force':
					// 強制で全てのセレクトボックスをグルーピングする
					break;
				default:
					if (this.selectTree.selectName && this.selectTree.selectName != attrs.get('name')) {

						var _group = (this.grouping == 'OFF') ? undefined : this.groupItem;
						if (_group) {
							this.contentTree.addElement(_group);
							this.contentTree.addTree(this.selectTree);
							this.contentTree.endElement();
						} else {
							this.contentTree.addTree(this.selectTree);
						}

						if(this.panelMode == 'OFF') {
							// 注釈の表示
							if(this.selectGroupIndex < this.footerTexts.length && this.footerTexts[this.selectGroupIndex]) {
								var _note = this.getItem('note');
								if (_note) {
									this.contentTree.addElement(_note);
									this.contentTree.addText(this.footerTexts[this.selectGroupIndex]);
									this.contentTree.endElement();
								} else {
									this.contentTree.addText(this.footerTexts[this.selectGroupIndex]);
								}
							}
						}

						if (!this.footerTree.isEmpty()) {
							this.contentTree.addTree(this.footerTree);
						}

						this.selectGroupIndex++;
						this.selectTree = new gcparts.ElementTree('selectTree');
						this.footerTree = new gcparts.ElementTree('footerTree');
					}
					break;
				}

				if (this.miniSize == 'ON') {
					// ミニサイズ表示
					attrs.set('data-mini', 'true');
				}
				if (this.required == 'ON') {
					// 入力の必須化
					attrs.set('required', 'required');
				}
				if (this.inline == 'ON') {
					// インライン表示
					attrs.set('data-inline', 'true');
				}
				if (this.menuType == 'JQM') {
					// JQM独自メニュー
					attrs.set('data-native-menu', 'false');
				}
				if (this.formId) {
					// form属性の設定
					attrs.set('form', this.formId);
				}

				var _item = this.getItem('item');
				if (_item) {
					this.selectTree.addElement(_item);
				}

				var _select = this.getItem('select');
				if (_select) {
					_select.mergeAttribute(this.removeAttrs(attrs));
					this.selectTree.addElement(_select);
				} else {
					this.selectTree.addElement(tagName, this.removeAttrs(attrs));
				}
				this.selectTree.selectName = attrs.get('name');

				if (this.menuType == 'JQM') {
					// JQM独自メニューのときは、セレクトボックスの先頭にプレースホルダ用のoption要素を追加する
					var _placeholder = this.getItem('placeholder');
					if (_placeholder) {
						var _t = _placeholder.getAttribute('title') || '選択';
						this.selectTree.addElement(_placeholder);
						this.selectTree.addText(_t);
						this.selectTree.endElement();
					}
				}

				this.skip = true;
				return this.CONTINUE;
			},

			'/select': function step1EndSelect(tagName) {

				if (!this.selectTree.isEmpty()) {
					this.selectTree.endElement();
				}
				var _item = this.getItem('item');
				if (_item) {
					this.selectTree.endElement();
				}

				this.skip = false;
				return this.CONTINUE;
			},

			'option': function step1StartOption(tagName, attrs) {
				this.selectTree.addElement(tagName, this.removeAttrs(attrs));
				return this.CONTINUE;
			},

			'/option': function step1EndOption(tagName) {
				this.selectTree.endElement();
				return this.CONTINUE;
			},

			'#text': function step1Text(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}
				if (this.skip) {
					this.selectTree.addText(data);
				} else {

					if (this.panelMode == 'OFF') {
						// パネルモードOFF
						return this.CONTINUE;
					}
					if (this.selectTree.isEmpty()) {
						this.contentTree.addText(data);
					} else {
						this.footerTree.addText(data);
					}
				}
				return this.CONTINUE;
			},

			'*': function step1StartOther(tagName, attrs) {
				if (this.skip) {
					return this.CONTINUE;
				}
				return this.SUSPEND;
			},

			'/*': function step1EndOther(tagName) {
				if (this.skip) {
					return this.CONTINUE;
				}
				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				if (this.selectTree.isEmpty()) {
					this.contentTree.endElement();
				} else {
					this.footerTree.endElement();
				}
				return this.CONTINUE;
			}

		},

		step2s: {

			'*': function step2sStartOther(tagName, attrs) {
				var gclabel = attrs.get("data-gc-label");
				switch(gclabel) {
				case 'title':
					// 見出し用のラベルパスまたは固定文言の指定
					this.itemName = 'title';
					return this.SUSPEND;
				}
				// エラーメッセージ部分の指定がされている場合は、SUSPEND
				if (this.errorTextValue) {
					switch(this.errorTextType) {
					case '1':
						// data-gc-label属性指定
						if (this.errorTextValue == gclabel) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '2':
						// id属性指定
						if (this.errorTextValue == attrs.get('id')) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '3':
						// class属性指定
						if (this.matchClassAttribute(attrs.get('class'), this.errorTextValue)) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '4':
						// 要素名指定
						if (this.errorTextValue.toLowerCase() == tagName) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					}
				}
				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					if (this.selectTree.isEmpty()) {
						this.contentTree.addElement(step);
					} else {
						this.footerTree.addElement(step);
					}
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					if (this.selectTree.isEmpty()) {
						this.contentTree.addElement(tagName, this.removeAttrs(attrs));
					} else {
						this.footerTree.addElement(tagName, this.removeAttrs(attrs));
					}
					break;
				}
				return this.CONTINUE;
			},

			'/*': function step2sEndOther(tagName) {

				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					if (this.selectTree.isEmpty()) {
						this.contentTree.endElement();
					} else {
						this.footerTree.endElement();
					}
					break;
				}
				return this.CONTINUE;
			},

			'#text': function step2sText(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}

				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				if (this.selectTree.isEmpty()) {
					this.contentTree.addText(data);
				} else {
					this.footerTree.addText(data);
				}
				return this.CONTINUE;
			},

			'select' : function step2sStartSelect(tagName, attrs) {
				return this.OUT_RESEND;
			},

			'input | button | textarea | keygen' : function step2sStartOtherFormComponents(tagName, attrs) {
				return this.TRASH;
			},

			'script | noscript' : function step2sStartScript(tagName, attrs) {
				return this.TRASH;
			}
		},


		step3s: {

			'in': function step3sIn(tagName, attrs) {

				if (this.itemName == 'title') {
					this.currentTree = this.titleTree;
				} else {
					if (this.selectTree.isEmpty()) {
						this.currentTree = this.contentTree;
					} else {
						this.currentTree = this.footerTree;
					}
					this.currentTree.addElement(this.getItem(this.itemName));
				}
				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.currentTree.addElement(step);
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'out': function step3sOut(tagName) {
				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.endElement();
					break;
				}
				if (this.itemName != 'title') {
					this.currentTree.endElement();
				}
				return this.OUT;
			},

			'#text': function step3sText(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');

				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}
				this.currentTree.addText(data);
				return this.CONTINUE;
			},

			'select': function step3sStartSelect(tagName, attrs) {
				return this.OUT_RESEND;
			},

			'button | textarea | keygen' : function step3sEndOtherFormComponents(tagName, attrs) {
				return this.TRASH;
			},

			'*': function step3sStartOther(tagName, attrs) {
				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'/*': function step3sEndOther(tagName) {
				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.endElement();
					break;
				}
				return this.CONTINUE;
			},

			'script | noscript' : function step3sStartScript(tagName, attrs) {
				return this.TRASH;
			}

		},


		finished : {
			'/#doc' : function finishedEndDoc() {

				if (this.titleTree.isEmpty() &&
						this.selectTree.isEmpty() &&
						this.contentTree.isEmpty() &&
						this.footerTree.isEmpty()) {
					// ツリーに出力がなければ外枠も出力しない
					return this.EXIT;
				}

				var _wrapper = this.getItem('wrapper');
				var _title = this.getItem('title');

				this.addElement(_wrapper);

				if (this.titleTree.isEmpty()) {
					var _dataRole = _wrapper.getAttribute('data-role') || '';
					var _titleText = '';
					switch(_dataRole){
					case 'fieldcontain':
					//case 'controlgroup':
						_wrapper.setAttribute('class', 'ui-hide-label');
						break;
					}
					_title.setAttribute('class', 'ui-hidden-accessible');
					_titleText = '';

					this.addElement(_title);
					this.addText(_titleText);
					this.endElement();
				} else {
					this.addElement(_title);
					this.addTree(this.titleTree);
					this.endElement();
				}

				var _contents = this.getItem("contents");
				if (_contents) {
					this.addElement(_contents);
				}

				if (!this.contentTree.isEmpty()) {
					this.addTree(this.contentTree);
				}

				if (!this.selectTree.isEmpty()) {

					var _group;
					if (this.grouping != 'OFF') {
						_group  = this.groupItem;
					}
					if (_group) {
						this.addElement(_group);
						this.addTree(this.selectTree);
						this.endElement();
					} else {
						this.addTree(this.selectTree);
					}

					if(this.panelMode == 'OFF') {
						// 注釈の表示
						if(this.selectGroupIndex < this.footerTexts.length && this.footerTexts[this.selectGroupIndex]) {
							var _note = this.getItem('note');
							if (_note) {
								this.addElement(_note);
								this.addText(this.footerTexts[this.selectGroupIndex]);
								this.endElement();
							} else {
								this.addText(this.footerTexts[this.selectGroupIndex]);
							}
						}
					}
				}

				if (_contents) {
					this.endElement();
				}

				if (!this.footerTree.isEmpty()) {
					this.addElement(this.getItem('note'));
					this.addTree(this.footerTree);
					this.endElement();
				}

				this.endElement();	// closer wrapper element

				return this.EXIT;
			}
		},

		failed : {
			'/#doc' : function failedEndDoc() {
				return this.ERROR;
			}
		}
	};
	gcparts.partsRules[rule._name] =  rule;
}());
