/**
 * jQuery Mobile フォームパーツ
 */

(function() {
	var rule = {
		_name: "JQMTEXT01",
		_dic: "jQueryMobile テキストボックスパーツ",

		start : {
			'#doc' : function step1StartDoc() {

				// パラメータの取得・初期化
				// フォームID
				this.formId = this.getParameter('formId') || '';

				// 入力HTMLに含まれるフォームコンポーネント以外のコンテンツを出力するかどうか
				this.panelMode = this.getParameter('panelMode') || 'OFF';

				// 注釈
				this.footerTexts = [];
				// type属性
				this.types = [];
				// プレースホルダ
				this.placeholders = [];
				for (var i = 0; i < 100; i++) {
					this.footerTexts[i] = this.getParameter('footerText' + (i + 1)) || '';
					this.types[i] = this.getParameter('type' + (i + 1)) || 'none';
					this.placeholders[i] = this.getParameter('placeholder' + (i + 1)) || '';
				}
				// テキストボックス出現カウンタ
				this.textIndex = 0;

				// クリアボタン表示
				this.clearButton = this.getParameter('clearButton') || 'OFF';
				// 入力の必須化
				this.required = this.getParameter('required') || 'OFF';
				// ミニサイズ表示
				this.miniSize = this.getParameter('miniSize') || 'OFF';
				// エラーメッセージ部分の探し方
				this.errorTextType = this.getParameter('errorTextType') || '';
				// エラーメッセージ部分の指定値
				this.errorTextValue = this.getParameter('errorTextValue') || '';

				// バッファ変数
				// 見出し
				this.titleTree = new gcparts.ElementTree('titleTree');

				// コンテンツツリー
				this.contentTree = new gcparts.ElementTree('contents');

				// title / label / errorTextのどれかによって処理を違えるべく、step2sにサスペンド時に値を保持する
				this.itemName;

				// PC側のフォーム系要素の属性のうち、削除する属性定義（ブラックリスト）（item:start.removeAttrs）
				var _removes = this.getItem('start.removeAttrs');
				if (_removes) {
					this.removes = _removes.getAttribute('value').replace(/\s*/g, '').split(',');
				} else {
					this.removes = [];
				}
				this.removeAttrs = function(attrs) {
					if (!attrs) {
						return attrs;
					}
					this.removes.forEach(function(r){
						r = r.toLowerCase();
						if (attrs.has(r)) {
							attrs.remove(r);
						}
					});
					return attrs;
				}
				this.matchClassAttribute = function(classNames, target) {
					var _class = classNames.split(' ');
					for (var i = 0; i < _class.length; i++) {
						if (_class[i] == target) {
							// find class attribute value
							return true;
						}
					}
					return false;
				}
				return this.OUT;
			},
		},

		step1: {

			'input': function step1StartInput(tagName, attrs) {
				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();

				switch(_type) {
				case 'button':
				case 'submit':
				case 'reset':
				case 'image':
				case 'hidden':
				case 'radio':
				case 'checkbox':
				case 'file':
					return this.OUT_RESEND;
				}

				if (this.textIndex < this.types.length && this.types[this.textIndex] != 'none') {
					// type属性の変更（元がtextの場合）
					if (_type == 'text') {
						if (this.types[this.textIndex] == 'number' && gcutil.isGroup('ios')) {
							attrs.set('type', 'text');
							attrs.set('pattern', '[0-9]*');
						} else {
							attrs.set('type', this.types[this.textIndex]);
						}
					}
				}
				if (this.textIndex < this.placeholders.length && this.placeholders[this.textIndex]) {
					// プレースホルダ
					attrs.set('placeholder', this.placeholders[this.textIndex]);
				}
				if (this.clearButton == 'ON') {
					// クリアボタン
					attrs.set('data-clear-btn', 'true');
				}
				if (this.miniSize == 'ON') {
					// ミニサイズ表示
					attrs.set('data-mini', 'true');
				}
				if (this.required == 'ON') {
					// 入力の必須化
					attrs.set('required', 'required');
				}
				if (this.formId) {
					// form属性の設定
					attrs.set('form', this.formId);
				}

				var _textbox = this.getItem('textbox');
				var _item = this.getItem('item');
				if (_item && this.panelMode == 'OFF') {
					this.contentTree.addElement(_item);
				}

				if (_textbox) {
					_textbox.mergeAttribute(this.removeAttrs(attrs));
					this.contentTree.addElement(_textbox);
				} else {
					this.contentTree.addElement(tagName, this.removeAttrs(attrs));
				}

				this.contentTree.endElement();

				// 注釈テキストの表示
				if (this.panelMode == 'OFF') {
					if (this.textIndex < this.footerTexts.length && this.footerTexts[this.textIndex]) {
						var _note = this.getItem('note');
						if (_note) {
							this.contentTree.addElement(_note);
							this.contentTree.addText(this.footerTexts[this.textIndex]);
							this.contentTree.endElement();
						} else {
							this.contentTree.addText(this.footerTexts[this.textIndex]);
						}
					}
					if (_item) {
						this.contentTree.endElement();
					}
				}
				this.textIndex++;

				return this.CONTINUE;
			},

			'/input' : function step1EndInput(tagName) {
				return this.CONTINUE;
			},

			'#text': function step1Text(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}
				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}
				this.contentTree.addText(data);
				return this.CONTINUE;
			},

			'*': function step1StartOther(tagName, attrs) {
				return this.SUSPEND;
			},

			'/*': function step1EndOther(tagName) {
				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}
				this.contentTree.endElement();
				return this.CONTINUE;
			}

		},

		step2s: {

			'*': function step2sStartOther(tagName, attrs) {
				var gclabel = attrs.get("data-gc-label");
				switch(gclabel) {
				case 'title':
					// 見出し用のラベルパスまたは固定文言の指定
					this.itemName = 'title';
					return this.SUSPEND;
				}
				// エラーメッセージ部分の指定がされている場合は、SUSPEND
				if (this.errorTextValue) {
					switch(this.errorTextType) {
					case '1':
						// data-gc-label属性指定
						if (this.errorTextValue == gclabel) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '2':
						// id属性指定
						if (this.errorTextValue == attrs.get('id')) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '3':
						// class属性指定
						if (this.matchClassAttribute(attrs.get('class'), this.errorTextValue)) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					case '4':
						// 要素名指定
						if (this.errorTextValue.toLowerCase() == tagName) {
							this.itemName = 'errorText';
							return this.SUSPEND;
						}
						break;
					}
				}

				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.contentTree.addElement(step);
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.contentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'/*': function step2sEndOther(tagName) {

				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}

				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.contentTree.endElement();
					break;
				}
				return this.CONTINUE;
			},

			'#text': function step2sText(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}
				if (this.panelMode == 'OFF') {
					// パネルモードOFF
					return this.CONTINUE;
				}
				this.contentTree.addText(data);
				return this.CONTINUE;
			},

			'input': function step2sStartInput(tagName, attrs) {
				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();

				switch(_type) {
				case 'button':
				case 'submit':
				case 'reset':
				case 'image':
				case 'hidden':
				case 'radio':
				case 'checkbox':
				case 'file':
					// 出力から除外する
					return this.TRASH;
				default:
					return this.OUT_RESEND;
				}
			},

			'select | button | textarea | keygen' : function step2sStartOtherFormComponents(tagName, attrs) {
				return this.TRASH;
			},

			'script | noscript' : function step2sStartScript(tagName, attrs) {
				return this.TRASH;
			}

		},


		step3s: {

			'in': function step3sIn(tagName, attrs) {

				if (this.itemName == 'title') {
					this.currentTree = this.titleTree;
				} else {
					this.currentTree = this.contentTree;
					this.currentTree.addElement(this.getItem(this.itemName));
				}
				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.currentTree.addElement(step);
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'out': function step3sOut(tagName) {
				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.endElement();
					break;
				}
				if (this.itemName != 'title') {
					this.currentTree.endElement();
				}
				return this.OUT;
			},

			'#text': function step3sText(data) {
				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');

				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}
				this.currentTree.addText(data);
				return this.CONTINUE;
			},

			'input': function step3sStartInput(tagName, attrs) {
				var _type = attrs.get('type') || '';
				_type = _type.toLowerCase();

				switch(_type) {
				case 'button':
				case 'submit':
				case 'reset':
				case 'image':
				case 'hidden':
				case 'radio':
				case 'checkbox':
				case 'file':
					// 出力から除外する
					return this.TRASH;
				default:
					return this.OUT_RESEND;
				}
			},

			'select | textarea | button | keygen': function step3sStartOtherFormComponents(tagName, attrs) {
				return this.TRASH;
			},

			'*': function step3sStartOther(tagName, attrs) {
				switch(tagName) {
				case 'th':
				case 'td':
				case 'li':
				case 'dt':
				case 'dd':
					// 要素を変えて無害化する
					var step = this.getStep();
					step.mergeAttribute(this.removeAttrs(attrs));
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.addElement(tagName, this.removeAttrs(attrs));
					break;
				}
				return this.CONTINUE;
			},

			'/*': function step3sEndOther(tagName) {
				switch(tagName) {
				case 'form':
				case 'table':
				case 'tbody':
				case 'tr':
				case 'ul':
				case 'dl':
					// この要素自体は無視する
					break;
				default:
					this.currentTree.endElement();
					break;
				}
				return this.CONTINUE;
			},

			'script | noscript' : function step3sStartScript(tagName, attrs) {
				return this.TRASH;
			}

		},


		finished : {
			'/#doc' : function finishedEndDoc() {

				if (this.titleTree.isEmpty() &&
						this.contentTree.isEmpty()) {
					// ツリーに出力がなければ外枠も出力しない
					return this.EXIT;
				}

				var _wrapper = this.getItem('wrapper');
				var _title = this.getItem('title');

				this.addElement(_wrapper);

				if (this.titleTree.isEmpty()) {
					var _dataRole = _wrapper.getAttribute('data-role') || '';
					var _titleText = '';
					switch(_dataRole){
					case 'fieldcontain':
					//case 'controlgroup':
						_wrapper.setAttribute('class', 'ui-hide-label');
						break;
					}
					_title.setAttribute('class', 'ui-hidden-accessible');
					_titleText = '';

					this.addElement(_title);
					this.addText(_titleText);
					this.endElement();
				} else {
					this.addElement(_title);
					this.addTree(this.titleTree);
					this.endElement();
				}

				var _contents = this.getItem("contents");
				if (_contents) {
					this.addElement(_contents);
				}
				if (!this.contentTree.isEmpty()) {
					this.addTree(this.contentTree);
				}
				if (_contents) {
					this.endElement();
				}

				this.endElement();	// closer wrapper element

				return this.EXIT;
			}
		},

		failed : {
			'/#doc' : function failedEndDoc() {
				return this.ERROR;
			}
		}
	};
	gcparts.partsRules[rule._name] =  rule;
}());
