/**
 * 汎用コンポーネント テーブルフォーム
 */
(function() {


	var rule = {

		_name: "TABLEFORM02",
		_dic: "テーブルフォーム",

		start: {
			'#doc': function startStartDoc() {

				// PC側のフォーム系要素の属性のうち、削除する属性定義（ブラックリスト）（item:start.removeAttrs）
				var _removes = this.getItem('start.removeAttrs');
				if (_removes) {
					this.removes = _removes.getAttribute('value').replace(/\s*/g, '').split(',');
				} else {
					this.removes = [];
				}

				this.removeAttrs = function(attrs) {
					if (!attrs) {
						return attrs;
					}
					this.removes.forEach(function(r){
						r = r.toLowerCase();
						if (attrs.has(r)) {
							attrs.remove(r);
						}
					});
					return attrs;
				}
				this.cellCount = 0;

				this.convertImageToSubmit = this.getParameter('convertImageToSubmit') || 'OFF';

				return this.OUT;
			}
		},

		step1: {

			'form' : function step1StartForm(tagName, attrs) {
				this.addElement({ 'tagName' : tagName, 'attrs' : this.removeAttrs(attrs) });
				this.addElement(this.getStep());
				this.formProcess = true;
				return this.CONTINUE;
			},
			'/form' : function step1EndForm(tagName) {
				this.endElement();
				this.endElement();
				this.formProcess = false;
				return this.CONTINUE;
			},
			'tr' : function step1StartTr(tagName, attrs) {
				this.cellCount = 0;
				return this.CONTINUE;
			},
			'/tr' : function step1EndTr(tagName) {
				return this.CONTINUE;
			},

			'th | td | dd': function step1StartThTdDd(tagName, attrs) {
				var itemName;
				if (this.cellCount == 0) {
					itemName = 'step1.header';
				} else {
					itemName = 'step1.field';
				}
				var item = this.getItem(itemName);
				this.addElement(item);
				return this.CONTINUE;
			},

			'dt': function step1StartDt(tagName, attrs) {
				this.cellCount = 0;
				var itemName = 'step1.header';
				var item = this.getItem(itemName);
				this.addElement(item);
				return this.CONTINUE;
			},

			'div | p': function step1StartDivP(tagName, attrs) {
				// フォーム要素内のdiv,pはddにする
				if (this.formProcess) {
					this.cellCount = 0;
					var itemName = 'step1.field';
					var item = this.getItem(itemName);
					this.addElement(item);
				}
				return this.CONTINUE;
			},

			'/th | /td | /dd | /dt': function step1EndCell(tagName) {
				this.cellCount++;
				this.endElement();
				return this.CONTINUE;
			},

			'/div | /p': function step1EndDivP(tagName) {
				if (this.formProcess) {
					this.cellCount++;
					this.endElement();
				}
				return this.CONTINUE;
			},

			'a' : function step1StartA(tagName, attrs) {
				this.addElement({ 'tagName' : tagName, 'attrs' : this.removeAttrs(attrs) });
				return this.CONTINUE;
			},

			'/a' : function step1EndA(tagName) {
				this.endElement();
				return this.CONTINUE;
			},

			'label' : function step1StartImg(tagName, attrs) {
				this.addElement({ 'tagName' : tagName, 'attrs' : this.removeAttrs(attrs) });
				return this.CONTINUE;
			},

			'/label' : function step1StartImg(tagName, attrs) {
				this.endElement();
				return this.CONTINUE;
			},

			'img' : function step1StartImg(tagName, attrs) {
				this.addElement({ 'tagName' : tagName, 'attrs' : this.removeAttrs(attrs) });
				this.endElement();
				return this.CONTINUE;
			},

			'#text' : function step1Text(data) {

				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}

				var elem = this.getItem('step1.text');
				this.addElement(elem);
				this.addText(data);
				this.endElement();

				return this.CONTINUE;
			},

			'input' : function step1Input(tagName, attrs) {

				if (this.convertImageToSubmit == 'ON') {
					var type = attrs.get('type');
					if (type == 'image' || type == 'IMAGE') {
						// type='image'をtype='submit'に変更する
						attrs.set('type', 'submit');
						var alt = attrs.get('alt');
						if (alt) {
							attrs.set('value', alt);
						}
					}
				}
				this.addElement({ 'tagName' : tagName, 'attrs' : this.removeAttrs(attrs) });
				return this.CONTINUE;
			},

			'/input' : function step1EndInput(tagName) {
				this.endElement();
				return this.CONTINUE;
			},

			'keygen' : function step1StartKeygen(tagName, attrs) {
				this.addElement({ 'tagName' : tagName, 'attrs' : this.removeAttrs(attrs) });
				return this.CONTINUE;
			},

			'/keygen' : function step1EndKeygen(tagName, attrs) {
				this.endElement();
				return this.CONTINUE;
			},

			'button' : function step1StartButton(tagName, attrs) {
				this.addElement({ 'tagName' : tagName, 'attrs' : this.removeAttrs(attrs) });
				return this.CONTINUE;
			},

			'/button' : function step1EndButton(tagName) {
				this.endElement();
				return this.CONTINUE;
			},

			'script' : function step1StartScript(tagName, attrs) {
				return this.SUSPEND;
			},

			'select | datalist | textarea' : function step1StartSelectTextarea(tagName, attrs) {
				return this.SUSPEND;
			},

		},

		step2s: {

			'in' : function step2sIn(tagName, attrs) {

				switch(tagName) {
				case 'select':
				case 'datalist':
				case 'textarea':
					this.addElement({ 'tagName' : tagName, 'attrs' : this.removeAttrs(attrs) });
					return this.CONTINUE;

				case 'script':
					this.addElement({ 'tagName' : tagName, 'attrs' : this.removeAttrs(attrs) });
					return this.CONTINUE;

				default:
					return this.STAY;
				}
			},

			'out': function step2sOut(tagName) {
				this.endElement();
				return this.OUT;
			},

			'optgroup' : function step2sStartOptgroup(tagName, attrs) {
				this.addElement({ 'tagName' : tagName, 'attrs' : this.removeAttrs(attrs) });
				return this.CONTINUE;
			},

			'/optgroup' : function step2sEndOptgroup(tagName) {
				this.endElement();
				return this.CONTINUE;
			},

			'option' : function step2sStartOption(tagName, attrs) {
				this.addElement({ 'tagName' : tagName, 'attrs' : this.removeAttrs(attrs) });
				return this.CONTINUE;
			},

			'/option' : function step2sEndOption(tagName) {
				this.endElement();
				return this.CONTINUE;
			},

			'#text' : function step2sText(data) {

				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (!d.match(/^[ \r\n\t\f]*$/)) {
					this.addText(data);
				}

				return this.CONTINUE;
			},
		},

		finished: {
			'/#doc': function finishedStartDoc() {
				return this.EXIT;
			}
		},

		failed: {
			'/#doc': function failedEndDoc() {
				return this.EXIT;
			}
		}
	};

	gcparts.partsRules[rule._name] = rule;

}());
