/**
 * 汎用コンポーネント テキストリスト
 */
(function() {

	var rule = {

		_name: "TEXTLIST02",
		_dic: "テキストリスト",

		start: {
			'#doc': function startStartDoc() {

				// オプション値の取得と正規化
				this.enableTruncate = this.getParameter('enableTruncate');
				if (this.enableTruncate === 'ON') {
					this.enableTruncate = true;
				} else {
					this.enableTruncate = false;
				}

				this.displayCount = this.getParameter('displayCount');

				this.anchorIndex = this.getParameter('anchorIndex');
				if (!this.displayCount) {
					this.displayCount = 0;
				}
				if (!this.anchorIndex) {
					this.anchorIndex = 0;
				}

				// フラグ変数の初期化
				this.isAddUl = false;
				this.gclabelTagNames = [];

				// リスト件数
				this.listCount = 0;

				// パーツID
				this.partsId = 'gcp' + gcutil.randomString(5, '_-');

				// TEXTMENU01と同じ属性定義を導入
				// PC側のアンカー属性のうち、出力に取り入れる属性定義（item:start.attrs）
				this.bringAttrs = {};
				var baList = ['href'];
				var item = this.getItem('start.attrs');
				if (item) {
					baList = item.getAttribute('value').replace(/\s*/g, '').split(',');
				}
				for (var i = 0; i < baList.length; i++) {
					// 大文字の属性名に対応
					var ba = baList[i].toLowerCase();
					this.bringAttrs[ba] = ba;
				}

				// アンカーの属性マージ関数
				this.mergeAttrs = function(element, attrs) {
					for (var ar in attrs.attrs) {
						if (this.bringAttrs[ar]) {
							var value = attrs.get(ar);
							if (value) {
								element.setAttribute(ar, value);
							}
						}
					}
					return element;
				};

				// クライアントサイドJavaScriptの出力
				var script = this.getItem('start.readmore');
				if (script) {
					var code = script.getElement().textContent;
					if (code) {
						this.addElement(script);
						code = code.split("#gcpId").join("#" + this.partsId);
						this.addText(code);
						this.endElement();
					}
				}

				return this.OUT;
			}
		},

		step1: {

			'li': function step1StartLi(tagName, attrs) {

				// 最初の１回だけULをツリーに出力
				if (!this.isAddUl) {
					this.addElement(this.getStep());
					this.isAddUl = true;
				}

				return this.SUSPEND;
			},

			'dt': function step1StartDt(targName, attrs) {

				// 最初の１回だけULをツリーに出力
				if (!this.isAddUl) {
					this.addElement(this.getStep());
					this.isAddUl = true;
				}

				return this.OUT_RESEND;
			},

			'/#doc': function step1EndDoc() {

				// ヘルパーに溜まっているデータを出力する
				if (this.list && this.list.getItemCount() > 0) {
					this.addBuilder(this.list);
					this.listCount++;
					this.list = undefined;
				}
				if (this.isAddUl) {
					this.endElement();
				}
				return this.OUT;
			}

		},

		step2s: {

			'li': function step2sStartLi(tagName, attrs) {

				var elem = this.getStep();
				if (this.displayCount > 0 && this.listCount >= this.displayCount) {
					// 表示上限を超えたら不可視リストで作成
					elem.setAttribute('style', 'display:none;');
				}

				this.list = new gcparts.ListMakerTwo(elem, this.anchorIndex);
				return this.CONTINUE;

			},

			'/li': function step2sEndLi(tagName) {

				if (this.list.getItemCount() > 0) {
					this.listCount++;
				}

				// リストアイテムヘルパーが出力するツリーを追加する
				this.addBuilder(this.list);
				this.list = undefined;
				return this.OUT;

			},

			'dt': function step2sStartDt(tagName, attrs) {

				// ヘルパーに溜まっているデータを出力する
				if (this.list && this.list.getItemCount() > 0) {
					this.addBuilder(this.list);
					this.list = undefined;
					this.listCount++;
				}

				var elem = this.getStep();
				if (this.displayCount > 0 && this.listCount >= this.displayCount) {
					// 表示上限を超えたら不可視リストで作成
					elem.setAttribute('style', 'display:none;');
				}
				this.list = new gcparts.ListMakerTwo(elem, this.anchorIndex);
				return this.CONTINUE;

			},

			'ul': function step2sStartUl(tagName, attrs) {

				if (this.list.getItemCount() > 0) {
					this.listCount++;
				}

				// リストアイテムヘルパーが出力するツリーを追加する
				this.addBuilder(this.list);
				this.list = undefined;
				return this.OUT;
			},

			'a': function step2sStartA(tagName, attrs) {
				if (!this.enableTruncate || this.displayCount == 0 || this.listCount < this.displayCount) {
					var item = this.getItem('step2s.anchor');
					this.list.setAnchorElement(this.mergeAttrs(item, attrs));
				}
				return this.CONTINUE;
			},

			'img': function step2sStartImg(tagName, attrs) {
				if (!this.enableTruncate || this.displayCount == 0 || this.listCount < this.displayCount) {
					var item = this.getItem('step2s.image');
					item.setAttribute('src', attrs.get('src'));
					this.list.addItem(item);
				}
				return this.CONTINUE;
			},

			'/img': function step2sEndImg(tagName) {
				if (!this.enableTruncate || this.displayCount == 0 || this.listCount < this.displayCount) {
					this.list.closeItem();
				}
				return this.CONTINUE;
			},

			'del': function step2sStartDel(tagName, attrs) {
				if (!this.enableTruncate || this.displayCount == 0 || this.listCount < this.displayCount) {
					var item = this.getItem('step2s.del');
					this.list.addItem(item);
				}
				return this.CONTINUE;
			},

			's': function step2sStartS(tagName, attrs) {
				if (!this.enableTruncate || this.displayCount == 0 || this.listCount < this.displayCount) {
					var item = this.getItem('step2s.s');
					this.list.addItem(item);
				}
				return this.CONTINUE;
			},

			'strike': function step2sStartStrike(tagName, attrs) {
				if (!this.enableTruncate || this.displayCount == 0 || this.listCount < this.displayCount) {
					var item = this.getItem('step2s.strike');
					this.list.addItem(item);
				}
				return this.CONTINUE;
			},

			'/del|/s|/strike': function step2sEndDel(tagName) {
				if (!this.enableTruncate || this.displayCount == 0 || this.listCount < this.displayCount) {
					this.list.closeItem();
				}
				return this.CONTINUE;
			},

			'#text': function step2sText(data) {
				if (!this.enableTruncate || this.displayCount == 0 || this.listCount < this.displayCount) {
					var nbsp = String.fromCharCode( 160 );
					var d = data.replace(nbsp,'');

					if (!d.match(/^[ \r\n\t\f]*$/)) {
						var item = this.getItem('step2s.text');
						this.list.addItem(item, data);
						this.list.closeItem(item);
					}
				}
				return this.CONTINUE;
			},

			'script | style' : function step2sStartScriptStyle(tagName, attrs) {
				return this.TRASH;
			},

			'*' : function step2sStartOther(tagName, attrs) {
				if (!this.enableTruncate || this.displayCount == 0 || this.listCount < this.displayCount) {
					var gclabel = attrs.get("data-gc-label");
					if (gclabel) {
						// 終了タグ判定用
						this.gclabelTagNames.push(tagName);
						var item = this.getItem('step2s.other');
						if (item.hasAttribute("class")) {
							gclabel += " " + item.getAttribute("class");
						}
						item.setAttribute('class', gclabel);
						this.list.addItem(item);
					}
				}
				return this.CONTINUE;
			},

			'/*': function step2sEndOther(tagName) {
				if (!this.enableTruncate || this.displayCount == 0 || this.listCount < this.displayCount) {
					if (this.gclabelTagNames.length > 0 && tagName == this.gclabelTagNames[this.gclabelTagNames.length - 1]) {
						this.gclabelTagNames.pop();
						this.list.closeItem();
					}
				}
				return this.CONTINUE;
			},

			'/#doc': function step2sEndDoc() {

				// ヘルパーに溜まっているデータを出力する
				if (this.list && this.list.getItemCount() > 0) {
					this.addBuilder(this.list);
					this.listCount++;
					this.list = undefined;
				}
				if (this.isAddUl) {
					this.endElement();
				}
				return this.OUT;
			}
		},

		finished: {
			'/#doc': function finishedStartDoc() {

				if (this.countAddedStep('step2s') == 0) {
					return this.ERROR;
				}

				if (this.displayCount > 0 && this.listCount > this.displayCount && !this.enableTruncate) {
				// 表示上限を超えたらもっと見るを作成
					var more = this.getItem('finished.more');
					var count = this.getItem('finished.count');
					if (more) {
						more.setAttribute('id', this.partsId);
						var title = more.getAttribute('title') || 'more';

						this.addElement(more);
						this.addText(title);

						this.addElement(count);
						this.addText(this.displayCount);
						this.endElement();
						this.endElement();
					}
				}

				return this.EXIT;
			}
		},

		failed: {
			'/#doc': function failedEndDoc() {
				return this.EXIT;
			}
		}
	};

	gcparts.partsRules[rule._name] = rule;

}());
