/**
 * 汎用コンポーネント テーブル列削除
 */
(function() {


	var rule = {

		_name: "TRIMTABLE01",
		_dic: "列削除テーブル",

		start: {
			'#doc': function startStartDoc() {

				// オプション値の取得と正規化
				var _colNumbers = this.getParameter('colNumbers');
				if (!_colNumbers) {
					this.colNumbers = [];
				} else {
					this.colNumbers = _colNumbers.replace(/\s*/g, '').split(',');
				}
				// 削除セルリスト
				this.deleteCols = [];

				this.removeAttrs = {};
				var removes = [];
				var item = this.getItem('start.remove');
				if (item) {
					removes = item.getAttribute('value').replace(/\s*/g, '').split(',');
				}
				for (var i = 0; i < removes.length; i++) {
					/* v1.1.0 大文字の属性名に対応 */
					var r = removes[i].toLowerCase();
					this.removeAttrs[r] = r;
				}

				// 属性削除関数
				this.trim = function(attrs) {
					for (var ar in attrs.attrs) {
						if (this.removeAttrs[ar]) {
							attrs.remove(ar);
						}
					}
					return attrs;
				};

				// パーツID
				this.partsId = 'gcp' + gcutil.randomString(5, '_-');

				// テーブルの深さカウンタ
				this.level = 0;
				// 列カウンタ
				this.cellCount = 0;
				// 1行目のセルデータバッファ
				this.headerTds = [];

				return this.OUT;

			}
		},

		step1: {

			'tr': function step1StartTr(tagName, attrs) {

				if (this.table) {

					// 次の1行を追加
					if (gclog.isDebugEnabled()) {
						gclog.debug('insert new row =' + this.table.getRow());
					}

					this.currentTr = this.table.insertRow(this.table.getRow());
					this.currentTd = this.currentTr.td(0);

					if (!this.currentTd.isEnabled()) {
						this.currentTd = this.currentTd.right(true);
					}

				}
				return this.CONTINUE;
			},

			'/tr': function step1EndTr(tagName) {

				if (!this.table) {

					if (gclog.isDebugEnabled()) {
						gclog.debug('create new TableMaker cellCount=' + this.cellCount, 2);
					}

					if (this.cellCount <= 0) {
						gclog.error('table has no cell. cellCount=' + this.cellCount, 2);
						return this.ERROR;
					}

					this.table = new gcparts.TableMaker(1, this.cellCount);
					this.currentTr = this.table.tr(0);
					var colNo = 0
					this.currentTd = this.currentTr.td(colNo);

					for (var i = 0; i < this.headerTds.length; i++) {
						var type = this.headerTds[i]['type'];
						switch (type) {
							case 'gcparts.Td':

								if (colNo > 0) {
									this.currentTd = this.currentTd.right(true);
								}
								var _rs = this.headerTds[i]['rowspan'];
								var _cs = this.headerTds[i]['colspan'];
								var _head = this.headerTds[i]['head'];
								if (_rs > 1) {
									this.currentTd.rowspan(_rs);
								}
								if (_cs > 1) {
									this.currentTd.colspan(_cs);
								}
								this.currentTd.assignHead(_head);

								colNo++;
								break;
							case 'addElement':
								this.currentTd.addElement(this.headerTds[i]['value']);
								break;
							case 'endElement':
								this.currentTd.endElement();
								break;
							case 'addText':
								this.currentTd.addText(this.headerTds[i]['value']);
								break;
							default:
						}
					}
				}

				this.cellCount = 0;
				return this.CONTINUE;

			},

			'td | th' : function step1StartTdTh(tagName, attrs) {

				var cspan = attrs.get('colspan') || 1;
				var rspan = attrs.get('rowspan') || 1;
				cspan = cspan - 0;
				rspan = rspan - 0;

				if (this.currentTd) {
					if (cspan > 1) {
						this.currentTd.colspan(cspan);
					}
					if (rspan > 1) {
						this.currentTd.rowspan(rspan);
					}
					if (tagName == 'th') {
						this.currentTd.assignHead(true);
					}
				} else {
					// いったん配列に貯める
					var idx = this.headerTds.length;
					this.headerTds[idx] = {'type' : 'gcparts.Td', 'colspan' : cspan, 'rowspan': rspan, 'head' : (tagName == 'th') };

					var _del = false;
					for (var i = 0; i < this.colNumbers.length; i++) {
						if (this.cellCount == this.colNumbers[i]) {
							_del = true;
							break;
						}
					}
					this.deleteCols[this.cellCount] = _del;
					for(var i = 1; i < cspan; i++) {
						this.deleteCols[this.cellCount + i] = _del;
					}
					this.cellCount += (cspan * 1);
				}
				return this.CONTINUE;
			},

			'/td | /th' : function step1StartTdTh(tagName, attrs) {

				if (this.currentTd) {
					// 次のセルに移動
					this.currentTd = this.currentTd.right(true);
				}
				return this.CONTINUE;
			},

			'#text' : function step1StartText(data) {

				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}

				var item = this.getItem('step1.text');
				if (this.currentTd) {
					this.currentTd.addElement(item);
					this.currentTd.addText(data);
					this.currentTd.endElement();
				} else {
					var idx = this.headerTds.length;
					this.headerTds[idx] = {'type' : 'addElement', 'value' : item };
					this.headerTds[idx+1] = {'type' : 'addText', 'value' : data };
					this.headerTds[idx+2] = {'type' : 'endElement' };
				}
				return this.CONTINUE;
			},

			'a' : function step1StartA(tagName, attrs) {
				var elem = {'tagName' : tagName, 'attrs' : attrs };
				if (this.currentTd) {
					this.currentTd.addElement(elem);
				} else {
					var idx = this.headerTds.length;
					this.headerTds[idx] = {'type' : 'addElement', 'value' : elem };
				}
				return this.CONTINUE;
			},

			'/a' : function step1EndA(tagName) {
				if (this.currentTd) {
					this.currentTd.endElement();
				} else {
					var idx = this.headerTds.length;
					this.headerTds[idx] = {'type' : 'endElement' };
				}
				return this.CONTINUE;
			},

			'img' : function step1StartImg(tagName, attrs) {

				var item = this.getItem('step1.image');
				item.setAttribute('src', attrs.get('src'));

				if (this.currentTd) {
					this.currentTd.addElement(item);
					this.currentTd.endElement();
				} else {
					var idx = this.headerTds.length;
					this.headerTds[idx] = {'type' : 'addElement', 'value' : item };
					this.headerTds[idx+1] = {'type' : 'endElement' };
				}
				return this.CONTINUE;
			},

			'tbody' : function step1StartTbody(tagName, attrs) {

				this.level = 1;
				return this.CONTINUE;
			},

			'table' : function step1StartTable(tagName, attrs) {

				if (this.level == 0) {
					return this.CONTINUE;
				} else {
					// テーブル中に出現した子テーブルはセルデータとして情報を入れたいのでサスペンド
					return this.SUSPEND;
				}
			},

			'/tbody' : function step1EndTbody(tagName) {

				if (this.table) {
					var idx = 0;
					for (var i = 0; i < this.deleteCols.length; i++) {
						if (this.deleteCols[i]) {
							if (gclog.isDebugEnabled()) {
								gclog.debug('delete col [' + i + '] actually index is[' + idx + ']');
							}
							this.table.deleteCol(idx);
						} else {
							idx++;
						}
					}
					this.addBuilder(this.table);
				}

				// テーブルの深さカウンタ
				this.level = 0;
				// 列カウンタ
				this.cellCount = 0;
				// 1行目のセルデータバッファ
				this.headerTds = [];

				this.table = undefined;
				this.currentTd = undefined;

				return this.CONTINUE;
			},

			'script' : function step1StartScript(tagName, attrs) {
				return this.TRASH;
			}
		},

		step2s: {

			'in' : function step2sIn(tagName, attrs) {
				switch(tagName) {
					case 'table':
						var step = this.getStep();
						var elem = {'tagName' : tagName, 'attrs' : this.trim(attrs) };

						if (this.currentTd) {
							this.currentTd.addElement(step);
							this.currentTd.addElement(elem);
						} else {
							var idx = this.headerTds.length;
							this.headerTds[idx] = {'type' : 'addElement', 'value' : step };
							this.headerTds[idx+1] = {'type' : 'addElement', 'value' : elem };
						}

						return this.CONTINUE;
					default:
						return this.STAY;
				}
			},

			'out' : function step2sOut(tagName) {

				if (this.currentTd) {
					this.currentTd.endElement();
				} else {
					var idx = this.headerTds.length;
					this.headerTds[idx] = {'type' : 'endElement' };
				}
				return this.OUT;
			},

			'table': function step2sStartTable(tagName, attrs) {
				var elem = {'tagName' : tagName, 'attrs' : this.trim(attrs) };
				if (this.currentTd) {
					this.currentTd.addElement(elem);
				} else {
					var idx = this.headerTds.length;
					this.headerTds[idx] = { 'type' : 'addElement', 'value' : elem };
				}
				return this.CONTINUE;
			},


			'/table': function step2sEndTable(tagName) {
				if (this.currentTd) {
					this.currentTd.endElement();
				} else {
					var idx = this.headerTds.length;
					this.headerTds[idx] = {'type' : 'endElement' };
				}
				return this.CONTINUE;
			},


			'tr | th | td' : function step2sStartThThTd(tagName, attrs) {
				var elem = {'tagName' : tagName, 'attrs' : this.trim(attrs) };
				if (this.currentTd) {
					this.currentTd.addElement(elem);
				} else {
					var idx = this.headerTds.length;
					this.headerTds[idx] = {'type' : 'addElement', 'value' : elem };
				}
				return this.CONTINUE;
			},

			'/tr | /th | /td' : function step2sEndTrThTd(tagName) {
				if (this.currentTd) {
					this.currentTd.endElement();
				} else {
					var idx = this.headerTds.length;
					this.headerTds[idx] = {'type' : 'endElement' };
				}
				return this.CONTINUE;
			},

			'#text' : function step2sText(data) {

				var nbsp = String.fromCharCode( 160 );
				var d = data.replace(nbsp,'');
				if (d.match(/^[ \r\n\t\f]*$/)) {
					return this.CONTINUE;
				}

				var item = this.getItem('step2s.text');
				if (this.currentTd) {
					this.currentTd.addElement(item);
					this.currentTd.addText(data);
					this.currentTd.endElement();
				} else {
					var idx = this.headerTds.length;
					this.headerTds[idx] = {'type' : 'addElement', 'value' : item };
					this.headerTds[idx+1] = {'type' : 'addText', 'value' : data };
					this.headerTds[idx+2] = {'type' : 'endElement' };
				}
				return this.CONTINUE;
			},

			'a' : function step2sStartA(tagName, attrs) {

				var elem = {'tagName' : tagName, 'attrs' : attrs };
				if (this.currentTd) {
					this.currentTd.addElement(elem);
				} else {
					var idx = this.headerTds.length;
					this.headerTds[idx] = {'type' : 'addElement', 'value' : elem };
				}
				return this.CONTINUE;
			},

			'/a' : function step2sEndA(tagName) {
				if (this.currentTd) {
					this.currentTd.endElement();
				} else {
					var idx = this.headerTds.length;
					this.headerTds[idx] = {'type' : 'endElement' };
				}

				return this.CONTINUE;
			},

			'img' : function step2sStartImg(tagName, attrs) {
				var item = this.getItem('step2s.image');
				item.setAttribute('src', attrs.get('src'));

				if (this.currentTd) {
					this.currentTd.addElement(item);
					this.currentTd.endElement();
				} else {
					var idx = this.headerTds.length;
					this.headerTds[idx] = {'type' : 'addElement', 'value' : item };
					this.headerTds[idx + 1] = {'type' : 'endElement' };
				}
				return this.CONTINUE;
			},

			'script' : function step2sStartScript(tagName, attrs) {
				return this.TRASH;
			}
		},

		finished: {
			'/#doc': function finishedStartDoc() {

				return this.EXIT;
			}
		},

		failed: {
			'/#doc': function failedEndDoc() {
				return this.EXIT;
			}
		}
	};

	gcparts.partsRules[rule._name] = rule;

}());
