/*1.0.0*/
/*
 * gchelper.js
 *
 * Copyright 2013- Symmetric Co., Ltd. all rights reserved.
 */
(function() {

Td = function Td(idx, left, owner) {
	this._colspan = 0;
	this._rowspan = 0;
	this._empty = true;
	this._enabled = true;
	this._head = false;
	this._span = null;
	this._owner = owner;
	this._idx = idx;
	this._left = left;
	this._right = null;
	this._up = null;
	this._down = null;
	this._tdList = new Array();
	this._tree = new gcparts.ElementTree();
	this._attrs = new Object();

	// このセルの左と上とのリンクをつなげる
	this._left = left;
	if(left) {
		this._left._right = this;
	}
	if(owner._prev) {
		var _up = owner._prev.td(idx);
		var _oldDown = _up._down;
		if(_oldDown) {
			_oldDown._up = this;
			this._down = _oldDown;
		}
		_up._down = this;
		this._up = _up;
	} else if(owner._next) {
		var _down = owner._next.td(idx);
		var _oldUp = _down._up;
		if(_oldUp) {
			_oldUp._down = this;
			this._up = _oldUp;
		}
		_down._up = this;
		this._down = _down;
	}

	// rowspanチェック
	var _spanOwner = null;
	if(this._up && this._up._span) {
		_spanOwner = this._up._span;
		if(!this._down) {
			// 末尾に足す場合は、rowspanの自動拡張は行わない
			var _rs = _spanOwner._rowspan;
			var _len = this._owner._idx - _spanOwner._owner._idx;
			if(_rs <= _len) {
				// rowspanを満たした
				_spanOwner = null;
			}
		}
	} else if(this._down && this._down._span) {
		_spanOwner = this._down._span;
	}
	if(_spanOwner && _spanOwner._rowspan) {
		// rowspanかあったので、このセルを無効化する
		this._disable(_spanOwner);
		if(idx == _spanOwner._idx && this._down) {
			// 同一列で行末に挿入しない場合はrowspanをカウントアップする
			_spanOwner._rowspan++;
			_spanOwner._attrs['rowspan'] = _spanOwner._rowspan;
		}
	} else {
		// 行末ですぐ上のセルにrowspanがセットされている場合はrowspanを拡張せずに足す
		if(!this._down && this._up && this._up._rowspan) {
			this._disable(this._up);
		}
	}

};

Td.prototype.toString = function() {
	return this._th ? 'Th' : 'Td';
};

/**
 * ツリーに要素を足す
 * @param e 要素
 */
Td.prototype.addElement = function(e, a) {
	this._tree.addElement(e, a);
	this._empty = false;
};

/**
 * ツリーの現在の要素を閉じる
 */
Td.prototype.endElement = function() {
	this._tree.notifyAddElementEnd();
};

/**
 * セルにツリーを入れる
 * @param t ツリー
 */
Td.prototype.addTree = function(t) {
	this._tree.addTree(t);
	this._empty = false;
};


/**
 * THインスタンスとして扱う
 * @param bool_ 省略可能
 */
Td.prototype.assignHead = function(bool_) {
	if(typeof bool_ === 'undefined') {
		// 省略時は真
		this._head = true;
	} else {
		if(bool_) {
			this._head = true;
		} else {
			this._head = false;
		}
	}
};

/**
 * ツリーにテキストを足す
 * @param t 文字列
 */
Td.prototype.addText = function(t) {
	this._tree.addText(t);
	this._empty = false;
};

/**
 * 下のカラムの参照を取得する。
 * @param enabled 有効なカラムだけ
 * @param fill データが入っているものだけ
 * @return カラム。無い場合はnull
 */
Td.prototype.down = function(enabled, fill) {
	var _r = this._down;
	if(enabled) {
		while(_r && !_r.isEnabled()) {
			_r = _r._down;
		}
	}
	return _r;
};

/**
 * 上のカラムの参照を取得する。
 * @param enabled 有効なカラムだけ
 * @param fill データが入っているものだけ
 * @return カラム。無い場合はnull
 */
Td.prototype.up = function(enabled, fill) {
	var _r = this._up;
	if(enabled) {
		while(_r && !_r.isEnabled()) {
			_r = _r._up;
		}
	}
	return _r;
};

/**
 * ←のカラムの参照を取得する。
 * @param enabled 有効なカラムだけ
 * @param fill データが入っているものだけ
 * @return カラム。無い場合はnull
 */
Td.prototype.left = function(enabled, fill) {
	var _r = this._left;
	if(enabled) {
		while(_r && !_r.isEnabled()) {
			_r = _r._left;
		}
	}
	return _r;
};

/**
 * 右のカラムの参照を取得する。
 * @param enabled 有効なカラムだけ
 * @param fill データが入っているものだけ
 * @return カラム。無い場合はnull
 */
Td.prototype.right = function(enabled, fill) {
	var _r = this._right;
	if(enabled) {
		while(_r && !_r.isEnabled()) {
			_r = _r._right;
		}
	}
	return _r;
};

/**
 * colspanを設定する。右のカラムが無効になる
 * @param col 行数
 */
Td.prototype.colspan = function(col) {
	var _target = this._right;
	for(var _i = 1; _target && _i < col; _i++) {
		_target._disable(this);
		if(this._rowspan) {
			// カレントにrowspanが設定されている場合は進んだセルの行を無効にする
			_target._rowspan2(this._rowspan);
		}
		_target = _target._right;
	}
	this._colspan = col;
	this._attrs["colspan"] = col;
};

/**
 * rowspanを設定する。下のカラムが無効になる
 * @param row 行数
 */
Td.prototype.rowspan = function(row) {
	var _target = this._down;
	for(var _i = 1; _target && _i < row; _i++) {
		_target._disable(this);
		if(this._colspan) {
			_target._colspan2(this._colspan);
		}
		_target = _target._down;
	}
	this._rowspan = row;
	this._attrs["rowspan"] = row;
};

/**
 * カラムがcospan等の効果で無効になっていないかどうか
 * @return 有効の場合はtrue
 */
Td.prototype.isEnabled = function() {
	return this._enabled;
};

/**
 * カラムが空かどうか
 * @return データが入っていない場合はtrue
 */
Td.prototype.isEmpty = function() {
	return this._empty;
};

/**
 * THインスタンスか否か
 * @return THインスタンスの場合はtrue
 */
Td.prototype.isHead = function() {
	return this._head;
};

/**
 * 属している列の参照を取得する
 * @return 列
 */
Td.prototype.tr = function() {
	return this._owner;
};

/**
 * 属性を設定する
 * @param n 属性名
 * @param v 属性値
 */
Td.prototype.setAttribute = function(n, v) {
	this._attrs[n] = v;
};

/**
 * 現在のカラムインデックスを取得する。
 * @return インデックス0～
 */
Td.prototype.getColIndex = function() {
	return this._idx;
};

Td.prototype._disable = function(base) {
	this._span = base;
	this._enabled = false;
};

Td.prototype._colspan2 = function(col) {
	var _target = this._right;
	for(var _i = 1; _target && _i < col; _i++) {
		_target._disable(this);
		_target = _target._right;
	}
};

Td.prototype._rowspan2 = function(row) {
	var _target = this._down;
	for(var _i = 1; _target && _i < row; _i++) {
		_target._disable(this);
		_target = _target._down;
	}
};

Td.prototype._deleteRow = function() {
	if(this._up) {
		// 上にセルがある
		if(this._down) {
			// 下にもセルがある
			this._up._down = this._down;
			this._down._up = this._up;
		} else {
			// 自身が行末だった
			this._up._down = null;
		}
	} else if(this._down) {
		// 上には無いが下にセルがある
		// 自身が行頭だった
		this._down._up = null;
	}
	if(this._span) {
		// rowspanの行数を減らす
		this._span._rowspan--;
		this._span._attrs["rowspan"] = this._span._rowspan;
	}
};

Td.prototype._deleteCol = function() {

	// インデックスを更新
	var _cur = this._right;
	while(_cur != null) {
		_cur._idx--;
		_cur = _cur._right;
	}

	if(this._left) {
		// 左にセルがある
		if(this._right) {
			// 右にもセルがある
			this._left._right = this._right;
			this._right._left = this._left;
		} else {
			// 自身が列末だった
			this._left._right = null;
		}
	} else if(this._right) {
		// 左には無いが右にセルがある
		// 自身が列頭だった
		this._right._left = null;
	}
	if(this._span) {
		// colspanの行数を減らす
		this._span._colspan--;
		this._span._attrs["colspan"] = this._span._colspan;
	}
};

/**
 * 列オブジェクト作成
 * @param idx 行番号。0～
 * @param カラム数
 * @param 1つ前の列の参照。null可
 * @param 1つ後の列の参照。null可
 */
Tr = function Tr(idx, col, prev, next) {
	this._col = col;
	this._prev = prev;
	this._next = next;
	this._tdList = new Array();
	this._idx = idx;
	this._attrs = new Object();

	// リンクをつなげる
	if(prev) {
		prev._next = this;
	}
	if(next) {
		next._prev = this;
	}

	var _before = null;
	for(var _i = 0; _i < col; _i++) {
		this._tdList[_i] = new Td(_i, _before, this);
		_before = this._tdList[_i];
	}
	_before = null;
};

Tr.prototype.toString = function() {
	return 'Tr';
};

/**
 * 指定した列番号のカラムを取得する
 * @param 列番号。0～
 * @return カラム。無ければnull
 */
Tr.prototype.td = function(col) {
	return this._tdList[col];
};

/**
 * 次の行の参照を取得する
 * @return 次の行
 */
Tr.prototype.next = function() {
	return this._next;
};

/**
 * 前の行の参照を取得する
 * @return 前の行
 */
Tr.prototype.prev = function() {
	return this._prev;
};

/**
 * 現在の行の上に行を足す
 * @return 追加した行
 */
Tr.prototype.insertPrev = function() {
	return null;
};

/**
 * 現在の行の下に行を足す
 * @return 追加した行
 */
Tr.prototype.insertNext = function() {
	return null;
};

/**
 * 属性を設定する
 * @param n 属性名
 * @param v 属性値
 */
Tr.prototype.setAttribute = function(n, v) {
	this._attrs[n] = v;
};

/**
 * 現在の列インデックスを取得する。
 * @return インデックス0～
 */
Tr.prototype.getRowIndex = function() {
	return this._idx;
};

Tr.prototype._deleteRow = function() {
	for(var _i = 0; _i < this._col; _i++) {
		var _td = this._tdList[_i];
		// 行削除用メソッド
		_td._deleteRow();
	}

	// 自身の行参照を外す
	if(this._prev) {
		this._prev._next = this._next;
	}
	if(this._next) {
		this._next._prev = this._prev;
	}
	// インデックスを更新
	var _cur = this._next;
	while(_cur != null) {
		_cur._idx--;
		_cur = _cur._next;
	}
};

Tr.prototype._deleteCol = function(idx) {
	var _td = this._tdList[idx];
	// 列削除用メソッド
	_td._deleteCol();

	this._tdList.splice(idx, 1);
	this._col--;

};


/**
 * テーブルを作成する
 * @param col 列数
 * @param row 行数
 */
gcparts.TableMaker = function TableMaker(row, col) {
	if(col == 0 || row == 0) {
		throw new Error("invalid argument");
	}
	this._col = col;
	this._row = row;
	this._trList = new Array();
	this._attrs = new Object();

	var _before = null;
	for(var _i = 0; _i < row; _i++) {
		this._trList[_i] = new Tr(_i, col, _before);
		_before = this._trList[_i];
	}
	_before = null;
};

gcparts.TableMaker.prototype.toString = function() {
	return 'gcparts.TableMaker';
};

/**
 * テーブルの行数を取得する
 * @return 行数
 */
gcparts.TableMaker.prototype.getRow = function() {
	return this._row;
};

/**
 * テーブルの列数を取得する
 * @return 列数
 */
gcparts.TableMaker.prototype.getCol = function() {
	return this._col;
};

/**
 * 指定した行の参照を取得する
 * @param row 行数
 * @return 行(Tr)
 */
gcparts.TableMaker.prototype.tr = function(row) {
	if(row >= this._row) {
		return null;
	}
	return this._trList[row];
};

/**
 * 指定した場所のカラムの参照を取得する
 * @param col 列数
 * @param row 行数
 * @return カラム(Td)
 */
gcparts.TableMaker.prototype.td = function(row, col) {
	var _r = this._trList[row];
	if(!_r) {
		return null;
	}

	return _r.td(col);
};

/**
 * 属性を設定する
 * @param n 属性名
 * @param v 属性値
 */
gcparts.TableMaker.prototype.setAttribute = function(n, v) {
	this._attrs[n] = v;
};

/**
 * 行を追加する。rowspanが有効になっている行間に足した場合は、rowspanの距離が延びる。
 * @param idx 追加する行番号。0：行頭～行数：行末
 * @return 追加した行。不正なインデックスを指定した場合はnull
 */
gcparts.TableMaker.prototype.insertRow = function(idx) {
	if(this._row < idx || idx < 0) {
		// エラー
		return null;
	}
	var _ins = null;
	if(this._row == idx) {
		// 末尾に対する追加
		var _before = this._trList[idx - 1];
		_ins = new Tr(idx, this._col, _before, null);
		this._trList[idx] = _ins;
	} else if(0 == idx) {
		// 先頭に対する追加
		var _after = this._trList[1];
		if(!_after) {
			_after = this._trList[0];
		}
		_ins = new Tr(idx, this._col, null, _after);
		this._trList.unshift(_ins);
	} else {
		var _before = this._trList[idx - 1];
		var _after = this._trList[idx];
		_ins = new Tr(idx, this._col, _before, _after);
		this._trList.splice(idx, 0, _ins);
	}
	// インデックスを更新
	var _cur = _ins._next;

//	gclog.debug("ins:" + _ins.getRowIndex());
//	if(_cur)
//	gclog.debug("cur:" + _cur.getRowIndex());


	while(_cur != null) {
		_cur._idx++;
		_cur = _cur._next;
	}
	this._row++;
	return _ins;
};

/**
 * 行を削除する。
 * @param 削除する行インデックス(0～)
 * @return 削除に成功した場合はtrue
 */
gcparts.TableMaker.prototype.deleteRow = function(idx) {
	if(this._row <= idx || idx < 0) {
		// エラー
		return false;
	}

	var _target = this._trList[idx];
	_target._deleteRow();
	// 列を縮める
	this._row--;
	this._trList.splice(idx, 1);
	return true;
}

/**
 * 列を削除する。
 * @param 削除する列インデックス(0～)
 * @return 削除に成功した場合はtrue
 */
gcparts.TableMaker.prototype.deleteCol = function(idx) {
	if(this._col <= idx || idx < 0) {
		// エラー
		return false;
	}

	for(var _i = 0; _i < this._row; _i++) {
		var _tr = this._trList[_i];
		_tr._deleteCol(idx);
	}

	// 列を縮める
	this._col--;
	return true;
}

gcparts.TableMaker.prototype._getTree = function() {
	var _tbl = new gcparts.ElementTree();
	var _row = this.getRow();
	var _col = this.getCol();

	_tbl.addElement('table', this._attrs);

	for(var _r = 0; _r < _row; _r++) {
		var _tr = this._trList[_r];
		_tbl.addElement('tr', _tr._attrs);
		for(var _c = 0; _c < _col; _c++) {
			var _td = _tr.td(_c);
			if(_td.isEnabled()) {
				var tag = _td.isHead() ? 'th' : 'td';
				_tbl.addElement(tag, _td._attrs);
				_tbl.addTree(_td._tree);
				_tbl.notifyAddElementEnd();
			}
		}
		_tbl.notifyAddElementEnd();
	}
	_tbl.notifyAddElementEnd();


	return _tbl;
};







/**
 * リストを項目を作成するヘルパー関数群
 * @param listElement エレメント
 * @param anchorType 使用するアンカー（0:先頭のアンカーを使う、それ以外：最後のアンカーを使う）
 */
gcparts.ListMaker = function ListMaker(listElement, anchorType) {

	this._listElement = listElement;
	this._anchorType = anchorType || 0;
	this._anchorElement = null;
	this._itemList = [];

	return this;
};

gcparts.ListMaker.prototype.toString = function() {
	return 'gcparts.ListMaker';
};

gcparts.ListMaker.prototype.setAnchorElement = function(element) {

	if (!this._anchorElement) {
		this._anchorElement = element;
	} else if (this._anchorElement && this._anchorType != 0){
		this._anchorElement = element;
	}

};

gcparts.ListMaker.prototype.addItem = function(element, text) {
	var item = { 'element': element, '#text': text };
	this._itemList[this._itemList.length] = item;
};


gcparts.ListMaker.prototype.getItemCount = function() {
	return this._itemList.length;
};


gcparts.ListMaker.prototype._getTree = function(anchorNo) {

	var _tree = new gcparts.ElementTree();
	var _itemLen = this._itemList.length;

	if (_itemLen == 0) {
		return _tree;
	}

	_tree.addElement(this._listElement);

	if (this._anchorElement) {
		// A要素の追加
		_tree.addElement(this._anchorElement);
	}

	// リストコンテンツ
	for (var i = 0; i < _itemLen; i++) {
		var _item = this._itemList[i];
		if (_item['element']) {
			_tree.addElement(_item['element']);
		}
		if (_item['#text']) {
			_tree.addText(_item['#text']);
		}
		if (_item['element']) {
			_tree.endElement();
		}
	}

	if (this._anchorElement) {
		// A要素の閉じ
		_tree.endElement();
	}

	_tree.endElement();
	return _tree;

};


/**
 * リストを項目を作成するヘルパー関数群
 *
 * @param listElement
 *            エレメント
 * @param anchorType
 *            使用するアンカー（0:先頭のアンカーを使う、それ以外：最後のアンカーを使う）
 */
gcparts.ListMakerTwo = function ImageListMakerTwo(listElement, anchorType) {

	this._listElement = listElement;
	this._anchorType = anchorType || 0;
	this._anchorElement = null;
	this._itemList = [];

	return this;
};

gcparts.ListMakerTwo.prototype.toString = function() {
	return 'gcparts.ListMakerTwo';
};

gcparts.ListMakerTwo.prototype.setAnchorElement = function(element) {

	if (!this._anchorElement) {
		this._anchorElement = element;
	} else if (this._anchorElement && this._anchorType != 0) {
		this._anchorElement = element;
	}

};

gcparts.ListMakerTwo.prototype.addItem = function(element, text) {
	var item = {
		'element' : element,
		'#text' : text
	};
	this._itemList[this._itemList.length] = item;
};

gcparts.ListMakerTwo.prototype.getItemCount = function() {
	return this._itemList.length;
};

gcparts.ListMakerTwo.prototype.closeItem = function() {
	this._itemList[this._itemList.length] = {
		'element' : 'CLOSE'
	};
};

gcparts.ListMakerTwo.prototype._getTree = function() {

	var _tree = new gcparts.ElementTree();
	var _itemLen = this._itemList.length;
	if (_itemLen == 0) {
		return _tree;
	}

	_tree.addElement(this._listElement);

	if (this._anchorElement) {
		// A要素の追加
		_tree.addElement(this._anchorElement);
	}

	// リストコンテンツ
	for ( var i = 0; i < _itemLen; i++) {
		var _item = this._itemList[i];

		if (_item['element'] && _item['element'] == 'CLOSE') {
			_tree.endElement();
			continue;
		}
		if (_item['element']) {
			_tree.addElement(_item['element']);
		}
		if (_item['#text']) {
			_tree.addText(_item['#text']);
		}
	}

	if (this._anchorElement) {
		// A要素の閉じ
		_tree.endElement();
	}

	_tree.endElement();
	return _tree;

};

/**
 * 画像付きリスト項目を作成する
 *
 * @param listElement
 *            リスト要素
 * @param imageElements
 *            画像要素
 * @param thumbnailSize
 *            サムネイル画像横幅・高さの閾値
 */
gcparts.ImageListMakerTwo = function ImageListMakerTwo(listElement,
		imageElements, thumbnailSize, enableNoImage) {

	this._listElement = listElement;

	this.thumbnailElement = imageElements.thumbnail;
	this.iconElement = imageElements.icon;

	this.thumbnailWidth = thumbnailSize.width - 0;
	this.thumbnailHeight = thumbnailSize.height - 0;

	// 代替画像ツリー
	this.noImageTree = new gcparts.ElementTree();
	this.noImageTree.addElement(this.thumbnailElement);
	this.noImageTree.endElement();

	this.enableNoImage = enableNoImage ? enableNoImage : false;

	this._itemList = [];

	return this;
};

gcparts.ImageListMakerTwo.prototype.toString = function() {
	return 'gcparts.ImageListMaker';
};

gcparts.ImageListMakerTwo.prototype.addItem = function(element, text) {
	var item = {
		'element' : element,
		'#text' : text
	};
	this._itemList[this._itemList.length] = item;
};

gcparts.ImageListMakerTwo.prototype.closeItem = function() {
	this._itemList[this._itemList.length] = {
		'element' : 'CLOSE'
	};
},

gcparts.ImageListMakerTwo.prototype.getItemCount = function() {
	return this._itemList.length;
};

/**
 * バッファに溜まっている要素の中からサムネイルにふさわしい画像を探す（次候補） 画像の横幅・高さが未指定の画像を探し、最初に見つかった画像を返す
 */
gcparts.ImageListMakerTwo.prototype.searchThumbnailNextOption = function() {

	var image;
	for ( var i = 0; i < this._itemList.length; i++) {
		var _item = this._itemList[i];
		if (!_item['element']) {
			continue;
		}
		if (_item['element'] == 'CLOSE') {
			continue;
		}
		var _tname = _item['element'].tagName.toUpperCase();
		if (_tname != 'IMG') {
			continue;
		}
		var _attrs = _item['element'].attrs;
		var w = _attrs.getInt('width');
		var h = _attrs.getInt('height');
		if (w == -1 && h == -1) {
			return _item;
		}
	}
	// サムネイルにふさわしい画像は見つからなかった
	return null;
};
/**
 * バッファに溜まっている要素の中からサムネイルにふさわしい画像を探す 画像の横幅・高さが閾値を満たしている画像を探し、最初に見つかった画像を探す
 */
gcparts.ImageListMakerTwo.prototype.searchThumbnailImage = function() {

	for ( var i = 0; i < this._itemList.length; i++) {
		var _item = this._itemList[i];
		if (!_item['element']) {
			continue;
		}
		if (_item['element'] == 'CLOSE') {
			continue;
		}
		var _tname = _item['element'].tagName.toUpperCase();
		if (_tname != 'IMG') {
			continue;
		}
		var _attrs = _item['element'].attrs;

		var gcp = _attrs.get('gcp');
		if (gcp == 'mark:thumbnail') {
			return _item;
		}

		/* v1.1.0 属性値が%で設定されていた場合は、幅・高さの指定なしとみなす */
		var w = _attrs.get('width');
		var h = _attrs.get('height');
		if (w && w.slice(-1) != '%') {
			w = _attrs.getInt('width');
		} else {
			w = -1;
		}
		if (h && h.slice(-1) != '%') {
			h = _attrs.getInt('height');
		} else {
			h = -1;
		}

		var thumbnail = false;
		if (w != -1 && h != -1) {
			/* ver1.1.0 オプションに負の値が与えられた場合、比較ができていなかったバグを修正 */
			if (this.thumbnailWidth <= 0 && this.thumbnailHeight <= 0) {
				// サイズ指定なし
				thumbnail = true;
			} else if (this.thumbnailWidth > 0 && this.thumbnailHeight > 0) {
				if (w >= this.thumbnailWidth && h >= this.thumbnailHeight) {
					thumbnail = true;
				}
			} else if (this.thumbnailWidth > 0 && w >= this.thumbnailWidth) {
				thumbnail = true;
			} else if (this.thumbnailHeight > 0 && h >= this.thumbnailHeight) {
				thumbnail = true;
			}
		} else if (w != -1) {
			if (this.thumbnailWidth <= 0) {
				thumbnail = true;
			} else if (w >= this.thumbnailWidth) {
				thumbnail = true;
			}
		} else if (h != -1) {
			if (this.thumbnailHeight <= 0) {
				thumbnail = true;
			} else if (h >= this.thumbnailHeight) {
				thumbnail = true;
			}
		}
		if (thumbnail) {
			return _item;
		}
	}
	// 画像サイズが閾値を満たしている画像が見つからなかったので、次点を探す
	return this.searchThumbnailNextOption();
};

gcparts.ImageListMakerTwo.prototype._getTree = function() {

	var _tree = new gcparts.ElementTree();
	var _itemLen = this._itemList.length;
	if (_itemLen == 0) {
		return _tree;
	}

	_tree.addElement(this._listElement);

	// バッファに溜まっている画像からサムネイル画像を探す
	var thumbnail = this.searchThumbnailImage();
	if (!thumbnail && this.enableNoImage) {
		// リストに１つも画像がないので、NOIMAGEを表示する
		_tree.addTree(this.noImageTree);
	}

	// リストコンテンツ
	for ( var i = 0; i < _itemLen; i++) {
		var _item = this._itemList[i];

		if (_item['element'] && _item['element'] == 'CLOSE') {
			_tree.endElement();
			continue;
		}
		if (_item['element']) {
			var tname = _item['element'].tagName.toUpperCase();
			if (tname == 'IMG') {
				var image;
				if (thumbnail == _item) {
					image = this.thumbnailElement.cloneWithDeepAttrs();
				} else {
					image = this.iconElement.cloneWithDeepAttrs();
				}
				var imgAttrs = _item['element'].attrs;
				if (imgAttrs.has('src')) {
					image.setAttribute('src', imgAttrs.get('src'));
				}
				if (imgAttrs.has('alt')) {
					image.setAttribute('alt', imgAttrs.get('alt'));
				}
				if (imgAttrs.has('title')) {
					image.setAttribute('title', imgAttrs.get('title'));
				}
				_tree.addElement(image);
			} else {
				_tree.addElement(_item['element']);
			}
		}
		if (_item['#text']) {
			_tree.addText(_item['#text']);
		}
	}
	_tree.endElement();
	return _tree;

};

/**
 * 画像付きリスト項目を作成する(アンカーが各アイテム全てを内包)
 *
 * @param listElement
 *            リスト要素
 * @param imageElements
 *            画像要素
 * @param thumbnailSize
 *            サムネイル画像横幅・高さの閾値
 */
gcparts.ImageListMakerThree = function ImageListMakerThree(listElement,
		imageElements, thumbnailSize, enableNoImage, anchorType) {

	this._listElement = listElement;

	this.thumbnailElement = imageElements.thumbnail;
	this.iconElement = imageElements.icon;

	this.thumbnailWidth = thumbnailSize.width - 0;
	this.thumbnailHeight = thumbnailSize.height - 0;

	// 代替画像ツリー
	this.noImageTree = new gcparts.ElementTree();
	this.noImageTree.addElement(this.thumbnailElement);
	this.noImageTree.endElement();

	this.enableNoImage = enableNoImage ? enableNoImage : false;

	this._anchorType = anchorType || 0;
	this._anchorElement = null;

	this._itemList = [];

	return this;
};

gcparts.ImageListMakerThree.prototype.toString = function() {
	return 'gcparts.ImageListMakerThree';
};

gcparts.ImageListMakerThree.prototype.setAnchorElement = function(element) {

	if (!this._anchorElement) {
		this._anchorElement = element;
	} else if (this._anchorElement && this._anchorType != 0) {
		this._anchorElement = element;
	}

};

gcparts.ImageListMakerThree.prototype.addItem = function(element, text) {
	var item = {
		'element' : element,
		'#text' : text
	};
	this._itemList[this._itemList.length] = item;
};

gcparts.ImageListMakerThree.prototype.closeItem = function() {
	this._itemList[this._itemList.length] = {
		'element' : 'CLOSE'
	};
},

gcparts.ImageListMakerThree.prototype.getItemCount = function() {
	return this._itemList.length;
};

/**
 * バッファに溜まっている要素の中からサムネイルにふさわしい画像を探す（次候補） 画像の横幅・高さが未指定の画像を探し、最初に見つかった画像を返す
 */
gcparts.ImageListMakerThree.prototype.searchThumbnailNextOption = function() {

	var image;
	for ( var i = 0; i < this._itemList.length; i++) {
		var _item = this._itemList[i];
		if (!_item['element']) {
			continue;
		}
		if (_item['element'] == 'CLOSE') {
			continue;
		}
		var _tname = _item['element'].tagName.toUpperCase();
		if (_tname != 'IMG') {
			continue;
		}
		var _attrs = _item['element'].attrs;
		var w = _attrs.getInt('width');
		var h = _attrs.getInt('height');
		if (w == -1 && h == -1) {
			return _item;
		}
	}
	// サムネイルにふさわしい画像は見つからなかった
	return null;
};
/**
 * バッファに溜まっている要素の中からサムネイルにふさわしい画像を探す 画像の横幅・高さが閾値を満たしている画像を探し、最初に見つかった画像を探す
 */
gcparts.ImageListMakerThree.prototype.searchThumbnailImage = function() {

	for ( var i = 0; i < this._itemList.length; i++) {
		var _item = this._itemList[i];
		if (!_item['element']) {
			continue;
		}
		if (_item['element'] == 'CLOSE') {
			continue;
		}
		var _tname = _item['element'].tagName.toUpperCase();
		if (_tname != 'IMG') {
			continue;
		}
		var _attrs = _item['element'].attrs;

		var gcp = _attrs.get('gcp');
		if (gcp == 'mark:thumbnail') {
			return _item;
		}

		/* v1.1.0 属性値が%で設定されていた場合は、幅・高さの指定なしとみなす */
		var w = _attrs.get('width');
		var h = _attrs.get('height');
		if (w && w.slice(-1) != '%') {
			w = _attrs.getInt('width');
		} else {
			w = -1;
		}
		if (h && h.slice(-1) != '%') {
			h = _attrs.getInt('height');
		} else {
			h = -1;
		}

		var thumbnail = false;
		if (w != -1 && h != -1) {
			/* ver1.1.0 オプションに負の値が与えられた場合、比較ができていなかったバグを修正 */
			if (this.thumbnailWidth <= 0 && this.thumbnailHeight <= 0) {
				// サイズ指定なし
				thumbnail = true;
			} else if (this.thumbnailWidth > 0 && this.thumbnailHeight > 0) {
				if (w >= this.thumbnailWidth && h >= this.thumbnailHeight) {
					thumbnail = true;
				}
			} else if (this.thumbnailWidth > 0 && w >= this.thumbnailWidth) {
				thumbnail = true;
			} else if (this.thumbnailHeight > 0 && h >= this.thumbnailHeight) {
				thumbnail = true;
			}
		} else if (w != -1) {
			if (this.thumbnailWidth <= 0) {
				thumbnail = true;
			} else if (w >= this.thumbnailWidth) {
				thumbnail = true;
			}
		} else if (h != -1) {
			if (this.thumbnailHeight <= 0) {
				thumbnail = true;
			} else if (h >= this.thumbnailHeight) {
				thumbnail = true;
			}
		}
		if (thumbnail) {
			return _item;
		}
	}
	// 画像サイズが閾値を満たしている画像が見つからなかったので、次点を探す
	return this.searchThumbnailNextOption();
};

gcparts.ImageListMakerThree.prototype._getTree = function() {

	var _tree = new gcparts.ElementTree();
	var _itemLen = this._itemList.length;
	if (_itemLen == 0) {
		return _tree;
	}

	_tree.addElement(this._listElement);

	if (this._anchorElement) {
		_tree.addElement(this._anchorElement);
	}

	// バッファに溜まっている画像からサムネイル画像を探す
	var thumbnail = this.searchThumbnailImage();
	if (!thumbnail && this.enableNoImage) {
		// リストに１つも画像がないので、NOIMAGEを表示する
		_tree.addTree(this.noImageTree);
	}

	// リストコンテンツ
	for ( var i = 0; i < _itemLen; i++) {
		var _item = this._itemList[i];

		if (_item['element'] && _item['element'] == 'CLOSE') {
			_tree.endElement();
			continue;
		}
		if (_item['element']) {
			var tname = _item['element'].tagName.toUpperCase();
			if (tname == 'IMG') {
				var image;
				if (thumbnail == _item) {
					image = this.thumbnailElement.cloneWithDeepAttrs();
				} else {
					image = this.iconElement.cloneWithDeepAttrs();
				}
				var imgAttrs = _item['element'].attrs;
				if (imgAttrs.has('src')) {
					image.setAttribute('src', imgAttrs.get('src'));
				}
				if (imgAttrs.has('alt')) {
					image.setAttribute('alt', imgAttrs.get('alt'));
				}
				if (imgAttrs.has('title')) {
					image.setAttribute('title', imgAttrs.get('title'));
				}
				_tree.addElement(image);
			} else {
				_tree.addElement(_item['element']);
			}
		}
		if (_item['#text']) {
			_tree.addText(_item['#text']);
		}
	}

	if (this._anchorElement) {
		_tree.endElement();
	}

	_tree.endElement();
	return _tree;

};

/**
 * メニュー項目を作成する
 * @param design ツリー作成時に使用するデザイン要素群
 */
gcparts.TextMenuMaker = function TextMenuMaker(design) {

	this._anchorType = 0;
	this._textWrapperElement = design.text;
	this._subTreeElement = design.subTree;

	this._anchorElement = null;
	this._itemList = [];
	this._textFlg = false;
	this._subMenu = false;

	return this;
};

gcparts.TextMenuMaker.prototype.toString = function() {
	return 'gcparts.TextMenuMaker';
};

gcparts.TextMenuMaker.prototype.setAnchorElement = function(element) {
	if (!this._anchorElement) {
		this._anchorElement = element;
	} else if (this._anchorElement && this._anchorType != 0){
		this._anchorElement = element;
	}
};

gcparts.TextMenuMaker.prototype.addItem = function(element, text) {
	if (text) {
		this._textFlg = true;
	}
	var item = { 'element': element, '#text': text };
	this._itemList[this._itemList.length] = item;
};

gcparts.TextMenuMaker.prototype.getItemCount = function() {
	return this._itemList.length;
};

gcparts.TextMenuMaker.prototype.hasContent = function() {

	if (this._anchorElement) {
		return true;
	} else if (this._itemList.length > 0) {
		return true;
	}
	return false;
};

gcparts.TextMenuMaker.prototype.setSubMenu = function(_bool) {
	this._subMenu = _bool;
};

gcparts.TextMenuMaker.prototype.searchImageAltAttr = function() {

	var _itemLen = this._itemList.length;
	if (_itemLen == 0) {
		return null;
	}

	for (var i = 0; i < _itemLen; i++) {
		var _item = this._itemList[i];
		if (_item['element'] && _item['element'].tagName && _item['element'].tagName.toUpperCase() == 'IMG') {
			var alt = _item['element'].attrs.get('alt');
			if (alt && alt.length > 0) {
				return alt;
			}
		}
	}
	return null;
};

gcparts.TextMenuMaker.prototype.searchAnchorTitle = function() {
	if (!this._anchorElement) {
		return null;
	}
	var title = this._anchorElement.getAttribute('title');
	if (title) {
		return title;
	} else {
		return null;
	}
};

gcparts.TextMenuMaker.prototype.getAltText = function() {
	var text = this.searchImageAltAttr();
	if (text) {
		return text;
	}
	text = this.searchAnchorTitle();
	if (text) {
		return text;
	}
	return null;
};



gcparts.TextMenuMaker.prototype._getTree = function() {

	var _tree = new gcparts.ElementTree();
	if (!this._textFlg) {
		// テキスト情報がない
		var text = this.getAltText();
		if (text) {
			this.addItem(this._textWrapperElement, text);
		}
	}

	var _itemLen = this._itemList.length;
	// リンクもなくテキスト情報もなければツリーを出力しない
	if (_itemLen == 0 && !this._anchorElement) {
		return _tree;
	}


	if (this._subMenu) {
		_tree.addElement(this._subTreeElement);
		if (this._anchorElement) {
			// A要素の追加
			_tree.addElement(this._anchorElement);
		}

	} else {
		if (this._anchorElement) {
			// A要素の追加
			_tree.addElement(this._anchorElement);
		}
	}

	// メニューコンテンツ
	for (var i = 0; i < _itemLen; i++) {
		var _item = this._itemList[i];

		if (_item['element'] && _item['element'].tagName && _item['element'].tagName.toUpperCase() == 'IMG') {
			// 画像は除外
			continue;
		}
		var e = _item['element'];
		var t = _item['#text'];
		if (e) {
			_tree.addElement(e);
		}
		if (t) {
			_tree.addText(t);
		}
		if (e) {
			_tree.endElement();
		}
	}

	if (this._subMenu) {
		// A要素の閉じ
		if (this._anchorElement) {
			_tree.endElement();
		}
		_tree.endElement();

	} else {
		// A要素の閉じ
		if (this._anchorElement) {
			_tree.endElement();
		}
	}

	return _tree;

};

/**
 * 背景画像付きメニュー項目を作成する
 * @param design ツリー作成時に使用するデザイン要素群
 */
gcparts.ImageMenuMaker = function ImageMenuMaker(design) {

	this._anchorType = 0;
	this._iconElement = design.icon;
	this._textWrapperElement = design.text;
	this._subTreeElement = design.subTree;

	this._anchorElement = null;
	this._itemList = [];
	this._textFlg = false;
	this._subMenu = false;

	this._backgroundImage = null;

	return this;
};

gcparts.ImageMenuMaker.prototype.toString = function() {
	return 'gcparts.ImageMenuMaker';
};

gcparts.ImageMenuMaker.prototype.setAnchorElement = function(element) {
	if (!this._anchorElement) {
		this._anchorElement = element;
	} else if (this._anchorElement && this._anchorType != 0){
		this._anchorElement = element;
	}
};

gcparts.ImageMenuMaker.prototype.addItem = function(element, text) {
	if (text) {
		this._textFlg = true;
	}
	var item = { 'element': element, '#text': text };
	this._itemList[this._itemList.length] = item;
};

gcparts.ImageMenuMaker.prototype.getItemCount = function() {
	return this._itemList.length;
};

gcparts.ImageMenuMaker.prototype.hasContent = function() {
	if (this._anchorElement) {
		return true;
	} else if (this._itemList.length > 0) {
		return true;
	}
	return false;
};

gcparts.ImageMenuMaker.prototype.setSubMenu = function(_bool) {
	this._subMenu = _bool;
};

gcparts.ImageMenuMaker.prototype.setBackgroundImage = function(_image) {
	this._backgroundImage = _image;
};

gcparts.ImageMenuMaker.prototype.searchImageAltAttr = function() {

	var _itemLen = this._itemList.length;
	if (_itemLen == 0) {
		return null;
	}
	for (var i = 0; i < _itemLen; i++) {
		var _item = this._itemList[i];
		if (_item['element'] && _item['element'].tagName && _item['element'].tagName.toUpperCase() == 'IMG') {
			var alt = _item['element'].attrs.get('alt');
			if (alt && alt.length > 0) {
				return alt;
			}
		}
	}
	return null;
};

gcparts.ImageMenuMaker.prototype.searchAnchorTitle = function() {
	if (!this._anchorElement) {
		return null;
	}
	var title = this._anchorElement.getAttribute('title');
	if (title) {
		return title;
	} else {
		return null;
	}
};

gcparts.ImageMenuMaker.prototype.getAltText = function() {
	var text = this.searchImageAltAttr();
	if (text) {
		return text;
	}
	text = this.searchAnchorTitle();
	if (text) {
		return text;
	}
	return null;
};

gcparts.ImageMenuMaker.prototype._getTree = function() {

	var _tree = new gcparts.ElementTree();
	if (!this._textFlg) {
		// テキスト情報がない
		var text = this.getAltText();
		if (text) {
			this.addItem(this._textWrapperElement, text);
		}
	}

	var _itemLen = this._itemList.length;
	// リンクもなくテキスト情報もなければツリーを出力しない
	if (_itemLen == 0 && !this._anchorElement) {
		return _tree;
	}

	if (this._iconElement) {
		if (this._backgroundImage) {
			this._iconElement.setAttribute('style', 'background-image:' + this._backgroundImage);
		}
		// 背景画像が設定される要素（div）の追加
		_tree.addElement(this._iconElement);
	}

	if (this._subMenu) {
		_tree.addElement(this._subTreeElement);
		if (this._anchorElement) {
			// A要素の追加
			_tree.addElement(this._anchorElement);
		}
	} else {
		if (this._anchorElement) {
			// A要素の追加
			_tree.addElement(this._anchorElement);
		}
	}

	// メニューコンテンツ
	for (var i = 0; i < _itemLen; i++) {
		var _item = this._itemList[i];
		if (_item['element'] && _item['element'].tagName && _item['element'].tagName.toUpperCase() == 'IMG') {
			// 画像は除外
			continue;
		}
		var e = _item['element'];
		var t = _item['#text'];
		if (e) {
			_tree.addElement(e);
		}
		if (t) {
			_tree.addText(t);
		}
		if (e) {
			_tree.endElement();
		}
	}

	if (this._subMenu) {
		// A要素の閉じ
		if (this._anchorElement) {
			_tree.endElement();
		}
		_tree.endElement();

	} else {
		// A要素の閉じ
		if (this._anchorElement) {
			_tree.endElement();
		}
	}

	if (this._iconElement) {
		// 背景画像が設定される要素（div）の閉じ
		_tree.endElement();
	}

	return _tree;

};

/**
 * 画像付きリスト項目を作成する
 * @param listElement リスト要素
 * @param imageElements 画像要素
 * @param thumbnailSize サムネイル画像横幅・高さの閾値
 */
gcparts.ImageListMaker = function ImageListMaker(listElement, imageElements, thumbnailSize) {

	this._listElement = listElement;

	this.thumbnailElement = imageElements.thumbnail;
	this.iconElement = imageElements.icon;

	this.thumbnailWidth = thumbnailSize.width - 0;
	this.thumbnailHeight = thumbnailSize.height - 0;

	// 代替画像ツリー
	this.noImageTree = new gcparts.ElementTree();
	this.noImageTree.addElement(this.thumbnailElement);
	this.noImageTree.endElement();

	this._itemList = [];

	return this;
};

gcparts.ImageListMaker.prototype.toString = function() {
	return 'gcparts.ImageListMaker';
};

gcparts.ImageListMaker.prototype.addItem = function(element, text) {
	var item = { 'element': element, '#text': text };
	this._itemList[this._itemList.length] = item;
};

gcparts.ImageListMaker.prototype.closeItem = function() {
	this._itemList[this._itemList.length] = { 'element': 'CLOSE' };
},

gcparts.ImageListMaker.prototype.getItemCount = function() {
	return this._itemList.length;
};

/**
 * バッファに溜まっている要素の中からサムネイルにふさわしい画像を探す（次候補）
 * 画像の横幅・高さが未指定の画像を探し、最初に見つかった画像を返す
 */
gcparts.ImageListMaker.prototype.searchThumbnailNextOption = function() {

	var image;
	for (var i = 0; i < this._itemList.length; i++) {
		var _item = this._itemList[i];
		if (!_item['element']) {
			continue;
		}
		if (_item['element'] == 'CLOSE') {
			continue;
		}
		var _tname = _item['element'].tagName.toUpperCase();
		if (_tname != 'IMG') {
			continue;
		}
		var _attrs = _item['element'].attrs;
		var w = _attrs.getInt('width');
		var h = _attrs.getInt('height');
		if (w == -1 && h == -1) {
			return _item;
		}
	}
	// サムネイルにふさわしい画像は見つからなかった
	return null;
};
/**
 * バッファに溜まっている要素の中からサムネイルにふさわしい画像を探す
 * 画像の横幅・高さが閾値を満たしている画像を探し、最初に見つかった画像を探す
 */
gcparts.ImageListMaker.prototype.searchThumbnailImage = function() {

	for (var i = 0; i < this._itemList.length; i++) {
		var _item = this._itemList[i];
		if (!_item['element']) {
			continue;
		}
		if (_item['element'] == 'CLOSE') {
			continue;
		}
		var _tname = _item['element'].tagName.toUpperCase();
		if (_tname != 'IMG') {
			continue;
		}
		var _attrs = _item['element'].attrs;

		/* v1.1.0 属性値が%で設定されていた場合は、幅・高さの指定なしとみなす */
		var w = _attrs.get('width');
		var h = _attrs.get('height');
		if (w && w.slice(-1) != '%') {
			w = _attrs.getInt('width');
		} else {
			w = -1;
		}
		if (h && h.slice(-1) != '%') {
			h = _attrs.getInt('height');
		} else {
			h = -1;
		}

		var thumbnail = false;
		if (w != -1 && h != -1) {
			/* ver1.1.0 オプションに負の値が与えられた場合、比較ができていなかったバグを修正 */
			if (this.thumbnailWidth <= 0 && this.thumbnailHeight <= 0) {
				// サイズ指定なし
				thumbnail = true;
			} else if (this.thumbnailWidth > 0 && this.thumbnailHeight > 0) {
				if (w >= this.thumbnailWidth && h >= this.thumbnailHeight) {
					thumbnail = true;
				}
			} else if (this.thumbnailWidth > 0 && w >= this.thumbnailWidth) {
				thumbnail = true;
			} else if (this.thumbnailHeight > 0 && h >= this.thumbnailHeight) {
				thumbnail = true;
			}
		} else if (w != -1) {
			if (this.thumbnailWidth <= 0) {
				thumbnail = true;
			} else if (w >= this.thumbnailWidth) {
				thumbnail = true;
			}
		} else if (h != -1) {
			if (this.thumbnailHeight <= 0) {
				thumbnail = true;
			} else if (h >= this.thumbnailHeight) {
				thumbnail = true;
			}
		}
		if (thumbnail) {
			return _item;
		}
	}
	// 画像サイズが閾値を満たしている画像が見つからなかったので、次点を探す
	return this.searchThumbnailNextOption();
};

gcparts.ImageListMaker.prototype._getTree = function() {

	var _tree = new gcparts.ElementTree();
	var _itemLen = this._itemList.length;
	if (_itemLen == 0) {
		return _tree;
	}

	_tree.addElement(this._listElement);

	// バッファに溜まっている画像からサムネイル画像を探す
	var thumbnail = this.searchThumbnailImage();
	if (!thumbnail) {
		// リストに１つも画像がないので、NOIMAGEを表示する
		_tree.addTree(this.noImageTree);
	}

	// リストコンテンツ
	for (var i = 0; i < _itemLen; i++) {
		var _item = this._itemList[i];

		if (_item['element'] && _item['element'] == 'CLOSE') {
			_tree.endElement();
			continue;
		}
		if (_item['element']) {
			var tname = _item['element'].tagName.toUpperCase();
			if (tname == 'IMG') {
				var image;
				if (thumbnail == _item) {
					image = this.thumbnailElement.cloneWithDeepAttrs();
				} else {
					image = this.iconElement.cloneWithDeepAttrs();
				}
				image.setAttribute('src', _item['element'].attrs.get('src'));
				_tree.addElement(image);
			} else {
				_tree.addElement(_item['element']);
			}
		}
		if (_item['#text']) {
			_tree.addText(_item['#text']);
		}
	}
	_tree.endElement();
	return _tree;

};

/**
 * カスタムフォーム出力ヘルパー
 *
 * @param layoutData フォームレイアウト定義
 * @param design デザイン用の要素群
 */
gcparts.FormMaker = function FormMaker(layoutData, design) {

	this._layoutData = layoutData;
	this._design = {};
	this._design.title = design.title || {'tagName': 'h3'};
	this._design.frame = design.frame || {'tagName': 'div'};
	this._design.header = design.header || {'tagName' : 'div'};
	this._design.field = design.field || {'tagName' : 'div'};
	this._design.prev = design.prev || {'tagName' : 'span'};
	this._design.input = design.input || {'tagName' : 'span'};
	this._design.next = design.next || {'tagName' : 'span'};
	this._design.footer1 = design.footer1 || {'tagName' : 'div'};
	this._design.footer2 = design.footer2 || {'tagName' : 'div'};

	this._formElement = null;
	this._inputElements = [];
	this._hiddenElements = [];

	return this;

};

gcparts.FormMaker.prototype.toString = function() {
	return 'gcparts.FormMaker';
};

/**
 * フォーム項目を追加する
 * @param name  フォーム名(name属性値)
 * @param value 値(value属性値)
 * @param element 要素データ
 */
gcparts.FormMaker.prototype.addInputItem = function(name, value, element) {
	var _obj = { name : name, value : value, element : element };
	this._inputElements[this._inputElements.length] = _obj;
};

/**
 * フォーム情報を設定する
 * @param form
 */
gcparts.FormMaker.prototype.setForm = function(form) {
	this._formElement = form;
};

/**
 * hidden項目を追加する
 * @param element 要素データ
 */
gcparts.FormMaker.prototype.addHiddenItem = function(element) {
	this._hiddenElements[this._hiddenElements.length] = element;
};

/**
 * nameが一致するもしくはnameとvalueが一致するフォーム部品を探す。見つからないときはnullを返す
 * @param name
 * @param value 指定されているときは、nameとvalue両方が一致する項目を探す
 */
gcparts.FormMaker.prototype._searchInput = function(name, value) {

	for (var i = 0; i < this._inputElements.length; i++) {
		var input = this._inputElements[i];
		if (input['name'] != name) {
			continue;
		}
		if (value && input['value'] != value) {
			continue;
		}
		return input['element'];
	}

	return null;
};

gcparts.FormMaker.prototype._traverseFooter = function(node, tree) {

	for (var fc = node; fc != null; fc = fc.nextSibling) {
		var type = fc.nodeType;
		if (type == Node.ELEMENT_NODE) {
			var e = { 'tagName' : fc.nodeName };
			var ar = fc.attributes;
			var attrs = new gcparts.Attributes(ar);

			if (fc.nodeName == 'A') {
				// アンカーの場合のみ、新しいウィンドウを開くように属性を強制的に設定
				attrs.set('target', '_blank');
			}
			e['attrs'] = attrs;
			tree.addElement(e);

			// 再帰的にツリーを読む
			this._traverseFooter(fc.firstChild, tree);

			tree.endElement();
		} else if (type == Node.TEXT_NODE) {
			tree.addText(fc.nodeValue);
		}
	}
};

gcparts.FormMaker.prototype._getTree = function() {

	var _tree = new gcparts.ElementTree();

	_tree.addElement(this._formElement);

	for (var i = 0; i < this._hiddenElements.length; i++) {
		_tree.addElement(this._hiddenElements[i]);
		_tree.endElement();
	}

	for (var i = 0; i < this._layoutData.length; i++) {
		var layout = this._layoutData[i];

		// 見出しの出力 （例）<h3 class="title">お客様情報</h3>
		var _title = layout['title'];
		if (_title) {
			_tree.addElement(this._design.title.cloneWithDeepAttrs());
			_tree.addText(_title);
			_tree.endElement();
		}

		// １セクション開始
		_tree.addElement(this._design.frame);

		var items = layout['items'];
		for (var j = 0; j < items.length; j++) {

			var item = items[j];
			var _header = item['header'];
			var _footer1 = item['footer1'];
			var _footer2 = item['footer2'];
			var _inputs = item['inputs'] || [];
			var _layout = item['layout'];

			// 項目見出し
			if (_header) {
				_tree.addElement(this._design.header.cloneWithDeepAttrs());
				_tree.addText(_header);
				_tree.endElement();
			}

			// 入力項目（フォーム部品）
			var _inputsLen = _inputs.length;
			for (var k = 0; k < _inputsLen; k++) {

				var _name = _inputs[k]['name'];
				var _value = _inputs[k]['value'];
				var _prev = _inputs[k]['prev'];
				var _next = _inputs[k]['next'];
				var _placeholder = _inputs[k]['placeholder'];

				var elem = this._searchInput(_name, _value);
				if (!elem) {
					continue;
				}

				// 入力項目フィールド出力
				var _f = this._design.field.cloneWithDeepAttrs();
				if (_layout) {
					// クラス属性値(inline)のappend
					var c = _f.getAttribute('class');
					if (c) {
						c += ' ' + _layout;
					} else {
						c = _layout;
					}
					// さらに、何分割かを識別できるクラス属性値(inline2 or inline3 or inline4...)をappendする
					c += ' ' + _layout + _inputsLen;
					_f.setAttribute('class', c);
				}
				_tree.addElement(_f);

				// 入力項目の前に表示する文言の出力
				if (_prev) {
					var _p = this._design.prev.cloneWithDeepAttrs();
					var c = _p.getAttribute('class');
					if (c) {
						c += ' p_' + _name;
					} else {
						c = 'p_' + _name;
					}
					_p.setAttribute('class', c);
					_tree.addElement(_p);
					_tree.addText(_prev);
					_tree.endElement();
				}

				// フォーム部品の出力
				var _i = this._design.input.cloneWithDeepAttrs();
				var c = _i.getAttribute('class');
				if (c) {
					c += ' i_' + _name;
				} else {
					c = 'i_' + _name;
				}
				_i.setAttribute('class', c);
				_tree.addElement(_i);

				if (elem instanceof gcparts.ElementTree) {
					_tree.addTree(elem);
				} else if(elem instanceof gcparts.EventObject) {
					if (_placeholder) {
						elem.setAttribute('placeholder', _placeholder);
					}
					_tree.addElement(elem);
					_tree.endElement();
				} else {
					if (_placeholder) {
						elem.attrs.set('placeholder', _placeholder);
					}
					_tree.addElement(elem);
					_tree.endElement();
				}
				_tree.endElement();		// close _design.input

				// 入力項目の後に表示する文言
				if (_next) {
					var _n = this._design.next.cloneWithDeepAttrs();
					var c = _n.getAttribute('class');
					if (c) {
						c += ' n_' + _name;
					} else {
						c = 'n_' + _name;
					}
					_n.setAttribute('class', c);
					_tree.addElement(_n);
					_tree.addText(_next);
					_tree.endElement();
				}
				_tree.endElement();	// close _design.field
			}

			var rtag = /<("[^"]*"|\'[^\']*\'|[^\'">])*>/;
			//var rtag = /<\w+(?:\s+\w+(?:\s*=\s*(?:".*?"|'.*?'|[^'">\s]+))?)?(?:\s?\/)?>|<\/\w+>/g;
			// フッター１とフッター２はタグの入力を想定する
			if (_footer1) {
				_tree.addElement(this._design.footer1.cloneWithDeepAttrs());
				if (_footer1.match(rtag)) {
					var _f1 = $('<span>' + _footer1 + '</span>').get(0);
					var _c1 = _f1.childNodes;
					this._traverseFooter(_c1[0], _tree);
				} else {
					_tree.addText(_footer1);
				}
				_tree.endElement();
			}
			if (_footer2) {
				_tree.addElement(this._design.footer2.cloneWithDeepAttrs());
				if (_footer2.match(rtag)) {
					var _f2 = $('<span>' + _footer2 + '</span>').get(0);
					var _c2 = _f2.childNodes;
					this._traverseFooter(_c2[0], _tree);
				} else {
					_tree.addText(_footer2);
				}
				_tree.endElement();
			}
		}
		_tree.endElement();	// close _design.frame
	}
	_tree.endElement(); // close _formElement

	return _tree;
};

/**
 * カスタムフォーム出力ヘルパー(RELABELFORM02)
 *
 * @param layoutData
 *            フォームレイアウト定義
 * @param design
 *            デザイン用の要素群
 */
gcparts.FormMaker2 = function FormMaker2(layoutData, design) {

	this._layoutData = layoutData;
	this._design = {};
	this._design.title = design.title || { 'tagName' : 'h3' };
	this._design.frame = design.frame || { 'tagName' : 'div' };
	this._design.header = design.header || { 'tagName' : 'div' };
	this._design.field = design.field || { 'tagName' : 'div' };
	this._design.prev = design.prev || { 'tagName' : 'span' };
	this._design.input = design.input || {  'tagName' : 'span' };
	this._design.next = design.next || { 'tagName' : 'span' };
	this._design.footer1 = design.footer1 || { 'tagName' : 'div' };
	this._design.footer2 = design.footer2 || { 'tagName' : 'div' };
	this._design.label = design.label || { 'tagName' : 'label' };

	this._formElement = null;
	this._inputElements = [];
	this._hiddenElements = [];

	return this;

};

gcparts.FormMaker2.prototype.toString = function() {
	return 'gcparts.FormMaker2';
};

/**
 * 引数のname,label,id,className,valueとマッチするinput要素のレイアウト情報を取る
 */
gcparts.FormMaker2.prototype.searchAttributeSettings = function(name, label, id, classNames, value) {

	for ( var i = 0; i < this._layoutData.length; i++) {
		var items = this._layoutData[i]['items'];
		for ( var j = 0; j < items.length; j++) {
			var inputs = items[j]['inputs'];
			for ( var k = 0; k < inputs.length; k++) {

				if (inputs[k]['name'] && inputs[k]['name'] == name) {
					if (!inputs[k]['value']) {
						return inputs[k]['attributes'] || undefined;
					} else if (inputs[k]['value'] == value) {
						return inputs[k]['attributes'] || undefined;
					}
				}

				if (inputs[k]['label'] && inputs[k]['label'] == label) {
					if (!inputs[k]['value']) {
						return inputs[k]['attributes'] || undefined;
					} else if (inputs[k]['value'] == value) {
						return inputs[k]['attributes'] || undefined;
					}
				}

				if (inputs[k]['id'] && inputs[k]['id'] == id) {
					if (!inputs[k]['value']) {
						return inputs[k]['attributes'] || undefined;
					} else if (inputs[k]['value'] == value) {
						return inputs[k]['attributes'] || undefined;
					}
				}

				if (inputs[k]['className']) {
					for ( var h = 0; h < classNames.length; h++) {
						if (inputs[k]['className'] == classNames[h]) {
							if (!inputs[k]['value']) {
								return inputs[k]['attributes'] || undefined;
							} else if (inputs[k]['value'] == value) {
								return inputs[k]['attributes'] || undefined;
							}
						}
					}
				}
			}
		}
		return undefined;
	}
};

/**
 * フォーム項目をバッファに追加する
 *
 * @param name
 *            フォーム部品識別(name属性値)
 * @param label
 *            フォーム部品識別（data-gc-label属性値）
 * @param id
 *            フォーム部品識別（id属性値）
 * @param classNames
 *            フォーム部品識別（class属性値の配列）
 * @param value
 *            値(value属性値)
 * @param element
 *            要素データ
 */
gcparts.FormMaker2.prototype.addInputItem = function(name, label, id, classNames, value, element) {
	var _obj = {
		name : name,
		label : label,
		id : id,
		classNames : classNames,
		value : value,
		element : element
	};
	this._inputElements[this._inputElements.length] = _obj;
};

/**
 * フォーム情報を設定する
 *
 * @param form
 */
gcparts.FormMaker2.prototype.setForm = function(form) {
	this._formElement = form;
};

/**
 * hidden項目を追加する
 *
 * @param element
 *            要素データ
 */
gcparts.FormMaker2.prototype.addHiddenItem = function(element) {
	this._hiddenElements[this._hiddenElements.length] = element;
};

gcparts.FormMaker2.prototype._searchInputElement = function(name, label, id, className, value) {

	for ( var i = 0; i < this._inputElements.length; i++) {
		var input = this._inputElements[i];

		if (name && name == input['name']) {
			// name属性でマッチング
			if (!value) {
				return input['element'];
			} else if (value == input['value']) {
				return input['element'];
			}
		}

		if (label && label == input['label']) {
			// data-gc-label属性でマッチング
			if (!value) {
				return input['element'];
			} else if (value == input['value']) {
				return input['element'];
			}
		}

		if (id && id == input['id']) {
			// id属性でマッチング
			if (!value) {
				return input['element'];
			} else if (value == input['value']) {
				return input['element'];
			}
		}

		if (className) {
			for ( var j = 0; j < input['classNames'].length; j++) {
				var c = input['classNames'][j];
				if (className == c) {
					if (!value) {
						return input['element'];
					} else if (value == input['value']) {
						return input['element'];
					}
				}
			}
		}
	}
	return null;
};

gcparts.FormMaker2.prototype._traverseFooter = function(node, tree) {

	for ( var fc = node; fc != null; fc = fc.nextSibling) {
		var type = fc.nodeType;
		if (type == Node.ELEMENT_NODE) {
			var e = {
				'tagName' : fc.nodeName
			};
			var ar = fc.attributes;
			var attrs = new gcparts.Attributes(ar);

			if (fc.nodeName == 'A') {
				// アンカーの場合のみ、新しいウィンドウを開くように属性を強制的に設定
				attrs.set('target', '_blank');
			}
			e['attrs'] = attrs;
			tree.addElement(e);

			// 再帰的にツリーを読む
			this._traverseFooter(fc.firstChild, tree);

			tree.endElement();
		} else if (type == Node.TEXT_NODE) {
			tree.addText(fc.nodeValue);
		}
	}
};

gcparts.FormMaker2.prototype._getTree = function() {

	var _tree = new gcparts.ElementTree();

	_tree.addElement(this._formElement);

	for ( var i = 0; i < this._hiddenElements.length; i++) {
		_tree.addElement(this._hiddenElements[i]);
		_tree.endElement();
	}

	for ( var i = 0; i < this._layoutData.length; i++) {
		var layout = this._layoutData[i];

		// 見出しの出力 （例）<h3 class="title">お客様情報</h3>
		var _title = layout['title'];
		if (_title) {
			_tree.addElement(this._design.title.cloneWithDeepAttrs());
			_tree.addText(_title);
			_tree.endElement();
		}

		// １セクション開始
		_tree.addElement(this._design.frame);

		var items = layout['items'];
		for ( var j = 0; j < items.length; j++) {

			var item = items[j];
			var _header = item['header'];
			var _footer1 = item['footer1'];
			var _footer2 = item['footer2'];
			var _inputs = item['inputs'] || [];
			var _layout = item['layout'];

			// 項目見出し
			if (_header) {
				_tree.addElement(this._design.header.cloneWithDeepAttrs());
				_tree.addText(_header);
				_tree.endElement();
			}

			// 入力項目（フォーム部品）
			var _inputsLen = _inputs.length;
			for ( var k = 0; k < _inputsLen; k++) {

				var _name = _inputs[k]['name'];
				var _label = _inputs[k]['label'];
				var _id = _inputs[k]['id'];
				var _className = _inputs[k]['className'];
				var _value = _inputs[k]['value'];
				var _prev = _inputs[k]['prev'];
				var _next = _inputs[k]['next'];
				var _placeholder = _inputs[k]['placeholder'];
				var _attributes = _inputs[k]['attributes'] || {};

				var elem = this._searchInputElement(_name, _label, _id, _className, _value);
				if (!elem) {
					continue;
				}

				// 入力項目フィールド出力
				var _f = this._design.field.cloneWithDeepAttrs();
				if (_layout) {
					// クラス属性値(inline)のappend
					var c = _f.getAttribute('class');
					if (c) {
						c += ' ' + _layout;
					} else {
						c = _layout;
					}
					// さらに、何分割かを識別できるクラス属性値(inline2 or inline3 or inline4...)をappendする
					c += ' ' + _layout + _inputsLen;
					_f.setAttribute('class', c);
				}
				_tree.addElement(_f); // add _design.field
				_tree.addElement(this._design.label.cloneWithDeepAttrs()); // add label

				// 入力項目の前に表示する文言の出力
				if (_prev) {
					var _p = this._design.prev.cloneWithDeepAttrs();
					var c = _p.getAttribute('class');
					if (c) {
						c += ' p_' + _name;
					} else {
						c = 'p_' + _name;
					}
					_p.setAttribute('class', c);
					_tree.addElement(_p);
					_tree.addText(_prev);
					_tree.endElement();
				}

				// フォーム部品の出力
				var _i = this._design.input.cloneWithDeepAttrs();
				var c = _i.getAttribute('class');
				if (c) {
					c += ' i_' + _name;
				} else {
					c = 'i_' + _name;
				}
				var _prl = (_prev) ? _prev.length : 0;
				var _nxl = (_next) ? _next.length : 0;

				if (_prl > 0 || _nxl > 0) {
					var em = _prl + _nxl;
					c += ' calc' + em + 'em';
				}

				_i.setAttribute('class', c);
				_tree.addElement(_i);

				if (elem instanceof gcparts.ElementTree) {
					_tree.addTree(elem);
				} else if (elem instanceof gcparts.EventObject) {
					if (_attributes) {
						if (_attributes['type']) {
							elem.setAttribute('type', _attributes['type'].toLowerCase());
						}
						if (elem.getAttribute('type') == 'image' && _attributes['src']) {
							elem.setAttribute('src', _attributes['src']);
						} else {
							elem.removeAttribute('src');
						}
						if (_attributes['value']) {
							elem.setAttribute('value', _attributes['value']);
						}
						if (_attributes['placeholder']) {
							elem.setAttribute('placeholder', _attributes['placeholder']);
						} else if (_placeholder) {
							elem.setAttribute('placeholder', _placeholder);
						}
					} else if (_placeholder) {
						elem.setAttribute('placeholder', _placeholder);
					}
					_tree.addElement(elem);
					_tree.endElement();
				} else {
					if (_attributes) {
						if (_attributes['type']) {
							elem.attrs.set('type', _attributes['type']
									.toLowerCase());
						}
						if (elem.attrs.get('type') == 'image'
								&& _attributes['src']) {
							elem.attrs.set('src', _attributes['src']);
						} else {
							elem.attrs.remove('src');
						}
						if (_attributes['value']) {
							elem.attrs.set('value', _attributes['value']);
						}
						if (_attributes['placeholder']) {
							elem.attrs.set('placeholder', _attributes['placeholder']);
						} else if (_placeholder) {
							elem.attrs.set('placeholder', _placeholder);
						}
					} else if (_placeholder) {
						elem.attrs.set('placeholder', _placeholder);
					}
					_tree.addElement(elem);
					_tree.endElement();
				}

				_tree.endElement(); // close _design.input

				// 入力項目の後に表示する文言
				if (_next) {
					var _n = this._design.next.cloneWithDeepAttrs();
					var c = _n.getAttribute('class');
					if (c) {
						c += ' n_' + _name;
					} else {
						c = 'n_' + _name;
					}
					_n.setAttribute('class', c);
					_tree.addElement(_n);
					_tree.addText(_next);
					_tree.endElement();
				}
				_tree.endElement(); // close label element
				_tree.endElement(); // close _design.field
			}

			var rtag = /<("[^"]*"|\'[^\']*\'|[^\'">])*>/;
			// var rtag =
			// /<\w+(?:\s+\w+(?:\s*=\s*(?:".*?"|'.*?'|[^'">\s]+))?)?(?:\s?\/)?>|<\/\w+>/g;
			// フッター１とフッター２はタグの入力を想定する
			if (_footer1) {
				_tree.addElement(this._design.footer1.cloneWithDeepAttrs());
				if (_footer1.match(rtag)) {
					var _f1 = $('<span>' + _footer1 + '</span>').get(0);
					var _c1 = _f1.childNodes;
					this._traverseFooter(_c1[0], _tree);
				} else {
					_tree.addText(_footer1);
				}
				_tree.endElement();
			}
			if (_footer2) {
				_tree.addElement(this._design.footer2.cloneWithDeepAttrs());
				if (_footer2.match(rtag)) {
					var _f2 = $('<span>' + _footer2 + '</span>').get(0);
					var _c2 = _f2.childNodes;
					this._traverseFooter(_c2[0], _tree);
				} else {
					_tree.addText(_footer2);
				}
				_tree.endElement();
			}
		}
		_tree.endElement(); // close _design.frame
	}
	_tree.endElement(); // close _formElement

	return _tree;
};




/**
 * Bootstrap ListGroup項目を作成するヘルパー関数群
 *
 * @param listElement
 *            エレメント
 * @param anchorType
 *            使用するアンカー（0:先頭のアンカーを使う、それ以外：最後のアンカーを使う）
 *
 */
gcparts.BootstrapListGroupMaker = function BootstrapListGroupMaker(listElement, anchorType, anchorIcon) {

	this._listElement = listElement;
	this._anchorType = anchorType || 0;
	this._anchorElement = null;
	this._itemList = [];
	this._anchorIcon = anchorIcon;

	return this;
};

gcparts.BootstrapListGroupMaker.prototype.toString = function() {
	return 'gcparts.BootstrapListGroupMaker';
};

gcparts.BootstrapListGroupMaker.prototype.setAnchorElement = function(element) {

	if (!this._anchorElement) {
		this._anchorElement = element;
	} else if (this._anchorElement && this._anchorType != 0) {
		this._anchorElement = element;
	}

};

gcparts.BootstrapListGroupMaker.prototype.addItem = function(element, text) {
	var item = {
		'element' : element,
		'#text' : text
	};
	this._itemList[this._itemList.length] = item;
};

gcparts.BootstrapListGroupMaker.prototype.getItemCount = function() {
	return this._itemList.length;
};

gcparts.BootstrapListGroupMaker.prototype.closeItem = function() {
	this._itemList[this._itemList.length] = {
		'element' : 'CLOSE'
	};
};

gcparts.BootstrapListGroupMaker.prototype._getTree = function() {

	var _tree = new gcparts.ElementTree();
	var _itemLen = this._itemList.length;
	if (_itemLen == 0) {
		return _tree;
	}

	if (this._anchorElement) {
		// A要素の追加
// もっと見る機能は実装しない
//		if (this._listElement.hasAttribute('gc-hidden')) {
//			this._anchorElement.setAttribute('style', 'display: none;');
//		}
		_tree.addElement(this._anchorElement);
	} else {
// もっと見る機能は実装しない
//		if (this._listElement.hasAttribute('gc-hidden')) {
//			this._listElement.setAttribute('style', 'display: none;');
//		}
		_tree.addElement(this._listElement);
	}

	// リストコンテンツの中身
	for ( var i = 0; i < _itemLen; i++) {
		var _item = this._itemList[i];

		if (_item['element'] && _item['element'] == 'CLOSE') {
			_tree.endElement();
			continue;
		}
		if (_item['element']) {
			_tree.addElement(_item['element']);
		}
		if (_item['#text']) {
			_tree.addText(_item['#text']);
		}
	}

	if (this._anchorElement) {
		// A要素の閉じ
		if (this._anchorIcon) {
			_tree.addElement(this._anchorIcon.cloneWithDeepAttrs());
			_tree.endElement();
		}
		_tree.endElement();
	} else {
		_tree.endElement();
	}
	return _tree;

};

/**
 * 画像付きリスト項目を作成する(アンカーが各アイテム全てを内包)
 *
 * @param listElement
 *            リスト要素
 * @param imageElements
 *            画像要素
 * @param thumbnailSize
 *            サムネイル画像横幅・高さの閾値
 */
gcparts.BootstrapImageListMaker = function BootstrapImageListMaker(listElement,
		imageElements, thumbnailSize, contentElements, enableNoImage, anchorType) {

	this._listElement = listElement;

	this.thumbnailElement = imageElements.thumbnail;
	this.iconElement = imageElements.icon;
	this.wrapperElement = imageElements.wrapper;

	this.thumbnailWidth = thumbnailSize.width - 0;
	this.thumbnailHeight = thumbnailSize.height - 0;
	
	this.bodyElement = contentElements.body;
	this.contentElement = contentElements.content;
	

	// 代替画像ツリー
	this.noImageTree = new gcparts.ElementTree();
	this.noImageTree.addElement(this.thumbnailElement);
	this.noImageTree.endElement();

	this.enableNoImage = enableNoImage ? enableNoImage : false;

	this._anchorType = anchorType || 0;
	this._anchorElement = null;

	this._itemList = [];

	return this;
};

gcparts.BootstrapImageListMaker.prototype.toString = function() {
	return 'gcparts.BootstrapImageListMaker';
};

gcparts.BootstrapImageListMaker.prototype.setAnchorElement = function(element) {

	if (!this._anchorElement) {
		this._anchorElement = element;
	} else if (this._anchorElement && this._anchorType != 0) {
		this._anchorElement = element;
	}

};

gcparts.BootstrapImageListMaker.prototype.addItem = function(element, text) {
	var item = {
		'element' : element,
		'#text' : text
	};
	this._itemList[this._itemList.length] = item;
};

gcparts.BootstrapImageListMaker.prototype.closeItem = function() {
	this._itemList[this._itemList.length] = {
		'element' : 'CLOSE'
	};
},

gcparts.BootstrapImageListMaker.prototype.getItemCount = function() {
	return this._itemList.length;
};

/**
 * バッファに溜まっている要素の中からサムネイルにふさわしい画像を探す（次候補） 画像の横幅・高さが未指定の画像を探し、最初に見つかった画像を返す
 */
gcparts.BootstrapImageListMaker.prototype.searchThumbnailNextOption = function() {

	var image;
	for ( var i = 0; i < this._itemList.length; i++) {
		var _item = this._itemList[i];
		if (!_item['element']) {
			continue;
		}
		if (_item['element'] == 'CLOSE') {
			continue;
		}
		var _tname = _item['element'].tagName.toUpperCase();
		if (_tname != 'IMG') {
			continue;
		}
		var _attrs = _item['element'].attrs;
		var w = _attrs.getInt('width');
		var h = _attrs.getInt('height');
		if (w == -1 && h == -1) {
			return _item;
		}
	}
	// サムネイルにふさわしい画像は見つからなかった
	return null;
};
/**
 * バッファに溜まっている要素の中からサムネイルにふさわしい画像を探す 画像の横幅・高さが閾値を満たしている画像を探し、最初に見つかった画像を探す
 */
gcparts.BootstrapImageListMaker.prototype.searchThumbnailImage = function() {

	for ( var i = 0; i < this._itemList.length; i++) {
		var _item = this._itemList[i];
		if (!_item['element']) {
			continue;
		}
		if (_item['element'] == 'CLOSE') {
			continue;
		}
		var _tname = _item['element'].tagName.toUpperCase();
		if (_tname != 'IMG') {
			continue;
		}
		var _attrs = _item['element'].attrs;

		var gcp = _attrs.get('gcp');
		if (gcp == 'mark:thumbnail') {
			return _item;
		}

		/* v1.1.0 属性値が%で設定されていた場合は、幅・高さの指定なしとみなす */
		var w = _attrs.get('width');
		var h = _attrs.get('height');
		if (w && w.slice(-1) != '%') {
			w = _attrs.getInt('width');
		} else {
			w = -1;
		}
		if (h && h.slice(-1) != '%') {
			h = _attrs.getInt('height');
		} else {
			h = -1;
		}

		var thumbnail = false;
		if (w != -1 && h != -1) {
			/* ver1.1.0 オプションに負の値が与えられた場合、比較ができていなかったバグを修正 */
			if (this.thumbnailWidth <= 0 && this.thumbnailHeight <= 0) {
				// サイズ指定なし
				thumbnail = true;
			} else if (this.thumbnailWidth > 0 && this.thumbnailHeight > 0) {
				if (w >= this.thumbnailWidth && h >= this.thumbnailHeight) {
					thumbnail = true;
				}
			} else if (this.thumbnailWidth > 0 && w >= this.thumbnailWidth) {
				thumbnail = true;
			} else if (this.thumbnailHeight > 0 && h >= this.thumbnailHeight) {
				thumbnail = true;
			}
		} else if (w != -1) {
			if (this.thumbnailWidth <= 0) {
				thumbnail = true;
			} else if (w >= this.thumbnailWidth) {
				thumbnail = true;
			}
		} else if (h != -1) {
			if (this.thumbnailHeight <= 0) {
				thumbnail = true;
			} else if (h >= this.thumbnailHeight) {
				thumbnail = true;
			}
		}
		if (thumbnail) {
			return _item;
		}
	}
	// 画像サイズが閾値を満たしている画像が見つからなかったので、次点を探す
	return this.searchThumbnailNextOption();
};

gcparts.BootstrapImageListMaker.prototype._getTree = function() {

	var _tree = new gcparts.ElementTree();
	var _itemLen = this._itemList.length;
	if (_itemLen == 0) {
		return _tree;
	}

	_tree.addElement(this._listElement);

	if (this._anchorElement) {
		_tree.addElement(this._anchorElement);
	} else {
		_tree.addElement(this.wrapperElement.cloneWithDeepAttrs());
	}

	// バッファに溜まっている画像からサムネイル画像を探す
	var thumbnail = this.searchThumbnailImage();
	if (!thumbnail && this.enableNoImage) {
		// リストに１つも画像がないので、NOIMAGEを表示する
		_tree.addTree(this.noImageTree);
	} else {
		var image;
		if (thumbnail) {
			image = this.thumbnailElement.cloneWithDeepAttrs();
			
			var imgAttrs = thumbnail['element'].attrs;
			if (imgAttrs.has('src')) {
				image.setAttribute('src', imgAttrs.get('src'));
			}
			if (imgAttrs.has('alt')) {
				image.setAttribute('alt', imgAttrs.get('alt'));
			}
			if (imgAttrs.has('title')) {
				image.setAttribute('title', imgAttrs.get('title'));
			}
			_tree.addElement(image);
			_tree.endElement();
		}
	}
	
//	if (this._anchorElement) {
		_tree.endElement();
//	}
	
	_tree.addElement(this.bodyElement.cloneWithDeepAttrs());
	_tree.addElement(this.contentElement.cloneWithDeepAttrs());

	// リストコンテンツ
	for ( var i = 0; i < _itemLen; i++) {
		var _item = this._itemList[i];

		if (_item['element'] && _item['element'] == 'CLOSE') {
			_tree.endElement();
			continue;
		}
		if (_item['element']) {
			var tname = _item['element'].tagName.toUpperCase();
			if (tname == 'IMG') {
				var image;
				if (thumbnail != _item) {
					image = this.iconElement.cloneWithDeepAttrs();
					var imgAttrs = _item['element'].attrs;
					if (imgAttrs.has('src')) {
						image.setAttribute('src', imgAttrs.get('src'));
					}
					if (imgAttrs.has('alt')) {
						image.setAttribute('alt', imgAttrs.get('alt'));
					}
					if (imgAttrs.has('title')) {
						image.setAttribute('title', imgAttrs.get('title'));
					}
					_tree.addElement(image);
				}
			} else {
				_tree.addElement(_item['element']);
			}
		}
		if (_item['#text']) {
			_tree.addText(_item['#text']);
		}
	}
	// content
	_tree.endElement();
	// body
	_tree.endElement();

	_tree.endElement();
	return _tree;

};


}());
