/*1.0.0*/
/*
 * gcproc.js
 *
 * Copyright 2013- Symmetric Co., Ltd. all rights reserved.
 */
(function() {

	//--------------------------------------------------
    // preprocess functions
    //--------------------------------------------------

    /**
     * テキストリスト変換前処理。（TEXTLIST01）
     */
	gcproc.prepareTextlist01 = function prepareTextlist01(elem) {

		if (elem.size() == 0) {
			return elem;
		}

		var $div = $('<div></div>').append(elem);
		if ($div.find('ul,ol,li').size() > 0) {
			// 何もせずそのままリターン
			return elem;
		}

		if ($div.find('dt').size() > 0) {
			return elem;
		} else if ($div.find('dd').size() > 0) {
			$div.find('dd').each(function() {
				$(this).wrap('<li></li>');
			});
			return $div.children();
		}

		// リスト関連要素のないHTMLの場合（divやpの集まりなど）は、個数に応じてラップ単位を変える
		if (elem.size() == 1 && elem.get(0).tagName == 'TABLE') {
			// tbodyが必ず子要素に来ることを考慮し、trをfindしてラップする
			elem.find('tr').each(function(){
				$(this).wrap('<li></li>');
			});
			return elem;
		} else if (elem.size() == 1) {
			elem.children().each(function(){
				$(this).wrap('<li></li>');
			});
			return elem;
		} else {
			$div.children().each(function() {
				$(this).wrap('<li></li>');
			});
			return $div.children();
		}
	};


    /**
     * テキストメニュー変換前処理。（TEXTMENU01）
     */
	gcproc.prepareTextmenu01 = function prepareTextmenu01(elem) {

		if (elem.size() == 0) {
			return elem;
		}

		if (elem.size() == 1) {
			var tname = elem.get(0).tagName;
			if (tname == 'UL' || tname == 'OL') {
				// 選ばれているものがUL、OLリストなので前処理不要。そのままリターン
				return elem;
			}
			if (elem.children().size() == elem.children('ul,ol').size()) {
				// 中身がどれもUL、OLリストなので前処理不要。そのままリターン
				return elem;
			}
		} else {
			if (elem.size() == elem.filter('ul,ol').size()) {
				// 中身がどれもUL、OLリストなので前処理不要。そのままリターン
				return elem;
			}
		}

		var wrapRoot = false;

		if (elem.size() == 1) {
			elem.children().each(function(){
				if ($(this).get(0).tagName == 'LI') {
					// it means 'continue'
					return true;
				}
				$(this).wrap('<li></li>');
			});
			var $result = $('<ul></ul>').append(elem);

			return $result;

		} else {

			var $result = $('<ul></ul>').append(elem);
			$result.children().each(function(){
				if ($(this).get(0).tagName == 'LI') {
					// it means 'continue'
					return true;
				}
				$(this).wrap('<li></li>');
			});

			return $result;
		}

	};

    /**
     * 画像付きメニュー変換前処理。（IMAGEMENU01）
     */
	gcproc.prepareImagemenu01 = function prepareImagemenu01(elem) {

		if (elem.size() == 0) {
			return elem;
		}

		// 背景画像マッピング用の属性を設定する
		var count = 0;
		elem.each(function(){
			$(this).attr('gcidx', count);
			count++;
		});
		elem.find('*').each(function(){
			$(this).attr('gcidx', count);
			count++;
		});

		if (elem.size() == 1) {
			var tname = elem.get(0).tagName;
			if (tname == 'UL' || tname == 'OL') {
				// 選ばれているものがUL、OLリストなので前処理不要。そのままリターン
				return elem;
			}
			if (elem.children().size() == elem.children('ul,ol').size()) {
				// 中身がどれもUL、OLリストなので前処理不要。そのままリターン
				return elem;
			}
		} else {
			if (elem.size() == elem.filter('ul,ol').size()) {
				// 中身がどれもUL、OLリストなので前処理不要。そのままリターン
				return elem;
			}
		}

		var wrapRoot = false;

		if (elem.size() == 1) {
			elem.children().each(function() {
				if ($(this).get(0).tagName == 'LI') {
					// it means 'continue'
					return true;
				}
				$(this).wrap('<li></li>');
			});
			var $result = $('<ul></ul>').append(elem);
			return $result;

		} else {

			var $result = $('<ul></ul>').append(elem);
			$result.children().each(function() {
				if ($(this).get(0).tagName == 'LI') {
					// it means 'continue'
					return true;
				}
				$(this).wrap('<li></li>');
			});
			return $result;
		}
	};

    /**
     * 画像付きリスト変換前処理。（IMAGELIST01）
     */
	gcproc.prepareImagelist01 = function prepareImagelist01(elem) {

		if (elem.size() == 0) {
			return elem;
		}

		var $div = $('<div></div>').append(elem);
		if ($div.find('ul,ol,li').size() > 0) {
			// 何もせずそのままリターン
			return elem;
		}

		if ($div.find('dt').size() > 0) {
			return elem;
		} else if ($div.find('dd').size() > 0) {
			$div.find('dd').each(function() {
				$(this).wrap('<li></li>');
			});
			return $div.children();
		}

		// リスト関連要素のないHTMLの場合（divやpの集まりなど）は、個数に応じてラップ単位を変える
		if (elem.size() == 1 && elem.get(0).tagName == 'TABLE') {
			// tbodyが必ず子要素に来ることを考慮し、trをfindしてラップする
			elem.find('tr').each(function(){
				$(this).wrap('<li></li>');
			});
			return elem;
		} else if (elem.size() == 1) {
			elem.children().each(function(){
				$(this).wrap('<li></li>');
			});
			return elem;
		} else {
			$div.children().each(function() {
				$(this).wrap('<li></li>');
			});
			return $div.children();
		}
	};


    /**
     * リスト系パーツ用変換前処理。（標準の動作ではうまくでない、選んだ要素の中にLIやDTが潜んでいる場合などに使う）
     */
	gcproc.prepareNormalizeList = function prepareNormalizeList(elem) {
		if (elem.size() == 0) {
			return elem;
		}
		var $result;
		if (elem.size() === 1) {
			$result = elem;
		} else {
			$result = $('<ul></ul>').append(elem);
		}

		/*1.1.0 ネストしたliやdtがdivに置換できないバグの修正*/
		// ノーマライズ対象の要素を探し、その要素の末端からdiv要素に置換する
		$result.children().each(function() {
			var $target = $(this).find('ul,dl,li,dt');
			var size = $target.size();
			for(var i = 0; i < size; i++) {
				// 後の要素から取り出し置換する
				var idx = (size - (1 + i));
				$target.eq(idx).replaceWith(function(idx, html) {
					return $('<div>' + html + '</div>');
				});
			}
			if ($(this).get(0).tagName !== 'LI') {
				$(this).wrap('<li></li>');
			}
		});
		return $result;
	};

    /**
     * テキストパーツ用変換前処理
     */
	gcproc.prepareBasicText01 = function prepareBasicText01(elem) {
		var _size = elem.size();
		if (_size == 0) {
			return elem;
		}
		if (_size == elem.filter('li').size()) {
			var $result = $('<ul></ul>').append(elem);
			return $result;
		} else if (_size == elem.filter('dt,dd').size()) {
			var $result = $('<dl></dl>').append(elem);
			return $result;
		}
		return elem;
	};

    /**
     * 画像付きリスト変換前処理。（BSIMAGELIST01）
     */
	gcproc.prepareBsImagelist01 = function prepareBsImagelist01(elem) {

		if (elem.size() == 0) {
			return elem;
		}

		var $div = $('<div></div>').append(elem);
		if ($div.find('ul,ol,li').size() > 0) {
			// 何もせずそのままリターン
			return elem;
		}

		if ($div.find('dt').size() > 0) {
			return elem;
		} else if ($div.find('dd').size() > 0) {
			$div.find('dd').each(function() {
				$(this).wrap('<li></li>');
			});
			return $div.children();
		}

		// リスト関連要素のないHTMLの場合（divやpの集まりなど）は、個数に応じてラップ単位を変える
		if (elem.size() == 1 && elem.get(0).tagName == 'TABLE') {
			// tbodyが必ず子要素に来ることを考慮し、trをfindしてラップする
			elem.find('tr').each(function(){
				$(this).wrap('<li></li>');
			});
			return elem;
		} else if (elem.size() == 1) {
			elem.children().each(function(){
				$(this).wrap('<li></li>');
			});
			return elem;
		} else {
			$div.children().each(function() {
				$(this).wrap('<li></li>');
			});
			return $div.children();
		}
	};

	/**
	 * BootstrapPager変換前処理。（BSPAGER01）
	 */
	gcproc.prepareBsPager01 = function prepareBsPager01(elem) {
		var _size = elem.size();
		if (_size == 0) {
			return elem;
		}

		var $elem = elem;
		$elem.find('li:not(:has(a))').attr('class', 'active');
		$elem.find('li:has(a)').removeAttr('class', 'active');

		return $elem;
	};

	/**
	 * Bootstrapテキストパーツ用変換前処理
	 */
	gcproc.prepareBsText01 = function prepareBsText01(elem) {
		var _size = elem.size();
		if (_size == 0) {
			return elem;
		}
		if (_size == elem.filter('li').size()) {
			var $result = $('<ul></ul>').append(elem);
			return $result;
		} else if (_size == elem.filter('dt,dd').size()) {
			var $result = $('<dl></dl>').append(elem);
			return $result;
		}
		return elem;
	};

	gcproc.detectBsListPreproc = function detectBsListPreproc(elem) {
		var _size = elem.size();
		if (_size == 0) {
			return elem;
		}

		if (elem.attr("data-gc-label") == 'list_processed') {
			// 既に処理済み
			return elem;
		}


		if (elem.find("li,dl,dt").size() > 0) {
			return elem;
		}
		if (elem.filter("li,dl,dt").size() > 0) {
			return elem;
		}

		if (_size == 1) {
			return gcproc.prepareBsTextlist01(elem);
		} else {
			return gcproc.prepareBsNormalizeList(elem);
		}
	};
    /**
     * リスト系パーツ用変換前処理。（標準の動作ではうまくでない、選んだ要素の中にLIやDTが潜んでいる場合などに使う）
     */
	gcproc.prepareBsNormalizeList = function prepareBsNormalizeList(elem) {
		if (elem.size() == 0) {
			return elem;
		}
		var $result;
		if (elem.size() === 1) {
			$result = elem;
		} else {
			$result = $('<ul></ul>').append(elem);
		}
		$result.attr("data-gc-label", "list-processed");

		/*1.1.0 ネストしたliやdtがdivに置換できないバグの修正*/
		// ノーマライズ対象の要素を探し、その要素の末端からdiv要素に置換する
		$result.children().each(function() {
			var $target = $(this).find('ul,dl,li,dt');
			var size = $target.size();
			for(var i = 0; i < size; i++) {
				// 後の要素から取り出し置換する
				var idx = (size - (1 + i));
				$target.eq(idx).replaceWith(function(idx, html) {

					return $('<div>' + html + '</div>');
				});
			}
			if ($(this).get(0).tagName !== 'LI') {
				$(this).wrap('<li></li>');
			}
		});


		return $result;
	};

    /**
     * テキストリスト変換前処理。（TEXTLIST01）
     */
	gcproc.prepareBsTextlist01 = function prepareBsTextlist01(elem) {

		if (elem.size() == 0) {
			return elem;
		}
//		$result.attr("data-gc-label", "list-processed");

		var $div = $('<div></div>').append(elem);
		if ($div.find('ul,ol,li').size() > 0) {
			// 何もせずそのままリターン
			return elem;
		}

		if ($div.find('dt').size() > 0) {
			return elem;
		} else if ($div.find('dd').size() > 0) {
			$div.find('dd').each(function() {
				$(this).wrap('<li></li>');
			});
			return $div.children();
		}

		// リスト関連要素のないHTMLの場合（divやpの集まりなど）は、個数に応じてラップ単位を変える
		if (elem.size() == 1 && elem.get(0).tagName == 'TABLE') {
			// tbodyが必ず子要素に来ることを考慮し、trをfindしてラップする
			elem.find('tr').each(function(){
				$(this).wrap('<li></li>');
			});
			return elem;
		} else if (elem.size() == 1) {
			elem.children().each(function(){
				$(this).wrap('<li></li>');
			});
			return elem;
		} else {
			$div.children().each(function() {
				$(this).wrap('<li></li>');
			});
			return $div.children();
		}
	};
}());

