/**
 * postjs 1.0.0
 *
 * Copyright 2014- Symmetric Co., Ltd. All Rights Reserved.
 */
var genecode = function() {
	
	// ----- static変数 -----
	var postJsGlobalOption = {
		captureAjax: false
	};
	
	// postJsSettingオブジェクト（callback=ハンドラ, option=オプション設定）の配列。
	var postJsSettings = [];
	
	// キャプチャしたAJAX通信リスト。オブジェクトキーとして、methodとurlを含む。
	var postJsAllAjaxUrlList = [];
	
	// scriptexecutionイベントが発火したイベントIDのリスト
	var firedScriptExecutionEventIdList = [];
	
	var addEventListenerSupported = (document.addEventListener) ? true : false;
	
	// ----- 内部関数 -----
	var addEventListener = function(target, eventName, callback) {
		if (addEventListenerSupported) {
			target.addEventListener(eventName, callback, false);
		} else {
			target.attachEvent("on" + eventName, function() {
				callback.apply(target);
			});
		}
	};
	
	// array.filter()関数
	var filterArray = function(array, filter) {
		var newArray = [];
		var i;
		for (i = 0; i < array.length; i++) {
			var elem = array[i];
			if (filter(elem, i, array)) {
				newArray.push(elem);
			}
		}
		return newArray;
	};

	// array.forEach()関数
	var forEachArray = function(array, callback) {
		var i;
		for (i = 0; i < array.length; i++) {
			var elem = array[i];
			callback(elem, i, array);
		}
	};
	
	var getHeadElement = function() {
		if (document.head) {
			return document.head;
		} else {
			return document.getElementsByTagName("head")[0];
		}
	};

	var storeAjaxInfoToDocument = function() {
		jQuery(getHeadElement()).attr("data-gc-ajax-url", JSON.stringify(postJsAllAjaxUrlList));
	};
	
	var storeScriptExecutionEventToDocument = function() {
		jQuery(getHeadElement()).attr("data-gc-script-execution-event", JSON.stringify(firedScriptExecutionEventIdList));
	};

	var fireAjaxEvent = function(method, url) {
		postJsAllAjaxUrlList.push({
			method: method,
			url: url
		});
		storeAjaxInfoToDocument();
	};

	var outerHtml = function($obj) {
		var i;
		var html = "";
		
		for (i = 0; i < $obj.length; i++) {
			html += $obj.get(i).outerHTML;
		}
		
		return html;
	};
	
	/**
	 * booleanに変換する。
	 */
	var toBoolean = function(s, defaultValue) {
		if (s == null || s == undefined) {
			return defaultValue;
		}
		
		if (s == true || s == "true") {
			return true;
		} else if (s == false || s == "false") {
			return false;
		}
		
		return defaultValue;
	};
	
	/**
	 * 2つのノードリストを比較する。
	 *
	 * @return 0:完全一致、1:新しいノードあり、2:新しいノードはないが違いはあり
	 */
	var diffNewNodes = function(oldNodes, newNodes) {
		var i, j;
		var result = -1;
		
		if (oldNodes.length == newNodes.length) {
			// 完全一致しているケースが大半なのでまずは完全一致かどうかを調べる
			var fullMatch = true;
			for (i = 0; i < oldNodes.length; i++) {
				if (oldNodes[i] !== newNodes[i]) {
					fullMatch = false;
					break;
				}
			}
			if (fullMatch) {
				return 0;
			}
		}
		
		for (i = 0; i < newNodes.length; i++) {
			var newNode = newNodes[i];
			var found = false;
			for (j = 0; j < oldNodes.length; j++) {
				if (newNode === oldNodes[j]) {
					found = true;
					break;
				}
			}
			if (!found) {
				// 新しいノードあり
				return 1;
			}
		}
		
		return 2;
	};

	var executeOnNodeCreate = function($thisJquery, callback, option, repeat, context) {
		var i, j;
		var currentChildNodes = [];

		//二重実行を防止する
		if (context.lock) {
			return;
		}
		context.lock = true;
		
		for (i = 0; i < $thisJquery.length; i++) {
			var thisElem = $thisJquery.get(i);
			var childNodes = thisElem.childNodes;
			for (j = 0; j < childNodes.length; j++) {
				var childNode = childNodes[j];
				currentChildNodes.push(childNode);
			}
		}
		
		if (!context.prevChildNodes) {
			context.prevChildNodes = currentChildNodes;
			// 二重起動防止を解除
			context.lock = false;
			return;
		}
		
		var result = diffNewNodes(context.prevChildNodes, currentChildNodes);
		if (result == 0) {
			// 変更なし（完全一致）
			// 二重起動防止を解除
			context.lock = false;
			return;
		} else if (result == 2) {
			// 変更あり（新しいノードはない）
			context.prevChildNodes = currentChildNodes;
			// 二重起動防止を解除
			context.lock = false;
			return;
		}
		
		// コールバック実行
		executeHandler(callback, option, function() {
			// ハンドラ実行後に情報をリセット
			context.prevChildNodes = null;
			
			// 二重起動防止を解除
			context.lock = false;
			
			// 1回実行タイプはタイマーをSTOP
			if (!repeat) {
				clearInterval(context.intervalId);
			}
		});
		
		
	};
	
	var executeOnNodeChange = function($thisJquery, callback, option, repeat, context) {
		var currentHtml = outerHtml($thisJquery);
		
		//二重実行を防止する
		if (context.lock) {
			return;
		}
		context.lock = true;
		
		if (!context.prevHtml) {
			context.prevHtml = currentHtml;
			// 二重起動防止を解除
			context.lock = false;
			return;
		}
		if (context.prevHtml == currentHtml) {
			// 変更なし
			// 二重起動防止を解除
			context.lock = false;
			return;
		}
		
		// 変更を検出した
		// コールバック実行
		executeHandler(callback, option, function() {
			// ハンドラ実行後に情報をリセット
			context.prevHtml = null;

			// 二重起動防止を解除
			context.lock = false;
			
			if (!repeat) {
				// 1回実行タイプはここでSTOP
				// タイマーSTOP
				clearInterval(context.intervalId);
			}
		});

	};

	var isIgnoreHandlerByName = function(handlerName) {
		var ignore = postJsGlobalOption.ignore;
		if (ignore) {
			if (ignore == handlerName) {
				return true;
			}
		}
		return false;
	};
	
	var getPostProcessSettings = function(handlerName, excludeHandlerNames) {
		return filterArray(postJsSettings, function (postJsSetting, index) {
			// refがハンドラのnameと一致するものを探す
			if (postJsSetting.option.ref !== undefined && postJsSetting.option.ref == handlerName &&
				!isIgnoreHandlerByName(handlerName)) {
				return true;
			} else {
				return false;
			}
		});
	};
	
	var executePostProcess = function(option, excludeHandlerNames) {
		var handlerName = option.name;
		if (handlerName === undefined) {
			return;
		}
		
		var postProcessSettings = getPostProcessSettings(handlerName, excludeHandlerNames);
		forEachArray(postProcessSettings, function(postJsSetting) {
			executeHandler(postJsSetting.callback, postJsSetting.option, null, excludeHandlerNames);
		});
	};

	var executeHandler = function(callback, option, completeCallback, excludeHandlerNames) {
		var handlerName = option.name;
		if (handlerName && isIgnoreHandlerByName(handlerName)) {
			return;
		}
		if (excludeHandlerNames && excludeHandlerNames.indexOf(handlerName) >= 0) {
			// 一度実行したハンドラは実行しない
			return;
		}
		if (!excludeHandlerNames) {
			excludeHandlerNames = [handlerName];
		}
		var delay = parseInt(option.delay);
		if (!isNaN(delay)) {
			// nミリ秒後実行
			setTimeout(function() {
				callback.apply(this, [jQuery]);
				excludeHandlerNames.push(handlerName);
				executePostProcess(option, excludeHandlerNames);
				if (completeCallback) {
					completeCallback.apply();
				}
			}, delay);
		} else {
			// ディレイなし
			callback.apply(this, [jQuery]);
			excludeHandlerNames.push(handlerName);
			executePostProcess(option, excludeHandlerNames);
			if (completeCallback) {
				completeCallback.apply();
			}
		}
		
	};
	
	var dispatchScriptExecutionEvent = function(eventId) {
		forEachArray(postJsSettings, function (postJsSetting, index) {
			if (postJsSetting.option.event == "scriptexecution" && 
				postJsSetting.option.target !== undefined && postJsSetting.option.target == eventId) {
				executeHandler(postJsSetting.callback, postJsSetting.option);
			}
		});
	};

	return {
		// ----- 外部関数 -----
		/**
		 * postjsの初期化を行う。
		 *
		 * @param globalOption グローバルオプション
		 *  ignore: 実行を除外するpostJsハンドラの名前
		 *  captureAjax: AJAX通信一覧を取得するかどうか。true/falseで指定する。
		 *  preventAjax: AJAX通信を停止させるURL。スペース区切りもしくは配列で複数指定可。
		 */
		setupPostJs: function(globalOption) {
			if (globalOption) {
				jQuery.extend(postJsGlobalOption, globalOption);
				
				var captureAjax = toBoolean(globalOption.captureAjax, false);
				var preventAjaxUrl = globalOption.preventAjax;
				
				if (captureAjax || preventAjaxUrl) {
					// 配列に変換
					var preventAjaxUrlList = [];
					if (preventAjaxUrl) {
						if (Array.isArray(preventAjaxUrl)) {
							preventAjaxUrlList = preventAjaxUrl;
						} else {
							preventAjaxUrlList = preventAjaxUrl.split(" ");
						}
					}
					
					var originalJqueryAjax = jQuery.ajax;
					
					jQuery.extend({
						ajax: function (url, options) {
							if (typeof url === "object") {
								// 引数がオプションだけの場合
								options = url;
								url = undefined;
							}

							// メソッド
							var ajaxMethod = "GET";
							if (options.type) {
								ajaxMethod = options.type.toUpperCase();
							}
							
							// URL
							var ajaxUrl = url;
							if (!ajaxUrl) {
								ajaxUrl = options.url;
							}
					
							// AJAX一覧に追加
							if (captureAjax) {
								fireAjaxEvent(ajaxMethod, ajaxUrl);
							}
							
							// 除外URLの場合はAJAX実行を取りやめ
							if (preventAjaxUrlList.indexOf(ajaxUrl) >= 0) {
								// TODO:戻り値は？
								return;
							}
							
							if (!url) {
								return originalJqueryAjax.apply(this, [options]);
							} else {
								return originalJqueryAjax.apply(this, [url, options]);
							}
						}
					});
				}
			}
		},
		
		/**
		 *
		 * @param callback 第1引数にはjQueryオブジェクトが渡される。
		 * @param option
		 */
		postJs: function (callback, option) {
			var postJsSetting = {callback: callback, option: option};
			postJsSettings.push(postJsSetting);
			
			var startupType = option.type ? option.type : "now";
			var target = option.target;
			var repeat = toBoolean(option.repeat, true);
			option.repeat = repeat;
			var context = {
				lock: false,
				intervalId: null,
				prevChildNodes: null,
				prevHtml: null
			};
			
			if (startupType == "now") {
				// 即時実行
				executeHandler(callback, option);
				
			} else if (startupType == "docready") {
				// ページの読み込み完了時
				var delay = option.delay;
				jQuery(document).ready(function() {
					executeHandler(callback, option);
				});
				
			} else if (startupType == "ajax") {
				// ajax完了時に実行
				var doNothing = false;
				var useRegex = toBoolean(option.regex, false);
				var ajaxUrl = option.ajaxUrl;
				
				jQuery(document).ajaxComplete(function(event, xhr, settings) {
					if (doNothing) {
						return;
					}
					
					var match;
					if (useRegex) {
						match = settings.url.match(ajaxUrl);
					} else {
						match = (settings.url == ajaxUrl)
					}
					
					if (match) {
						executeHandler(callback, option);
						// 1回実行の場合
						if (!repeat) {
							doNothing = true;
						}
					}
				});

			} else if (startupType == "node") {
				// 特定要素直下への要素追加時
				var interval = 100;
				
				var intervalId = setInterval(function() {
					var $thisJquery = jQuery(target);
					executeOnNodeCreate($thisJquery, callback, option, repeat, context);
				}, interval);
				context.intervalId = intervalId;
				
			} else if (startupType == "nodechange") {
				// 特定要素内の子孫を含むノード変更時
				var interval = 100;
				
				var intervalId = setInterval(function() {
					var $thisJquery = jQuery(target);
					executeOnNodeChange($thisJquery, callback, option, repeat, context);
				}, interval);
				context.intervalId = intervalId;

			} else if (startupType == "event") {
				// イベント発生時
				// eventオプションおよびtargetオプションは必須
				if (!option.event) {
					return;
				}
				if (!target) {
					return;
				}
				
				if (option.event == "scriptexecution") {
					// スクリプト実行イベント
					addEventListener(document, "scriptevent", function(e) {
						if (e.detail.eventId == option.target) {
							dispatchScriptExecutionEvent(e.detail.eventId);
						}
					});
				} else if (target == "window") {
					// windowイベント
					var enabled = true;
					addEventListener(window, option.event, function(e) {
						if (!enabled) {
							return;
						}
						executeHandler(callback, option);
						// 繰り返しなしの場合はSTOP
						if (!repeat) {
							enabled = false;
						}
					});
				} else if (target == "document") {
					// documentイベント
					var enabled = true;
					addEventListener(document, option.event, function(e) {
						if (!enabled) {
							return;
						}
						executeHandler(callback, option);
						// 繰り返しなしの場合はSTOP
						if (!repeat) {
							enabled = false;
						}
					});
				} else {
					// 要素のイベント
					var enabled = true;
					jQuery(target).each(function() {
						addEventListener(this, option.event, function(e) {
							if (!enabled) {
								return;
							}
							executeHandler(callback, option);
							// 繰り返しなしの場合はSTOP
							if (!repeat) {
								enabled = false;
							}
						});
					});
				}
			} else if (startupType == "postprocess") {
				// 他のpostjsハンドラ実行後
				// ここでは特にやることはない
				
			}

		},
		
		/**
		 *
		 * @param eventId
		 */
		fireScriptExecutionEvent: function (eventId) {
			firedScriptExecutionEventIdList.push(eventId);
			storeScriptExecutionEventToDocument();
		}
		
		
	};

}();

	

